@@ -23,7 +23,7 @@ contents of the selected class in the icon-box. And so on....
 
 \since **ROOT version 6.24/00**
 
-TBrowser invokes by default the Web-based %ROOT file browser [RBrowser](ROOT::Experimental::RBrowser)
+TBrowser invokes by default the Web-based %ROOT file browser [RBrowser](ROOT::RBrowser)
 To change this behaviour, and invoke the standard TBrowser, one should put
 the following directive in the `.rootrc` file:
 ```
