@@ -236,6 +236,7 @@ TClingMethodInfo::TClingMethodInfo(cling::Interpreter *interp,
                                    TClingClassInfo *ci)
    : TClingDeclInfo(nullptr), fInterp(interp), fFirstTime(true), fTitle("")
 {
+   // Creating an interpreter transaction, needs locking.
    R__LOCKGUARD(gInterpreterMutex);
 
    if (!ci || !ci->IsValid()) {
@@ -285,6 +286,8 @@ TDictionary::DeclId_t TClingMethodInfo::GetDeclId() const
    if (!IsValid()) {
       return TDictionary::DeclId_t();
    }
+   // Next part interacts with clang, needs locking
+   R__LOCKGUARD(gInterpreterMutex);
    if (auto *FD = GetAsFunctionDecl())
       return (const clang::Decl*)(FD->getCanonicalDecl());
    return (const clang::Decl*)(GetAsUsingShadowDecl()->getCanonicalDecl());
@@ -302,6 +305,8 @@ const clang::UsingShadowDecl *TClingMethodInfo::GetAsUsingShadowDecl() const
 
 const clang::FunctionDecl *TClingMethodInfo::GetTargetFunctionDecl() const
 {
+   // May need to resolve the declaration interacting with clang, needs locking.
+   R__LOCKGUARD(gInterpreterMutex);
    const Decl *D = GetDecl();
    do {
       if (auto FD = dyn_cast<FunctionDecl>(D))
@@ -354,8 +359,9 @@ void TClingMethodInfo::Init(const clang::FunctionDecl *decl)
 void *TClingMethodInfo::InterfaceMethod() const
 {
    if (!IsValid()) {
-      return 0;
+      return nullptr;
    }
+   // TODO: can this lock be moved further deep?
    R__LOCKGUARD(gInterpreterMutex);
    TClingCallFunc cf(fInterp);
    cf.SetFunc(this);
@@ -372,6 +378,7 @@ int TClingMethodInfo::NArg() const
    if (!IsValid()) {
       return -1;
    }
+   // The next call locks the interpreter mutex.
    const clang::FunctionDecl *fd = GetTargetFunctionDecl();
    unsigned num_params = fd->getNumParams();
    // Truncate cast to fit cint interface.
@@ -383,6 +390,7 @@ int TClingMethodInfo::NDefaultArg() const
    if (!IsValid()) {
       return -1;
    }
+   // The next call locks the interpreter mutex.
    const clang::FunctionDecl *fd = GetTargetFunctionDecl();
    unsigned num_params = fd->getNumParams();
    unsigned min_args = fd->getMinRequiredArguments();
@@ -440,11 +448,14 @@ long TClingMethodInfo::Property() const
    if (llvm::isa<UsingShadowDecl>(declAccess))
       property |= kIsUsing;
 
+   // The next call locks the interpreter mutex.
    const clang::FunctionDecl *fd = GetTargetFunctionDecl();
    clang::AccessSpecifier Access = clang::AS_public;
    if (!declAccess->getDeclContext()->isNamespace())
       Access = declAccess->getAccess();
 
+   // From here on the method interacts with clang directly, needs locking.
+   R__LOCKGUARD(gInterpreterMutex);
    if ((property & kIsUsing) && llvm::isa<CXXConstructorDecl>(fd)) {
       Access = clang::AS_public;
       clang::CXXRecordDecl *typeCXXRD = llvm::cast<RecordType>(Type()->GetQualType())->getAsCXXRecordDecl();
@@ -470,10 +481,10 @@ long TClingMethodInfo::Property() const
          property |= kIsPublic;
          break;
       case clang::AS_protected:
-         property |= kIsProtected;
+         property |= kIsProtected | kIsNotReacheable;
          break;
       case clang::AS_private:
-         property |= kIsPrivate;
+         property |= kIsPrivate | kIsNotReacheable;
          break;
       case clang::AS_none:
          if (declAccess->getDeclContext()->isNamespace())
@@ -481,9 +492,15 @@ long TClingMethodInfo::Property() const
          break;
       default:
          // IMPOSSIBLE
+         assert(false && "Unexpected value for the access property value in Clang");
          break;
    }
 
+   if (!(property & kIsNotReacheable)) {
+      if (! ROOT::TMetaUtils::IsDeclReacheable(*fd))
+         property |= kIsNotReacheable;
+   }
+
    if (fd->isConstexpr())
       property |= kIsConstexpr;
    if (fd->getStorageClass() == clang::SC_Static) {
@@ -528,6 +545,7 @@ long TClingMethodInfo::ExtraProperty() const
       return 0L;
    }
    long property = 0;
+   // The next call locks the interpreter mutex.
    const clang::FunctionDecl *fd = GetTargetFunctionDecl();
    if (fd->isOverloadedOperator())
       property |= kIsOperator;
@@ -551,6 +569,9 @@ TClingTypeInfo *TClingMethodInfo::Type() const
       ti.Init(clang::QualType());
       return &ti;
    }
+
+   // The next part interacts with clang, thus needs locking.
+   R__LOCKGUARD(gInterpreterMutex);
    if (llvm::isa<clang::CXXConstructorDecl>(GetTargetFunctionDecl())) {
       // CINT claims that constructors return the class object.
       // For using-ctors of a base, claim that it "returns" the derived class.
@@ -578,6 +599,7 @@ std::string TClingMethodInfo::GetMangledName() const
    mangled_name.clear();
    const FunctionDecl* D = GetTargetFunctionDecl();
 
+   // Creating an interpreter transaction, needs locking.
    R__LOCKGUARD(gInterpreterMutex);
    cling::Interpreter::PushTransactionRAII RAII(fInterp);
    GlobalDecl GD;
@@ -595,16 +617,19 @@ std::string TClingMethodInfo::GetMangledName() const
 const char *TClingMethodInfo::GetPrototype()
 {
    if (!IsValid()) {
-      return 0;
+      return nullptr;
    }
    TTHREAD_TLS_DECL( std::string, buf );
    buf.clear();
    buf += Type()->Name();
    buf += ' ';
+   // The next call locks the interpreter mutex.
    const FunctionDecl *FD = GetTargetFunctionDecl();
    // Use the DeclContext of the decl, not of the target decl:
    // Used base functions should show as if they are part of the derived class,
    // e.g. `Derived Derived::Derived(int)`, not `Derived Base::Derived(int)`.
+   // Interacting with clang in the next part, needs locking
+   R__LOCKGUARD(gInterpreterMutex);
    if (const clang::TypeDecl *td = llvm::dyn_cast<clang::TypeDecl>(GetDecl()->getDeclContext())) {
       std::string name;
       clang::QualType qualType(td->getTypeForDecl(),0);
@@ -638,12 +663,18 @@ const char *TClingMethodInfo::GetPrototype()
 const char *TClingMethodInfo::Name() const
 {
    if (!IsValid()) {
-      return 0;
+      return nullptr;
    }
    if (!fNameCache.empty())
      return fNameCache.c_str();
 
-   ((TCling*)gCling)->GetFunctionName(GetDecl(), fNameCache);
+   {
+      // The data member needs to be filled. This calls into the interpreter,
+      // needs locking.
+      // TODO: Check if the lock can be moved further deep.
+      R__LOCKGUARD(gInterpreterMutex);
+      ((TCling *)gCling)->GetFunctionName(GetDecl(), fNameCache);
+   }
    return fNameCache.c_str();
 }
 
@@ -651,15 +682,18 @@ const char *TClingMethodInfo::TypeName() const
 {
    if (!IsValid()) {
       // FIXME: Cint does not check!
-      return 0;
+      return nullptr;
    }
+   // The next *two* calls lock the interpreter mutex. Lock here first instead
+   // of locking/unlocking twice.
+   R__LOCKGUARD(gInterpreterMutex);
    return Type()->Name();
 }
 
 const char *TClingMethodInfo::Title()
 {
    if (!IsValid()) {
-      return 0;
+      return nullptr;
    }
 
    //NOTE: We can't use it as a cache due to the "thoughtful" self iterator
@@ -672,6 +706,7 @@ const char *TClingMethodInfo::Title()
    // redecl chain (came from merging of pcms).
    const FunctionDecl *FD = GetTargetFunctionDecl();
 
+   // Creating an interpreter transaction, needs locking.
    R__LOCKGUARD(gInterpreterMutex);
 
    // Could trigger deserialization of decls.
