@@ -830,7 +830,7 @@ TGeoManager* TEveManager::GetGeometryByAlias(const TString& alias)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Get the default geometry.
-/// It should be registered via RegisterGeometryName("Default", <URL>).
+/// It should be registered via RegisterGeometryName("Default", `<URL>`).
 
 TGeoManager* TEveManager::GetDefaultGeometry()
 {
@@ -861,12 +861,7 @@ void TEveManager::SetStatusLine(const char* text)
 
 void TEveManager::ClearROOTClassSaved()
 {
-   TIter   nextcl(gROOT->GetListOfClasses());
-   TClass *cls;
-   while((cls = (TClass *)nextcl()))
-   {
-      cls->ResetBit(TClass::kClassSaved);
-   }
+   gROOT->ResetClassSaved();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
