@@ -21,8 +21,10 @@
 #include "TObjArray.h"
 #include "THttpServer.h"
 #include "TEnv.h"
+#include "TError.h"
 #include "TROOT.h"
 #include "TBase64.h"
+#include "TBufferJSON.h"
 
 #include <fstream>
 #include <iostream>
@@ -38,10 +40,10 @@
 #include <spawn.h>
 #endif
 
-using namespace ROOT::Experimental;
+using namespace ROOT;
 using namespace std::string_literals;
 
-/** \class ROOT::Experimental::RWebDisplayHandle
+/** \class ROOT::RWebDisplayHandle
 \ingroup webdisplay
 
 Handle of created web-based display
@@ -92,7 +94,6 @@ std::unique_ptr<RWebDisplayHandle::Creator> &RWebDisplayHandle::FindCreator(cons
 }
 
 namespace ROOT {
-namespace Experimental {
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
 /// Specialized handle to hold information about running browser process
@@ -120,7 +121,7 @@ public:
    {
    }
 
-   virtual ~RWebBrowserHandle()
+   ~RWebBrowserHandle() override
    {
 #ifdef _MSC_VER
       if (fHasPid)
@@ -137,7 +138,6 @@ public:
 
 };
 
-} // namespace Experimental
 } // namespace ROOT
 
 //////////////////////////////////////////////////////////////////////////////////////////////////
@@ -382,6 +382,9 @@ RWebDisplayHandle::ChromeCreator::ChromeCreator(bool _edge) : BrowserCreator(tru
 
    TestProg(gEnv->GetValue(fEnvPrefix.c_str(), ""));
 
+   if (!fProg.empty() && !fEdge)
+      fChromeVersion = gEnv->GetValue("WebGui.ChromeVersion", -1);
+
 #ifdef _MSC_VER
    if (fEdge)
       TestProg("\\Microsoft\\Edge\\Application\\msedge.exe", true);
@@ -404,8 +407,20 @@ RWebDisplayHandle::ChromeCreator::ChromeCreator(bool _edge) : BrowserCreator(tru
    fHeadlessExec = gEnv->GetValue((fEnvPrefix + "Headless").c_str(), "$prog --headless --disable-gpu $geometry \"$url\" --dump-dom &");
    fExec = gEnv->GetValue((fEnvPrefix + "Interactive").c_str(), "$prog $geometry --new-window --app=$url &"); // & in windows mean usage of spawn
 #else
-   fBatchExec = gEnv->GetValue((fEnvPrefix + "Batch").c_str(), "$prog --headless --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry $url");
-   fHeadlessExec = gEnv->GetValue((fEnvPrefix + "Headless").c_str(), "$prog --headless --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry \'$url\' --dump-dom >/dev/null &");
+#ifdef R__MACOSX
+   bool use_normal = true; // mac does not like new flag
+#else
+   bool use_normal = fChromeVersion < 119;
+#endif
+   if (use_normal) {
+      // old browser with standard headless mode
+      fBatchExec = gEnv->GetValue((fEnvPrefix + "Batch").c_str(), "$prog --headless --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry $url");
+      fHeadlessExec = gEnv->GetValue((fEnvPrefix + "Headless").c_str(), "$prog --headless --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry \'$url\' --dump-dom >/dev/null &");
+   } else {
+      // newer version with headless=new mode
+      fBatchExec = gEnv->GetValue((fEnvPrefix + "Batch").c_str(), "$prog --headless=new --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry $url");
+      fHeadlessExec = gEnv->GetValue((fEnvPrefix + "Headless").c_str(), "$prog --headless=new --no-sandbox --no-zygote --disable-extensions --disable-gpu --disable-audio-output $geometry \'$url\' &");
+   }
    fExec = gEnv->GetValue((fEnvPrefix + "Interactive").c_str(), "$prog $geometry --new-window --app=\'$url\' &");
 #endif
 }
@@ -652,26 +667,120 @@ bool RWebDisplayHandle::DisplayUrl(const std::string &url)
    return !!handle;
 }
 
+///////////////////////////////////////////////////////////////////////////////////////////////////
+/// Checks if configured browser can be used for image production
+
+bool RWebDisplayHandle::CheckIfCanProduceImages(RWebDisplayArgs &args)
+{
+   if ((args.GetBrowserKind() != RWebDisplayArgs::kFirefox) && (args.GetBrowserKind() != RWebDisplayArgs::kEdge) &&
+       (args.GetBrowserKind() != RWebDisplayArgs::kChrome) && (args.GetBrowserKind() != RWebDisplayArgs::kCEF) &&
+       (args.GetBrowserKind() != RWebDisplayArgs::kQt5) && (args.GetBrowserKind() != RWebDisplayArgs::kQt6)) {
+      bool detected = false;
+
+      auto &h1 = FindCreator("chrome", "ChromeCreator");
+      if (h1 && h1->IsActive()) {
+         args.SetBrowserKind(RWebDisplayArgs::kChrome);
+         detected = true;
+      }
+
+      if (!detected) {
+         auto &h2 = FindCreator("firefox", "FirefoxCreator");
+         if (h2 && h2->IsActive()) {
+            args.SetBrowserKind(RWebDisplayArgs::kFirefox);
+            detected = true;
+         }
+      }
+
+#ifdef _MSC_VER
+      if (!detected) {
+         auto &h3 = FindCreator("edge", "ChromeCreator");
+         if (h3 && h3->IsActive()) {
+            args.SetBrowserKind(RWebDisplayArgs::kEdge);
+            detected = true;
+         }
+      }
+#endif
+      return detected;
+   }
+
+   if (args.GetBrowserKind() == RWebDisplayArgs::kChrome) {
+      auto &h1 = FindCreator("chrome", "ChromeCreator");
+      return h1 && h1->IsActive();
+   }
+
+   if (args.GetBrowserKind() == RWebDisplayArgs::kFirefox) {
+      auto &h2 = FindCreator("firefox", "FirefoxCreator");
+      return h2 && h2->IsActive();
+   }
+
+#ifdef _MSC_VER
+   if (args.GetBrowserKind() == RWebDisplayArgs::kEdge) {
+      auto &h3 = FindCreator("edge", "ChromeCreator");
+      return h3 && h3->IsActive();
+   }
+#endif
+
+   return true;
+}
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+/// Returns true if image production for specified browser kind is supported
+/// If browser not specified - use currently configured browser or try to test existing web browsers
+
+bool RWebDisplayHandle::CanProduceImages(const std::string &browser)
+{
+   RWebDisplayArgs args(browser);
+
+   return CheckIfCanProduceImages(args);
+}
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 /// Produce image file using JSON data as source
 /// Invokes JSROOT drawing functionality in headless browser - Google Chrome or Mozilla Firefox
-
 bool RWebDisplayHandle::ProduceImage(const std::string &fname, const std::string &json, int width, int height, const char *batch_file)
 {
-   if (json.empty())
+   return ProduceImages(fname, {json}, {width}, {height}, batch_file);
+}
+
+///////////////////////////////////////////////////////////////////////////////////////////////////
+/// Produce image file(s) using JSON data as source
+/// Invokes JSROOT drawing functionality in headless browser - Google Chrome or Mozilla Firefox
+bool RWebDisplayHandle::ProduceImages(const std::string &fname, const std::vector<std::string> &jsons, const std::vector<int> &widths, const std::vector<int> &heights, const char *batch_file)
+{
+   if (jsons.empty())
       return false;
 
    std::string _fname = fname;
    std::transform(_fname.begin(), _fname.end(), _fname.begin(), ::tolower);
-
-   auto EndsWith = [_fname](const std::string &suffix) {
-      return (_fname.length() > suffix.length()) ? (0 == _fname.compare (_fname.length() - suffix.length(), suffix.length(), suffix)) : false;
+   auto EndsWith = [&_fname](const std::string &suffix) {
+      return (_fname.length() > suffix.length()) ? (0 == _fname.compare(_fname.length() - suffix.length(), suffix.length(), suffix)) : false;
    };
 
+   std::vector<std::string> fnames;
+
+   if (!EndsWith(".pdf")) {
+      bool has_quialifier = _fname.find("%") != std::string::npos;
+
+      if (!has_quialifier && (jsons.size() > 1)) {
+         _fname.insert(_fname.rfind("."), "%d");
+         has_quialifier = true;
+      }
+
+      for (unsigned n = 0; n < jsons.size(); n++) {
+         if (has_quialifier) {
+            auto expand_name = TString::Format(_fname.c_str(), (int) n);
+            fnames.emplace_back(expand_name.Data());
+         } else {
+            fnames.emplace_back(_fname);
+         }
+      }
+   }
+
    if (EndsWith(".json")) {
-      std::ofstream ofs(fname);
-      ofs << json;
+      for (unsigned n = 0; n < jsons.size(); ++n) {
+         std::ofstream ofs(fnames[n]);
+         ofs << jsons[n];
+      }
       return true;
    }
 
@@ -686,17 +795,21 @@ bool RWebDisplayHandle::ProduceImage(const std::string &fname, const std::string
       jsrootsys = jsrootsysdflt.Data();
    }
 
-   RWebDisplayArgs args; // set default browser kind, only Chrome or Firefox or CEF or Qt5 can be used here
-   if ((args.GetBrowserKind() != RWebDisplayArgs::kFirefox ) && (args.GetBrowserKind() != RWebDisplayArgs::kCEF) &&
-       (args.GetBrowserKind() != RWebDisplayArgs::kQt5) && (args.GetBrowserKind() != RWebDisplayArgs::kQt6))
-      args.SetBrowserKind(RWebDisplayArgs::kChrome);
+   RWebDisplayArgs args; // set default browser kind, only Chrome/Firefox/Edge or CEF/Qt5/Qt6 can be used here
+   if (!CheckIfCanProduceImages(args)) {
+      R__LOG_ERROR(WebGUILog()) << "Fail to detect supported browsers for image production";
+      return false;
+   }
+
+   auto isChromeBased = (args.GetBrowserKind() == RWebDisplayArgs::kChrome) || (args.GetBrowserKind() == RWebDisplayArgs::kEdge),
+        isFirefox = args.GetBrowserKind() == RWebDisplayArgs::kFirefox;
 
    std::string draw_kind;
 
    if (EndsWith(".pdf"))
       draw_kind = "draw"; // not a JSROOT drawing but Chrome capability to create PDF out of HTML page is used
-   else if (EndsWith("shot.png"))
-      draw_kind = (args.GetBrowserKind() == RWebDisplayArgs::kChrome) ? "draw" : "png";
+   else if (EndsWith("shot.png") && (jsons.size() == 1))
+      draw_kind = isChromeBased ? "draw" : "png"; // using screenshot
    else if (EndsWith(".svg"))
       draw_kind = "svg";
    else if (EndsWith(".png"))
@@ -723,25 +836,45 @@ bool RWebDisplayHandle::ProduceImage(const std::string &fname, const std::string
       return false;
    }
 
-   filecont = std::regex_replace(filecont, std::regex("\\$draw_width"), std::to_string(width));
-   filecont = std::regex_replace(filecont, std::regex("\\$draw_height"), std::to_string(height));
+   int max_width = 0, max_height = 0, page_margin = 10;
+   for (auto &w : widths)
+      if (w > max_width)
+         max_width = w;
+   for (auto &h : heights)
+      if (h > max_height)
+         max_height = h;
+
+   auto jsonw = TBufferJSON::ToJSON(&widths, TBufferJSON::kNoSpaces);
+   auto jsonh = TBufferJSON::ToJSON(&heights, TBufferJSON::kNoSpaces);
+
+   std::string mains;
+   for (auto &json : jsons) {
+      mains.append(mains.empty() ? "[" : ", ");
+      mains.append(json);
+   }
+   mains.append("]");
 
    if (strstr(jsrootsys,"http://") || strstr(jsrootsys,"https://") || strstr(jsrootsys,"file://"))
       filecont = std::regex_replace(filecont, std::regex("\\$jsrootsys"), jsrootsys);
    else
       filecont = std::regex_replace(filecont, std::regex("\\$jsrootsys"), "file://"s + jsrootsys);
 
-   filecont = std::regex_replace(filecont, std::regex("\\$draw_kind"), draw_kind);
+   filecont = std::regex_replace(filecont, std::regex("\\$page_margin"), std::to_string(page_margin) + "px");
+   filecont = std::regex_replace(filecont, std::regex("\\$page_width"), std::to_string(max_width + 2*page_margin) + "px");
+   filecont = std::regex_replace(filecont, std::regex("\\$page_height"), std::to_string(max_height + 2*page_margin) + "px");
 
-   filecont = std::regex_replace(filecont, std::regex("\\$draw_object"), json);
+   filecont = std::regex_replace(filecont, std::regex("\\$draw_kind"), draw_kind);
+   filecont = std::regex_replace(filecont, std::regex("\\$draw_widths"), jsonw.Data());
+   filecont = std::regex_replace(filecont, std::regex("\\$draw_heights"), jsonh.Data());
+   filecont = std::regex_replace(filecont, std::regex("\\$draw_objects"), mains);
 
    TString dump_name;
    if (draw_kind == "draw") {
-      if (args.GetBrowserKind() != RWebDisplayArgs::kChrome) {
-         R__LOG_ERROR(WebGUILog()) << "Creation of PDF files supported only by Chrome browser";
+      if (!isChromeBased) {
+         R__LOG_ERROR(WebGUILog()) << "Creation of PDF files supported only by Chrome-based browser";
          return false;
       }
-   } else if ((args.GetBrowserKind() == RWebDisplayArgs::kChrome) || (args.GetBrowserKind() == RWebDisplayArgs::kFirefox)) {
+   } else if (isChromeBased || isFirefox) {
       dump_name = "canvasdump";
       FILE *df = gSystem->TempFileName(dump_name);
       if (!df) {
@@ -767,7 +900,7 @@ try_again:
       tmp_name.Clear();
       html_name.Clear();
 
-      R__LOG_DEBUG(0, WebGUILog()) << "Using file content_len " << filecont.length() << " to produce batch image " << fname;
+      R__LOG_DEBUG(0, WebGUILog()) << "Using file content_len " << filecont.length() << " to produce batch images " << fname;
 
    } else {
       tmp_name = "canvasbody";
@@ -805,34 +938,37 @@ try_again:
       args.SetUrl("file://"s + gSystem->UnixPathName(html_name.Data()));
       args.SetPageContent(""s);
 
-      R__LOG_DEBUG(0, WebGUILog()) << "Using " << html_name << " content_len " << filecont.length() << " to produce batch image " << fname;
+      R__LOG_DEBUG(0, WebGUILog()) << "Using " << html_name << " content_len " << filecont.length() << " to produce batch images " << fname;
    }
 
-   TString tgtfilename = fname.c_str();
-   if (!gSystem->IsAbsoluteFileName(tgtfilename.Data()))
-      gSystem->PrependPathName(gSystem->WorkingDirectory(), tgtfilename);
-
    TString wait_file_name;
 
    args.SetStandalone(true);
    args.SetHeadless(true);
    args.SetBatchMode(true);
-   args.SetSize(width, height);
+   args.SetSize(widths[0], heights[0]);
 
    if (draw_kind == "draw") {
 
+      TString tgtfilename = fname.c_str();
+      if (!gSystem->IsAbsoluteFileName(tgtfilename.Data()))
+         gSystem->PrependPathName(gSystem->WorkingDirectory(), tgtfilename);
+
       wait_file_name = tgtfilename;
 
       if (EndsWith(".pdf"))
-         args.SetExtraArgs("--print-to-pdf="s + gSystem->UnixPathName(tgtfilename.Data()));
+         args.SetExtraArgs("--print-to-pdf-no-header --print-to-pdf="s + gSystem->UnixPathName(tgtfilename.Data()));
       else
          args.SetExtraArgs("--screenshot="s + gSystem->UnixPathName(tgtfilename.Data()));
 
-   } else if (args.GetBrowserKind() == RWebDisplayArgs::kFirefox) {
+      // remove target image file - we use it as detection when chrome is ready
+      gSystem->Unlink(tgtfilename.Data());
+
+   } else if (isFirefox) {
       // firefox will use window.dump to output produced result
       args.SetRedirectOutput(dump_name.Data());
       gSystem->Unlink(dump_name.Data());
-   } else if (args.GetBrowserKind() == RWebDisplayArgs::kChrome) {
+   } else if (isChromeBased) {
       // require temporary output file
       args.SetExtraArgs("--dump-dom");
       args.SetRedirectOutput(dump_name.Data());
@@ -842,9 +978,6 @@ try_again:
       gSystem->Unlink(dump_name.Data());
    }
 
-   // remove target image file - we use it as detection when chrome is ready
-   gSystem->Unlink(tgtfilename.Data());
-
    auto handle = RWebDisplayHandle::Display(args);
 
    if (!handle) {
@@ -865,7 +998,7 @@ try_again:
 
       auto dumpcont = handle->GetContent();
 
-      if ((dumpcont.length() > 20) && (dumpcont.length() < 60) && !chrome_tmp_workaround && (args.GetBrowserKind() == RWebDisplayArgs::kChrome)) {
+      if ((dumpcont.length() > 20) && (dumpcont.length() < 60) && !chrome_tmp_workaround && isChromeBased) {
          // chrome creates dummy html file with mostly no content
          // problem running chrome from /tmp directory, lets try work from home directory
          chrome_tmp_workaround = true;
@@ -878,38 +1011,57 @@ try_again:
       }
 
       if (draw_kind == "svg") {
-         auto p1 = dumpcont.find("<svg");
-         auto p2 = dumpcont.rfind("</svg>");
 
-         std::ofstream ofs(tgtfilename);
-         if ((p1 != std::string::npos) && (p2 != std::string::npos) && (p1 < p2)) {
-            ofs << dumpcont.substr(p1,p2-p1+6);
-         } else {
-            R__LOG_ERROR(WebGUILog()) << "Fail to extract SVG from HTML dump " << dump_name;
-            ofs << "Failure!!!\n" << dumpcont;
-            return false;
+         std::string::size_type p = 0;
+
+         for (auto & fn : fnames) {
+            auto p1 = dumpcont.find("<svg", p);
+            auto p2 = dumpcont.find("</svg></div>", p1 + 4);
+            p = p2 + 6;
+            std::ofstream ofs(fn);
+            if ((p1 != std::string::npos) && (p2 != std::string::npos) && (p1 < p2)) {
+               ofs << dumpcont.substr(p1, p2-p1+6);
+               ::Info("ProduceImage", "SVG file %s size %d bytes has been created", fn.c_str(), (int) (p2-p1+6));
+            } else {
+               R__LOG_ERROR(WebGUILog()) << "Fail to extract SVG from HTML dump " << dump_name;
+               ofs << "Failure!!!\n" << dumpcont;
+               return false;
+            }
          }
       } else {
 
-         auto p1 = dumpcont.rfind(";base64,");
-         auto p2 = dumpcont.rfind("></div>");
+         std::string::size_type p = 0;
 
-         if ((p1 != std::string::npos) && (p2 != std::string::npos) && (p1 < p2)) {
+         for (auto &fn : fnames) {
 
-            auto base64 = dumpcont.substr(p1+8, p2-p1-9);
-            auto binary = TBase64::Decode(base64.c_str());
+            auto p1 = dumpcont.find(";base64,", p);
+            auto p2 = dumpcont.find("></div>", p1 + 4);
+            p = p2 + 5;
 
-            std::ofstream ofs(tgtfilename, std::ios::binary);
-            ofs.write(binary.Data(), binary.Length());
-         } else {
-            R__LOG_ERROR(WebGUILog()) << "Fail to extract image from dump HTML code " << dump_name;
+            if ((p1 != std::string::npos) && (p2 != std::string::npos) && (p1 < p2)) {
 
-            return false;
+               auto base64 = dumpcont.substr(p1+8, p2-p1-9);
+               auto binary = TBase64::Decode(base64.c_str());
+
+               std::ofstream ofs(fn, std::ios::binary);
+               ofs.write(binary.Data(), binary.Length());
+
+               ::Info("ProduceImage", "Image file %s size %d bytes has been created", fn.c_str(), (int) binary.Length());
+            } else {
+               R__LOG_ERROR(WebGUILog()) << "Fail to extract image from dump HTML code " << dump_name;
+
+               return false;
+            }
          }
       }
+   } else if (EndsWith(".pdf")) {
+      ::Info("ProduceImage", "PDF file %s with %d pages has been created", fname.c_str(), (int) jsons.size());
    }
 
-   R__LOG_DEBUG(0, WebGUILog()) << "Create file " << fname;
+   if (fnames.size() == 1)
+      R__LOG_DEBUG(0, WebGUILog()) << "Create file " << fnames[0];
+   else
+      R__LOG_DEBUG(0, WebGUILog()) << "Create files " << TBufferJSON::ToJSON(&fnames, TBufferJSON::kNoSpaces);
 
    return true;
 }
