@@ -62,11 +62,109 @@ fIntegral(0), fIntegralStatus(kNoInt)
    fAxes.SetOwner();
 }
 
+THnBase::THnBase(const char *name, const char *title, Int_t dim, const Int_t *nbins,
+                 const std::vector<std::vector<double>> &xbins)
+   : TNamed(name, title), fNdimensions(dim), fAxes(dim), fBrowsables(dim), fEntries(0), fTsumw(0), fTsumw2(-1.),
+     fTsumwx(dim), fTsumwx2(dim), fIntegral(0), fIntegralStatus(kNoInt)
+{
+   if (Int_t(xbins.size()) != fNdimensions) {
+      Error("THnBase", "Mismatched number of dimensions %d with number of bin edge vectors %zu", fNdimensions,
+            xbins.size());
+   }
+   for (Int_t i = 0; i < fNdimensions; ++i) {
+      if (Int_t(xbins[i].size()) != (nbins[i] + 1)) {
+         Error("THnBase", "Mismatched number of bins %d with number of bin edges %zu", nbins[i], xbins[i].size());
+      }
+      TAxis *axis = new TAxis(nbins[i], xbins[i].data());
+      axis->SetName(TString::Format("axis%d", i));
+      fAxes.AddAtAndExpand(axis, i);
+   }
+   SetTitle(title);
+   fAxes.SetOwner();
+}
+
+THnBase::THnBase(const THnBase &other)
+   : TNamed(other), fNdimensions(other.fNdimensions), fAxes(fNdimensions), fBrowsables(fNdimensions),
+     fEntries(other.fEntries), fTsumw(other.fTsumw), fTsumw2(other.fTsumw2), fTsumwx(other.fTsumwx),
+     fTsumwx2(other.fTsumwx2), fIntegral(other.fIntegral), fIntegralStatus(other.fIntegralStatus)
+{
+
+   for (Int_t i = 0; i < fNdimensions; ++i) {
+      TAxis *axis = new TAxis(*static_cast<TAxis *>(other.fAxes[i]));
+      fAxes.AddAtAndExpand(axis, i);
+   }
+   fAxes.SetOwner();
+}
+
+THnBase &THnBase::operator=(const THnBase &other)
+{
+
+   if (this == &other)
+      return *this;
+
+   TNamed::operator=(other);
+   fNdimensions = other.fNdimensions;
+   fAxes = TObjArray(fNdimensions);
+   fBrowsables = TObjArray(fNdimensions);
+   fEntries = other.fEntries;
+   fTsumw = other.fTsumw;
+   fTsumw2 = other.fTsumw2;
+   fTsumwx = other.fTsumwx;
+   fTsumwx2 = other.fTsumwx2;
+   fIntegral = other.fIntegral;
+   fIntegralStatus = other.fIntegralStatus;
+
+   for (Int_t i = 0; i < fNdimensions; ++i) {
+      TAxis *axis = new TAxis(*static_cast<TAxis *>(other.fAxes[i]));
+      fAxes.AddAtAndExpand(axis, i);
+   }
+   fAxes.SetOwner();
+
+   return *this;
+}
+
+THnBase::THnBase(THnBase &&other)
+   : TNamed(other), fNdimensions(other.fNdimensions), fAxes(other.fAxes), fBrowsables(fNdimensions),
+     fEntries(other.fEntries), fTsumw(other.fTsumw), fTsumw2(other.fTsumw2), fTsumwx(other.fTsumwx),
+     fTsumwx2(other.fTsumwx2), fIntegral(std::move(other.fIntegral)), fIntegralStatus(other.fIntegralStatus)
+{
+
+   other.fAxes.SetOwner(false);
+   other.fAxes.Clear();
+   fAxes.SetOwner();
+}
+
+THnBase &THnBase::operator=(THnBase &&other)
+{
+
+   if (this == &other)
+      return *this;
+
+   TNamed::operator=(other);
+   fNdimensions = other.fNdimensions;
+   fAxes = other.fAxes;
+   fBrowsables = TObjArray(fNdimensions);
+   fEntries = other.fEntries;
+   fTsumw = other.fTsumw;
+   fTsumw2 = other.fTsumw2;
+   fTsumwx = other.fTsumwx;
+   fTsumwx2 = other.fTsumwx2;
+   fIntegral = std::move(other.fIntegral);
+   fIntegralStatus = other.fIntegralStatus;
+
+   other.fAxes.SetOwner(false);
+   other.fAxes.Clear();
+   fAxes.SetOwner();
+
+   return *this;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 /// Destruct a THnBase
 
 THnBase::~THnBase() {
-   if (fIntegralStatus != kNoInt) delete [] fIntegral;
+   if (fIntegralStatus != kNoInt)
+      fIntegral.clear();
 }
 
 
@@ -99,7 +197,7 @@ void THnBase::Init(const char* name, const char* title,
    SetNameTitle(name, title);
 
    TIter iAxis(axes);
-   const TAxis* axis = 0;
+   const TAxis* axis = nullptr;
    Int_t pos = 0;
    Int_t *nbins = new Int_t[axes->GetEntriesFast()];
    while ((axis = (TAxis*)iAxis())) {
@@ -125,12 +223,14 @@ void THnBase::Init(const char* name, const char* title,
       }
 
       nbins[pos] = reqaxis->GetNbins();
-      fAxes.AddAtAndExpand(new TAxis(*reqaxis), pos++);
+      fAxes.AddAtAndExpand(reqaxis, pos++);
    }
    fAxes.SetOwner();
 
    fNdimensions = axes->GetEntriesFast();
    InitStorage(nbins, chunkSize);
+   fTsumwx.Set(fNdimensions);
+   fTsumwx2.Set(fNdimensions);
    delete [] nbins;
 }
 
@@ -145,7 +245,7 @@ TH1* THnBase::CreateHist(const char* name, const char* title,
                          Bool_t keepTargetAxis ) const {
    const int ndim = axes->GetSize();
 
-   TH1* hist = 0;
+   TH1* hist = nullptr;
    // create hist with dummy axes, we fix them later.
    if (ndim == 1)
       hist = new TH1D(name, title, 1, 0., 1.);
@@ -155,7 +255,7 @@ TH1* THnBase::CreateHist(const char* name, const char* title,
       hist = new TH3D(name, title, 1, 0., 1., 1, 0., 1., 1, 0., 1.);
    else {
       Error("CreateHist", "Cannot create histogram %s with %d dimensions!", name, ndim);
-      return 0;
+      return nullptr;
    }
 
    TAxis* hax[3] = {hist->GetXaxis(), hist->GetYaxis(), hist->GetZaxis()};
@@ -214,7 +314,7 @@ THnBase* THnBase::CreateHnAny(const char* name, const char* title,
    // type of the TH1. The class name will be "TH??\0" where the first
    // ? is 1,2 or 3 and the second ? indicates the storage as C, S,
    // I, F or D.
-   THnBase* s = 0;
+   THnBase* s = nullptr;
    const char* cname( h->ClassName() );
    if (cname[0] == 'T' && cname[1] == 'H'
        && cname[2] >= '1' && cname[2] <= '3' && cname[4] == 0) {
@@ -240,7 +340,7 @@ break;
    }
    if (!s) {
       ::Warning("THnSparse::CreateHnAny", "Unknown Type of Histogram");
-      return 0;
+      return nullptr;
    }
 
    for (int i = 0; i < ndim; ++i) {
@@ -251,7 +351,7 @@ break;
    const TArray *array = dynamic_cast<const TArray*>(h);
    if (!array) {
       ::Warning("THnSparse::CreateHnAny", "Unknown Type of Histogram");
-      return 0;
+      return nullptr;
    }
 
    s->Add(h);
@@ -267,7 +367,7 @@ THnBase* THnBase::CreateHnAny(const char* name, const char* title,
                               const THnBase* hn, Bool_t sparse,
                               Int_t chunkSize /*= 1024 * 16*/)
 {
-   TClass* type = 0;
+   TClass* type = nullptr;
    if (hn->InheritsFrom(THnSparse::Class())) {
       if (sparse) type = hn->IsA();
       else {
@@ -281,7 +381,7 @@ THnBase* THnBase::CreateHnAny(const char* name, const char* title,
          else {
             hn->Error("CreateHnAny", "Type %s not implemented; please inform the ROOT team!",
                       hn->IsA()->GetName());
-            return 0;
+            return nullptr;
          }
          type = TClass::GetClass(TString::Format("THn%c", bintype));
       }
@@ -297,7 +397,7 @@ THnBase* THnBase::CreateHnAny(const char* name, const char* title,
          else if (hn->InheritsFrom(THnL::Class())) bintype = 'L';
          else if (hn->InheritsFrom(THnL64::Class())) {
             hn->Error("CreateHnAny", "Type THnSparse with Long64_t bins is not available!");
-            return 0;
+            return nullptr;
          }
          if (bintype) {
             type = TClass::GetClass(TString::Format("THnSparse%c", bintype));
@@ -306,12 +406,12 @@ THnBase* THnBase::CreateHnAny(const char* name, const char* title,
    } else {
       hn->Error("CreateHnAny", "Unhandled type %s, not deriving from THn nor THnSparse!",
                 hn->IsA()->GetName());
-      return 0;
+      return nullptr;
    }
    if (!type) {
       hn->Error("CreateHnAny", "Unhandled type %s, please inform the ROOT team!",
                 hn->IsA()->GetName());
-      return 0;
+      return nullptr;
    }
 
    THnBase* ret = (THnBase*)type->New();
@@ -398,7 +498,7 @@ void THnBase::GetRandom(Double_t *rand, Bool_t subBinRandom /* = kTRUE */)
 
    // generate a random bin
    Double_t p = gRandom->Rndm();
-   Long64_t idx = TMath::BinarySearch(GetNbins() + 1, fIntegral, p);
+   Long64_t idx = TMath::BinarySearch(GetNbins() + 1, fIntegral.data(), p);
    const Int_t nStaticBins = 40;
    Int_t bin[nStaticBins];
    Int_t* pBin = bin;
@@ -483,11 +583,11 @@ TObject* THnBase::ProjectionAny(Int_t ndim, const Int_t* dim,
       newaxes.AddAt(GetAxis(dim[d]),d);
    }
 
-   THnBase* hn = 0;
-   TH1* hist = 0;
-   TObject* ret = 0;
+   THnBase* hn = nullptr;
+   TH1* hist = nullptr;
+   TObject* ret = nullptr;
 
-   Bool_t* hadRange = 0;
+   Bool_t* hadRange = nullptr;
    Bool_t ignoreTargetRange = (option && (strchr(option, 'A') || strchr(option, 'a')));
    Bool_t keepTargetAxis = ignoreTargetRange || (option && (strchr(option, 'O') || strchr(option, 'o')));
    if (ignoreTargetRange) {
@@ -643,7 +743,7 @@ void THnBase::AddInternal(const THnBase* h, Double_t c, Bool_t rebinned)
       Sumw2();
    Bool_t haveErrors = GetCalculateErrors();
 
-   Double_t* x = 0;
+   Double_t* x = nullptr;
    if (rebinned) {
       x = new Double_t[fNdimensions];
    }
@@ -682,6 +782,18 @@ void THnBase::AddInternal(const THnBase* h, Double_t c, Bool_t rebinned)
    delete [] coord;
    delete [] x;
 
+   // add also the statistics
+   fTsumw += c * h->fTsumw;
+   if (haveErrors) {
+      fTsumw2 += c * c * h->fTsumw2;
+      if (h->fTsumwx.fN == fNdimensions && h->fTsumwx2.fN == fNdimensions) {
+         for (Int_t d = 0; d < fNdimensions; ++d) {
+            fTsumwx[d] += c * h->fTsumwx[d];
+            fTsumwx2[d] += c * h->fTsumwx2[d];
+         }
+      }
+   }
+
    Double_t nEntries = GetEntries() + c * h->GetEntries();
    SetEntries(nEntries);
 }
@@ -725,7 +837,7 @@ Long64_t THnBase::Merge(TCollection* list)
 
    Long64_t sumNbins = GetNbins();
    TIter iter(list);
-   const TObject* addMeObj = 0;
+   const TObject* addMeObj = nullptr;
    while ((addMeObj = iter())) {
       const THnBase* addMe = dynamic_cast<const THnBase*>(addMeObj);
       if (addMe) {
@@ -824,7 +936,7 @@ void THnBase::Multiply(TF1* f, Double_t c)
       TF1::RejectPoint(kFALSE);
 
       // Evaluate function at points
-      Double_t fvalue = f->EvalPar(x, NULL);
+      Double_t fvalue = f->EvalPar(x, nullptr);
 
       SetBinContent(i, c * fvalue * value);
       if (wantErrors) {
@@ -1074,7 +1186,7 @@ THnBase* THnBase::RebinBase(const Int_t* group) const
    Int_t ndim = GetNdimensions();
    TString name(GetName());
    for (Int_t d = 0; d < ndim; ++d)
-      name += Form("_%d", group[d]);
+      name += TString::Format("_%d", group[d]);
 
 
    TString title(GetTitle());
@@ -1082,10 +1194,10 @@ THnBase* THnBase::RebinBase(const Int_t* group) const
    if (posInsert == kNPOS) {
       title += " rebin ";
       for (Int_t d = 0; d < ndim; ++d)
-         title += Form("{%d}", group[d]);
+         title += TString::Format("{%d}", group[d]);
    } else {
       for (Int_t d = ndim - 1; d >= 0; --d)
-         title.Insert(posInsert, Form("{%d}", group[d]));
+         title.Insert(posInsert, TString::Format("{%d}", group[d]));
       title.Insert(posInsert, " rebin ");
    }
 
@@ -1154,7 +1266,7 @@ void THnBase::ResetBase(Option_t * /*option = ""*/)
    fTsumw = 0.;
    fTsumw2 = -1.;
    if (fIntegralStatus != kNoInt) {
-      delete [] fIntegral;
+      fIntegral.clear();
       fIntegralStatus = kNoInt;
    }
 }
@@ -1166,7 +1278,7 @@ Double_t THnBase::ComputeIntegral()
 {
    // delete old integral
    if (fIntegralStatus != kNoInt) {
-      delete [] fIntegral;
+      fIntegral.clear();
       fIntegralStatus = kNoInt;
    }
 
@@ -1177,7 +1289,7 @@ Double_t THnBase::ComputeIntegral()
    }
 
    // allocate integral array
-   fIntegral = new Double_t [GetNbins() + 1];
+   fIntegral.resize(GetNbins() + 1);
    fIntegral[0] = 0.;
 
    // fill integral array with contents of regular bins (non over/underflow)
@@ -1206,7 +1318,7 @@ Double_t THnBase::ComputeIntegral()
    // check sum of weights
    if (fIntegral[GetNbins()] == 0.) {
       Error("ComputeIntegral", "No hits in regular bins (non over/underflow).");
-      delete [] fIntegral;
+      fIntegral.clear();
       return 0.;
    }
 
@@ -1338,7 +1450,7 @@ void THnBase::Print(Option_t* options) const
    Bool_t optStat    = options && (strchr(options, 'S') || (strchr(options, 's')));
    Bool_t optContent = options && (strchr(options, 'C') || (strchr(options, 'c')));
 
-   Printf("%s (*0x%lx): \"%s\" \"%s\"", IsA()->GetName(), (unsigned long)this, GetName(), GetTitle());
+   Printf("%s (*0x%zx): \"%s\" \"%s\"", IsA()->GetName(), (size_t)this, GetName(), GetTitle());
    Printf("  %d dimensions, %g entries in %lld filled bins", GetNdimensions(), GetEntries(), GetNbins());
 
    if (optAxis) {
@@ -1430,7 +1542,7 @@ ClassImp(ROOT::Internal::THnBaseBrowsable);
 /// Construct a THnBaseBrowsable.
 
 ROOT::Internal::THnBaseBrowsable::THnBaseBrowsable(THnBase* hist, Int_t axis):
-fHist(hist), fAxis(axis), fProj(0)
+fHist(hist), fAxis(axis), fProj(nullptr)
 {
    TString axisName = hist->GetAxis(axis)->GetName();
    if (axisName.IsNull()) {
