@@ -47,7 +47,7 @@ These classes provide algorithms for integration of one-dimensional functions, w
 and for integration of multi-dimensional function using an adaptive method or MonteCarlo Integration (GSLMCIntegrator).
 The basic classes ROOT::Math::IntegratorOneDim provides a common interface for the one-dimensional methods while the class
 ROOT::Math::IntegratorMultiDim provides the interface for the multi-dimensional ones.
-The methods can be configured (e.g  setting the default method with its defult parameters) using the ROOT::Math::IntegratorOneDimOptions and
+The methods can be configured (e.g  setting the default method with its default parameters) using the ROOT::Math::IntegratorOneDimOptions and
 ROOT::Math::IntegratorMultiDimOptions classes.
 
 @ingroup  NumAlgo
@@ -82,8 +82,8 @@ Integration::GKRule. The default rule is 31 points.
 In the case of integration over infinite and semi-infinite ranges, the type used is always
 ADAPTIVESINGULAR applying a transformation from the original interval into (0,1).
 
-The ADAPTIVESINGULAR type is the most sophicticated type. When performances are
-important, it is then recommened to use the NONADAPTIVE type in case of smooth functions or
+The ADAPTIVESINGULAR type is the most sophisticated type. When performances are
+important, it is then recommended to use the NONADAPTIVE type in case of smooth functions or
  ADAPTIVE with a lower Gauss-Kronrod rule.
 
 For detailed description on GSL integration algorithms see the
@@ -121,7 +121,7 @@ public:
     */
     explicit
     IntegratorOneDim(IntegrationOneDim::Type type = IntegrationOneDim::kDEFAULT, double absTol = -1, double relTol = -1, unsigned int size = 0, unsigned int rule = 0) :
-       fIntegrator(0), fFunc(0)
+       fIntegrator(nullptr), fFunc(nullptr)
    {
       fIntegrator = CreateIntegrator(type, absTol, relTol, size, rule);
    }
@@ -140,7 +140,7 @@ public:
     */
    explicit
    IntegratorOneDim(const IGenFunction &f, IntegrationOneDim::Type type = IntegrationOneDim::kDEFAULT, double absTol = -1, double relTol = -1, unsigned int size = 0, int rule = 0) :
-      fIntegrator(0), fFunc(0)
+      fIntegrator(nullptr), fFunc(nullptr)
    {
       fIntegrator = CreateIntegrator(type, absTol, relTol, size, rule);
       SetFunction(f,true);
@@ -163,7 +163,7 @@ public:
    template<class Function>
    explicit
    IntegratorOneDim(Function & f, IntegrationOneDim::Type type = IntegrationOneDim::kDEFAULT, double absTol = -1, double relTol = -1, unsigned int size = 0, int rule = 0) :
-      fIntegrator(0), fFunc(0)
+      fIntegrator(nullptr), fFunc(nullptr)
    {
       fIntegrator = CreateIntegrator(type, absTol, relTol, size, rule);
       SetFunction(f);
@@ -175,10 +175,10 @@ public:
       if (fFunc) delete fFunc;
    }
 
-   // disable copy constructur and assignment operator
+   // disable copy constructor and assignment operator
 
 private:
-   IntegratorOneDim(const IntegratorOneDim &) : fIntegrator(0), fFunc(0) {}
+   IntegratorOneDim(const IntegratorOneDim &) : fIntegrator(nullptr), fFunc(nullptr) {}
    IntegratorOneDim & operator=(const IntegratorOneDim &) { return *this; }
 
 public:
@@ -188,7 +188,7 @@ public:
 
    /**
       method to set the a generic integration function
-      @param f integration function. The function type must implement the assigment operator, <em>  double  operator() (  double  x ) </em>
+      @param f integration function. The function type must implement the assignment operator, <em>  double  operator() (  double  x ) </em>
 
    */
 
@@ -244,10 +244,10 @@ public:
    }
 
 
-    /**
-      evaluate the Integral of a function f over the infinite interval (-inf,+inf)
-       @param f integration function. The function type must be a C++ callable object implementing operator()(double x)
-    */
+//    /**
+//      evaluate the Integral of a function f over the infinite interval (-inf,+inf)
+//       @param f integration function. The function type must be a C++ callable object implementing operator()(double x)
+//    */
 //    template<class Function>
 //    double Integral(const Function & f);
 
@@ -261,11 +261,11 @@ public:
    }
 
 
-    /**
-      evaluate the Integral of a function f over the semi-infinite interval (a,+inf)
-      @param f integration function. The function type must be a C++ callable object implementing operator()(double x)
-      @param a lower value of the integration interval
-    */
+//    /**
+//      evaluate the Integral of a function f over the semi-infinite interval (a,+inf)
+//      @param f integration function. The function type must be a C++ callable object implementing operator()(double x)
+//      @param a lower value of the integration interval
+//    */
 //    template<class Function>
 //    double IntegralUp(Function & f, double a);
 
@@ -280,11 +280,11 @@ public:
       return IntegralUp(a);
    }
 
-    /**
-      evaluate the Integral of a function f over the over the semi-infinite interval (-inf,b)
-      @param f integration function. The function type must be a C++ callable object implementing operator()(double x)
-      @param b upper value of the integration interval
-    */
+//    /**
+//      evaluate the Integral of a function f over the over the semi-infinite interval (-inf,b)
+//      @param f integration function. The function type must be a C++ callable object implementing operator()(double x)
+//      @param b upper value of the integration interval
+//    */
 //    template<class Function>
 //    double IntegralLow(Function & f, double b);
 
@@ -353,7 +353,7 @@ public:
    */
 
    double Integral(double a, double b) {
-      return fIntegrator == 0 ? 0 : fIntegrator->Integral(a,b);
+      return !fIntegrator ? 0 : fIntegrator->Integral(a,b);
    }
 
 
@@ -362,7 +362,7 @@ public:
    */
 
    double Integral( ) {
-      return fIntegrator == 0 ? 0 : fIntegrator->Integral();
+      return !fIntegrator ? 0 : fIntegrator->Integral();
    }
 
    /**
@@ -370,7 +370,7 @@ public:
       @param a lower value of the integration interval
    */
    double IntegralUp(double a ) {
-      return fIntegrator == 0 ? 0 : fIntegrator->IntegralUp(a);
+      return !fIntegrator ? 0 : fIntegrator->IntegralUp(a);
    }
 
    /**
@@ -378,7 +378,7 @@ public:
       @param b upper value of the integration interval
    */
    double IntegralLow( double b ) {
-      return fIntegrator == 0 ? 0 : fIntegrator->IntegralLow(b);
+      return !fIntegrator ? 0 : fIntegrator->IntegralLow(b);
    }
    /**
        define operator() for IntegralLow
@@ -394,7 +394,7 @@ public:
 
    */
    double Integral( const std::vector<double> & pts) {
-      return fIntegrator == 0 ? 0 : fIntegrator->Integral(pts);
+      return !fIntegrator ? 0 : fIntegrator->Integral(pts);
    }
 
    /**
@@ -402,29 +402,29 @@ public:
 
    */
    double IntegralCauchy(double a, double b, double c) {
-      return fIntegrator == 0 ? 0 : fIntegrator->IntegralCauchy(a,b,c);
+      return !fIntegrator ? 0 : fIntegrator->IntegralCauchy(a,b,c);
    }
 
    /**
       return  the Result of the last Integral calculation
    */
-   double Result() const { return fIntegrator == 0 ? 0 : fIntegrator->Result(); }
+   double Result() const { return !fIntegrator ? 0 : fIntegrator->Result(); }
 
    /**
       return the estimate of the absolute Error of the last Integral calculation
    */
-   double Error() const { return fIntegrator == 0 ? 0 : fIntegrator->Error(); }
+   double Error() const { return !fIntegrator ? 0 : fIntegrator->Error(); }
 
    /**
       return the Error Status of the last Integral calculation
    */
-   int Status() const { return fIntegrator == 0 ? -1 : fIntegrator->Status(); }
+   int Status() const { return !fIntegrator ? -1 : fIntegrator->Status(); }
 
    /**
       return number of function evaluations in calculating the integral
       (if integrator do not implement this function returns -1)
    */
-   int NEval() const { return fIntegrator == 0 ? -1 : fIntegrator->NEval(); }
+   int NEval() const { return !fIntegrator ? -1 : fIntegrator->NEval(); }
 
 
    // setter for control Parameters  (getters are not needed so far )
@@ -471,8 +471,8 @@ protected:
 
 private:
 
-   VirtualIntegratorOneDim * fIntegrator;   // pointer to integrator interface class
-   IGenFunction            * fFunc;         // pointer to owned function
+   VirtualIntegratorOneDim * fIntegrator;   ///< pointer to integrator interface class
+   IGenFunction            * fFunc;         ///< pointer to owned function
 
 };
 
