@@ -17,19 +17,21 @@
 #include <RooBinWidthFunction.h>
 #include <RooCategory.h>
 #include <RooDataHist.h>
+#include <RooExponential.h>
 #include <RooFit/Detail/JSONInterface.h>
 #include <RooFitHS3/JSONIO.h>
 #include <RooFormulaVar.h>
 #include <RooGenericPdf.h>
-#include <RooMultiVarGaussian.h>
-#include <RooTFnBinding.h>
 #include <RooHistFunc.h>
 #include <RooHistPdf.h>
-#include <RooProdPdf.h>
+#include <RooLognormal.h>
+#include <RooMultiVarGaussian.h>
+#include <RooPoisson.h>
 #include <RooPolynomial.h>
 #include <RooRealSumFunc.h>
 #include <RooRealSumPdf.h>
 #include <RooRealVar.h>
+#include <RooTFnBinding.h>
 #include <RooWorkspace.h>
 
 #include <TF1.h>
@@ -37,6 +39,9 @@
 
 #include "static_execute.h"
 
+#include <algorithm>
+#include <cctype>
+
 using RooFit::Detail::JSONNode;
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -55,34 +60,36 @@ namespace {
  * opening parenthesis, are not treated as arguments. The extracted
  * arguments are returned as a vector of strings.
  *
- * @param expression A string representing a mathematical expression.
+ * @param expr A string representing a mathematical expression.
  * @return A vector of strings representing the extracted arguments.
  */
-std::vector<std::string> extract_arguments(const std::string &expression)
+std::vector<std::string> extractArguments(std::string expr)
 {
+   // Get rid of whitespaces
+   expr.erase(std::remove_if(expr.begin(), expr.end(), [](unsigned char c) { return std::isspace(c); }), expr.end());
+
    std::vector<std::string> arguments;
-   size_t startidx = expression.size();
-   for (size_t i = 0; i < expression.size(); ++i) {
-      if (startidx >= expression.size()) {
-         if (isalpha(expression[i])) {
+   size_t startidx = expr.size();
+   for (size_t i = 0; i < expr.size(); ++i) {
+      if (startidx >= expr.size()) {
+         if (isalpha(expr[i])) {
             startidx = i;
          }
       } else {
-         if (!isdigit(expression[i]) && !isalpha(expression[i]) && expression[i] != '_') {
-            if (expression[i] == ' ')
-               continue;
-            if (expression[i] == '(') {
-               startidx = expression.size();
+         if (!isdigit(expr[i]) && !isalpha(expr[i]) && expr[i] != '_') {
+            if (expr[i] == '(') {
+               startidx = expr.size();
                continue;
             }
-            std::string arg(expression.substr(startidx, i - startidx));
-            startidx = expression.size();
+            std::string arg(expr.substr(startidx, i - startidx));
+            startidx = expr.size();
             arguments.push_back(arg);
          }
       }
    }
-   if (startidx < expression.size())
-      arguments.push_back(expression.substr(startidx));
+   if (startidx < expr.size()) {
+      arguments.push_back(expr.substr(startidx));
+   }
    return arguments;
 }
 
@@ -97,7 +104,7 @@ public:
       }
       TString formula(p["expression"].val());
       RooArgList dependents;
-      for (const auto &d : extract_arguments(formula.Data())) {
+      for (const auto &d : extractArguments(formula.Data())) {
          dependents.add(*tool->request<RooAbsReal>(d, name));
       }
       tool->wsImport(RooArg_t{name.c_str(), formula, dependents});
@@ -137,7 +144,6 @@ public:
       RooRealVar *obs = tool->requestArg<RooRealVar>(p, "observable");
 
       if (!pdf->dependsOn(*obs)) {
-         pdf->Print("t");
          RooJSONFactoryWSTool::error(std::string("pdf '") + pdf->GetName() + "' does not depend on observable '" +
                                      obs->GetName() + "' as indicated by parent RooBinSamplingPdf '" + name +
                                      "', please check!");
@@ -212,6 +218,49 @@ public:
    }
 };
 
+class RooPoissonFactory : public RooFit::JSONIO::Importer {
+public:
+   bool importArg(RooJSONFactoryWSTool *tool, const JSONNode &p) const override
+   {
+      std::string name(RooJSONFactoryWSTool::name(p));
+      RooAbsReal *x = tool->requestArg<RooAbsReal>(p, "x");
+      RooAbsReal *mean = tool->requestArg<RooAbsReal>(p, "mean");
+      tool->wsEmplace<RooPoisson>(name, *x, *mean, !p["integer"].val_bool());
+      return true;
+   }
+};
+
+class RooLogNormalFactory : public RooFit::JSONIO::Importer {
+public:
+   bool importArg(RooJSONFactoryWSTool *tool, const JSONNode &p) const override
+   {
+      std::string name(RooJSONFactoryWSTool::name(p));
+      RooAbsReal *x = tool->requestArg<RooAbsReal>(p, "x");
+      RooAbsReal *mu = tool->requestArg<RooAbsReal>(p, "mu");
+      RooAbsReal *sigma = tool->requestArg<RooAbsReal>(p, "sigma");
+
+      // TODO: check if the pdf was originally exported by ROOT, in which case
+      // it can be imported back without using the standard parametrization.
+      tool->wsEmplace<RooLognormal>(name, *x, *mu, *sigma, true);
+      return true;
+   }
+};
+
+class RooExponentialFactory : public RooFit::JSONIO::Importer {
+public:
+   bool importArg(RooJSONFactoryWSTool *tool, const JSONNode &p) const override
+   {
+      std::string name(RooJSONFactoryWSTool::name(p));
+      RooAbsReal *x = tool->requestArg<RooAbsReal>(p, "x");
+      RooAbsReal *c = tool->requestArg<RooAbsReal>(p, "c");
+
+      // TODO: check if the pdf was originally exported by ROOT, in which case
+      // it can be imported back without using the standard parametrization.
+      tool->wsEmplace<RooExponential>(name, *x, *c, true);
+      return true;
+   }
+};
+
 class RooMultiVarGaussianFactory : public RooFit::JSONIO::Importer {
 public:
    bool importArg(RooJSONFactoryWSTool *tool, const JSONNode &p) const override
@@ -263,6 +312,20 @@ public:
 // specialized exporter implementations
 ///////////////////////////////////////////////////////////////////////////////////////////////////////
 
+class RooAddPdfStreamer : public RooFit::JSONIO::Exporter {
+public:
+   std::string const &key() const override;
+   bool exportObject(RooJSONFactoryWSTool *, const RooAbsArg *func, JSONNode &elem) const override
+   {
+      const RooAddPdf *pdf = static_cast<const RooAddPdf *>(func);
+      elem["type"] << key();
+      RooJSONFactoryWSTool::fillSeq(elem["summands"], pdf->pdfList());
+      RooJSONFactoryWSTool::fillSeq(elem["coefficients"], pdf->coefList());
+      elem["extended"] << (pdf->extendMode() != RooAbsPdf::CanNotBeExtended);
+      return true;
+   }
+};
+
 class RooRealSumPdfStreamer : public RooFit::JSONIO::Exporter {
 public:
    std::string const &key() const override;
@@ -272,7 +335,7 @@ public:
       elem["type"] << key();
       RooJSONFactoryWSTool::fillSeq(elem["samples"], pdf->funcList());
       RooJSONFactoryWSTool::fillSeq(elem["coefficients"], pdf->coefList());
-      elem["extended"] << (pdf->extendMode() == RooAbsPdf::CanBeExtended);
+      elem["extended"] << (pdf->extendMode() != RooAbsPdf::CanNotBeExtended);
       return true;
    }
 };
@@ -311,7 +374,8 @@ public:
       if (!p.has_child("data")) {
          RooJSONFactoryWSTool::error("function '" + name + "' is of histogram type, but does not define a 'data' key");
       }
-      std::unique_ptr<RooDataHist> dataHist = RooJSONFactoryWSTool::readBinnedData(p["data"], name);
+      std::unique_ptr<RooDataHist> dataHist =
+         RooJSONFactoryWSTool::readBinnedData(p["data"], name, RooJSONFactoryWSTool::readAxes(p["data"]));
       tool->wsEmplace<RooHistFunc>(name, *dataHist->get(), *dataHist);
       return true;
    }
@@ -338,7 +402,8 @@ public:
       if (!p.has_child("data")) {
          RooJSONFactoryWSTool::error("function '" + name + "' is of histogram type, but does not define a 'data' key");
       }
-      std::unique_ptr<RooDataHist> dataHist = RooJSONFactoryWSTool::readBinnedData(p["data"], name);
+      std::unique_ptr<RooDataHist> dataHist =
+         RooJSONFactoryWSTool::readBinnedData(p["data"], name, RooJSONFactoryWSTool::readAxes(p["data"]));
       tool->wsEmplace<RooHistPdf>(name, *dataHist->get(), *dataHist);
       return true;
    }
@@ -382,9 +447,12 @@ public:
       TString expression(pdf->expression());
       for (size_t i = 0; i < pdf->nParameters(); ++i) {
          RooAbsArg *par = pdf->getParameter(i);
-         std::stringstream ss;
-         ss << "x[" << i << "]";
-         expression.ReplaceAll(ss.str().c_str(), par->GetName());
+         std::stringstream ss_1;
+         ss_1 << "x[" << i << "]";
+         std::stringstream ss_2;
+         ss_2 << "@" << i << "";
+         expression.ReplaceAll(ss_1.str().c_str(), par->GetName());
+         expression.ReplaceAll(ss_2.str().c_str(), par->GetName());
       }
       elem["expression"] << expression.Data();
       return true;
@@ -413,6 +481,64 @@ public:
    }
 };
 
+class RooPoissonStreamer : public RooFit::JSONIO::Exporter {
+public:
+   std::string const &key() const override;
+   bool exportObject(RooJSONFactoryWSTool *, const RooAbsArg *func, JSONNode &elem) const override
+   {
+      auto *pdf = static_cast<const RooPoisson *>(func);
+      elem["type"] << key();
+      elem["x"] << pdf->getX().GetName();
+      elem["mean"] << pdf->getMean().GetName();
+      elem["integer"] << !pdf->getNoRounding();
+      return true;
+   }
+};
+
+class RooLogNormalStreamer : public RooFit::JSONIO::Exporter {
+public:
+   std::string const &key() const override;
+   bool exportObject(RooJSONFactoryWSTool *tool, const RooAbsArg *func, JSONNode &elem) const override
+   {
+      auto *pdf = static_cast<const RooLognormal *>(func);
+
+      elem["type"] << key();
+      elem["x"] << pdf->getX().GetName();
+
+      auto &m0 = pdf->getMedian();
+      auto &k = pdf->getShapeK();
+
+      if(pdf->useStandardParametrization()) {
+         elem["mu"] << m0.GetName();
+         elem["sigma"] << k.GetName();
+      } else {
+         elem["mu"] << tool->exportTransformed(&m0, "lognormal", "log", "log(%s)");
+         elem["sigma"] << tool->exportTransformed(&k, "lognormal", "log", "log(%s)");
+      }
+
+      return true;
+   }
+};
+
+class RooExponentialStreamer : public RooFit::JSONIO::Exporter {
+public:
+   std::string const &key() const override;
+   bool exportObject(RooJSONFactoryWSTool *tool, const RooAbsArg *func, JSONNode &elem) const override
+   {
+      auto *pdf = static_cast<const RooExponential *>(func);
+      elem["type"] << key();
+      elem["x"] << pdf->variable().GetName();
+      auto &c = pdf->coefficient();
+      if (pdf->negateCoefficient()) {
+         elem["c"] << c.GetName();
+      } else {
+         elem["c"] << tool->exportTransformed(&c, "exponential", "inverted", "-%s");
+      }
+
+      return true;
+   }
+};
+
 class RooMultiVarGaussianStreamer : public RooFit::JSONIO::Exporter {
 public:
    std::string const &key() const override;
@@ -455,18 +581,22 @@ public:
       return keystring;                          \
    }
 
-template <>
-DEFINE_EXPORTER_KEY(RooFormulaArgStreamer<RooGenericPdf>, "generic_dist");
-template <>
-DEFINE_EXPORTER_KEY(RooFormulaArgStreamer<RooFormulaVar>, "generic_function");
-DEFINE_EXPORTER_KEY(RooRealSumPdfStreamer, "weighted_sum_dist");
-DEFINE_EXPORTER_KEY(RooRealSumFuncStreamer, "weighted_sum");
-DEFINE_EXPORTER_KEY(RooHistFuncStreamer, "histogram");
-DEFINE_EXPORTER_KEY(RooHistPdfStreamer, "histogram_dist");
+DEFINE_EXPORTER_KEY(RooAddPdfStreamer, "mixture_dist");
 DEFINE_EXPORTER_KEY(RooBinSamplingPdfStreamer, "binsampling");
 DEFINE_EXPORTER_KEY(RooBinWidthFunctionStreamer, "binwidth");
+DEFINE_EXPORTER_KEY(RooExponentialStreamer, "exponential_dist");
+template <>
+DEFINE_EXPORTER_KEY(RooFormulaArgStreamer<RooFormulaVar>, "generic_function");
+template <>
+DEFINE_EXPORTER_KEY(RooFormulaArgStreamer<RooGenericPdf>, "generic_dist");
+DEFINE_EXPORTER_KEY(RooHistFuncStreamer, "histogram");
+DEFINE_EXPORTER_KEY(RooHistPdfStreamer, "histogram_dist");
+DEFINE_EXPORTER_KEY(RooLogNormalStreamer, "lognormal_dist");
+DEFINE_EXPORTER_KEY(RooMultiVarGaussianStreamer, "multivariate_normal_dist");
+DEFINE_EXPORTER_KEY(RooPoissonStreamer, "poisson_dist");
 DEFINE_EXPORTER_KEY(RooPolynomialStreamer, "polynomial_dist");
-DEFINE_EXPORTER_KEY(RooMultiVarGaussianStreamer, "multinormal_dist");
+DEFINE_EXPORTER_KEY(RooRealSumFuncStreamer, "weighted_sum");
+DEFINE_EXPORTER_KEY(RooRealSumPdfStreamer, "weighted_sum_dist");
 DEFINE_EXPORTER_KEY(RooTFnBindingStreamer, "generic_function");
 
 ///////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -476,28 +606,35 @@ DEFINE_EXPORTER_KEY(RooTFnBindingStreamer, "generic_function");
 STATIC_EXECUTE([]() {
    using namespace RooFit::JSONIO;
 
-   registerImporter<RooFormulaArgFactory<RooGenericPdf>>("generic_dist", false);
-   registerImporter<RooFormulaArgFactory<RooFormulaVar>>("generic_function", false);
-   registerImporter<RooBinSamplingPdfFactory>("binsampling_dist", false);
    registerImporter<RooAddPdfFactory>("mixture_dist", false);
+   registerImporter<RooBinSamplingPdfFactory>("binsampling_dist", false);
+   registerImporter<RooBinWidthFunctionFactory>("binwidth", false);
+   registerImporter<RooExponentialFactory>("exponential_dist", false);
+   registerImporter<RooFormulaArgFactory<RooFormulaVar>>("generic_function", false);
+   registerImporter<RooFormulaArgFactory<RooGenericPdf>>("generic_dist", false);
    registerImporter<RooHistFuncFactory>("histogram", false);
    registerImporter<RooHistPdfFactory>("histogram_dist", false);
-   registerImporter<RooBinWidthFunctionFactory>("binwidth", false);
+   registerImporter<RooLogNormalFactory>("lognormal_dist", false);
+   registerImporter<RooMultiVarGaussianFactory>("multivariate_normal_dist", false);
+   registerImporter<RooPoissonFactory>("poisson_dist", false);
+   registerImporter<RooPolynomialFactory>("polynomial_dist", false);
    registerImporter<RooRealSumPdfFactory>("weighted_sum_dist", false);
    registerImporter<RooRealSumFuncFactory>("weighted_sum", false);
-   registerImporter<RooPolynomialFactory>("polynomial_dist", false);
-   registerImporter<RooMultiVarGaussianFactory>("multinormal_dist", false);
 
-   registerExporter<RooBinWidthFunctionStreamer>(RooBinWidthFunction::Class(), false);
+   registerExporter<RooAddPdfStreamer>(RooAddPdf::Class(), false);
    registerExporter<RooBinSamplingPdfStreamer>(RooBinSamplingPdf::Class(), false);
+   registerExporter<RooBinWidthFunctionStreamer>(RooBinWidthFunction::Class(), false);
+   registerExporter<RooExponentialStreamer>(RooExponential::Class(), false);
+   registerExporter<RooFormulaArgStreamer<RooFormulaVar>>(RooFormulaVar::Class(), false);
+   registerExporter<RooFormulaArgStreamer<RooGenericPdf>>(RooGenericPdf::Class(), false);
    registerExporter<RooHistFuncStreamer>(RooHistFunc::Class(), false);
    registerExporter<RooHistPdfStreamer>(RooHistPdf::Class(), false);
-   registerExporter<RooFormulaArgStreamer<RooGenericPdf>>(RooGenericPdf::Class(), false);
-   registerExporter<RooFormulaArgStreamer<RooFormulaVar>>(RooFormulaVar::Class(), false);
-   registerExporter<RooRealSumPdfStreamer>(RooRealSumPdf::Class(), false);
-   registerExporter<RooRealSumFuncStreamer>(RooRealSumFunc::Class(), false);
-   registerExporter<RooPolynomialStreamer>(RooPolynomial::Class(), false);
+   registerExporter<RooLogNormalStreamer>(RooLognormal::Class(), false);
    registerExporter<RooMultiVarGaussianStreamer>(RooMultiVarGaussian::Class(), false);
+   registerExporter<RooPoissonStreamer>(RooPoisson::Class(), false);
+   registerExporter<RooPolynomialStreamer>(RooPolynomial::Class(), false);
+   registerExporter<RooRealSumFuncStreamer>(RooRealSumFunc::Class(), false);
+   registerExporter<RooRealSumPdfStreamer>(RooRealSumPdf::Class(), false);
    registerExporter<RooTFnBindingStreamer>(RooTFnBinding::Class(), false);
 });
 
