@@ -32,19 +32,35 @@ using namespace std;
 
 ClassImp(RooBCPEffDecay);
 
-////////////////////////////////////////////////////////////////////////////////
-/// Constructor.
+/// \brief Constructor for RooBCPEffDecay.
+///
+/// Creates an instance of RooBCPEffDecay with the specified parameters.
+///
+/// \param[in] name         The name of the PDF.
+/// \param[in] title        The title of the PDF.
+/// \param[in] t            The time variable.
+/// \param[in] tag          The CP state category.
+/// \param[in] tau          The decay time parameter.
+/// \param[in] dm           The mixing frequency parameter.
+/// \param[in] avgMistag    The average mistag rate parameter.
+/// \param[in] CPeigenval   The CP eigen value parameter.
+/// \param[in] absLambda    The absolute value of the complex lambda parameter.
+/// \param[in] argLambda    The argument of the complex lambda parameter.
+/// \param[in] effRatio     The B0/B0bar efficiency ratio.
+/// \param[in] delMistag    Delta mistag rate parameter.
+/// \param[in] model        The resolution model.
+/// \param[in] type         The decay type.
 
 RooBCPEffDecay::RooBCPEffDecay(const char *name, const char *title,
                 RooRealVar& t, RooAbsCategory& tag,
                 RooAbsReal& tau, RooAbsReal& dm,
                 RooAbsReal& avgMistag, RooAbsReal& CPeigenval,
-                RooAbsReal& a, RooAbsReal& b,
+                RooAbsReal& absLambda, RooAbsReal& argLambda,
                 RooAbsReal& effRatio, RooAbsReal& delMistag,
                 const RooResolutionModel& model, DecayType type) :
   RooAbsAnaConvPdf(name,title,model,t),
-  _absLambda("absLambda","Absolute value of lambda",this,a),
-  _argLambda("argLambda","Arg(Lambda)",this,b),
+  _absLambda("absLambda","Absolute value of lambda",this,absLambda),
+  _argLambda("argLambda","Arg(Lambda)",this,argLambda),
   _effRatio("effRatio","B0/B0bar efficiency ratio",this,effRatio),
   _CPeigenval("CPeigenval","CP eigen value",this,CPeigenval),
   _avgMistag("avgMistag","Average mistag rate",this,avgMistag),
@@ -98,13 +114,6 @@ RooBCPEffDecay::RooBCPEffDecay(const RooBCPEffDecay& other, const char* name) :
 {
 }
 
-////////////////////////////////////////////////////////////////////////////////
-/// Destructor.
-
-RooBCPEffDecay::~RooBCPEffDecay()
-{
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 /// B0    : _tag = +1
 ///
@@ -206,7 +215,7 @@ void RooBCPEffDecay::generateEvent(Int_t code)
   }
 
   // Generate delta-t dependent
-  while(1) {
+  while(true) {
     double rand = RooRandom::uniform() ;
     double tval(0) ;
 
