@@ -22,51 +22,23 @@ distribution. Object of class RooGaussModel can be used
 for analytical convolutions with classes inheriting from RooAbsAnaConvPdf
 **/
 
-#include "RooFit.h"
-
 #include "TMath.h"
 #include "Riostream.h"
 #include "RooGaussModel.h"
 #include "RooMath.h"
 #include "RooRealConstant.h"
 #include "RooRandom.h"
+#include "RooBatchCompute.h"
 
 #include "TError.h"
 
-
-namespace {
-
-////////////////////////////////////////////////////////////////////////////////
-/// use the approximation: erf(z) = exp(-z*z)/(std::sqrt(pi)*z)
-/// to explicitly cancel the divergent exp(y*y) behaviour of
-/// CWERF for z = x + i y with large negative y
-
-std::complex<Double_t> evalCerfApprox(Double_t _x, Double_t u, Double_t c)
-{
-  static const Double_t rootpi= std::sqrt(std::atan2(0.,-1.));
-  const std::complex<Double_t> z(_x * c, u + c);
-  const std::complex<Double_t> zc(u + c, - _x * c);
-  const std::complex<Double_t> zsq((z.real() + z.imag()) * (z.real() - z.imag()),
-     2. * z.real() * z.imag());
-  const std::complex<Double_t> v(-zsq.real() - u*u, -zsq.imag());
-  const std::complex<Double_t> ev = std::exp(v);
-  const std::complex<Double_t> mez2zcrootpi = -std::exp(zsq)/(zc*rootpi);
-
-  return 2. * (ev * (mez2zcrootpi + 1.));
-}
-
-// Calculate exp(-u^2) cwerf(swt*c + i(u+c)), taking care of numerical instabilities
-inline std::complex<Double_t> evalCerf(Double_t swt, Double_t u, Double_t c)
-{
-  std::complex<Double_t> z(swt*c,u+c);
-  return (z.imag()>-4.0) ? (std::exp(-u*u)*RooMath::faddeeva_fast(z)) : evalCerfApprox(swt,u,c);
-}
-
-} // namespace
-
+#include <RooHeterogeneousMath.h>
 
 using namespace std;
 
+using RooHeterogeneousMath::evalCerf;
+using RooHeterogeneousMath::evalCerfApprox;
+
 ClassImp(RooGaussModel);
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -74,8 +46,8 @@ ClassImp(RooGaussModel);
 RooGaussModel::RooGaussModel(const char *name, const char *title, RooAbsRealLValue& xIn,
               RooAbsReal& _mean, RooAbsReal& _sigma) :
   RooResolutionModel(name,title,xIn),
-  _flatSFInt(kFALSE),
-  _asympInt(kFALSE),
+  _flatSFInt(false),
+  _asympInt(false),
   mean("mean","Mean",this,_mean),
   sigma("sigma","Width",this,_sigma),
   msf("msf","Mean Scale Factor",this,(RooRealVar&)RooRealConstant::value(1)),
@@ -89,8 +61,8 @@ RooGaussModel::RooGaussModel(const char *name, const char *title, RooAbsRealLVal
               RooAbsReal& _mean, RooAbsReal& _sigma,
               RooAbsReal& _msSF) :
   RooResolutionModel(name,title,xIn),
-  _flatSFInt(kFALSE),
-  _asympInt(kFALSE),
+  _flatSFInt(false),
+  _asympInt(false),
   mean("mean","Mean",this,_mean),
   sigma("sigma","Width",this,_sigma),
   msf("msf","Mean Scale Factor",this,_msSF),
@@ -104,8 +76,8 @@ RooGaussModel::RooGaussModel(const char *name, const char *title, RooAbsRealLVal
               RooAbsReal& _mean, RooAbsReal& _sigma,
               RooAbsReal& _meanSF, RooAbsReal& _sigmaSF) :
   RooResolutionModel(name,title,xIn),
-  _flatSFInt(kFALSE),
-  _asympInt(kFALSE),
+  _flatSFInt(false),
+  _asympInt(false),
   mean("mean","Mean",this,_mean),
   sigma("sigma","Width",this,_sigma),
   msf("msf","Mean Scale Factor",this,_meanSF),
@@ -159,105 +131,142 @@ Int_t RooGaussModel::basisCode(const char* name) const
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Double_t RooGaussModel::evaluate() const
+double RooGaussModel::evaluate() const
+{
+   auto arg1 = static_cast<RooAbsReal*>(basis().getParameter(1));
+   auto arg2 = static_cast<RooAbsReal*>(basis().getParameter(2));
+   double param1 = arg1 ? arg1->getVal() : 0.0;
+   double param2 = arg2 ? arg2->getVal() : 0.0;
+   return evaluate(x, mean * msf, sigma * ssf, param1, param2, _basisCode);
+}
+
+void RooGaussModel::computeBatch(double *output, size_t size,
+                                 RooFit::Detail::DataMap const &dataMap) const
+{
+   auto xVals = dataMap.at(x);
+   auto meanVals = dataMap.at(mean);
+   auto meanSfVals = dataMap.at(msf);
+   auto sigmaVals = dataMap.at(sigma);
+   auto sigmaSfVals = dataMap.at(ssf);
+
+   auto param1 = static_cast<RooAbsReal *>(basis().getParameter(1));
+   auto param2 = static_cast<RooAbsReal *>(basis().getParameter(2));
+   const double zeroVal = 0.0;
+   auto param1Vals = param1 ? dataMap.at(param1) : std::span<const double>{&zeroVal, 1};
+   auto param2Vals = param2 ? dataMap.at(param2) : std::span<const double>{&zeroVal, 1};
+
+   BasisType basisType = getBasisType(_basisCode);
+   double basisSign = _basisCode - 10 * (basisType - 1) - 2;
+
+   // We have an implementation also for CUDA right now only for the most used
+   // basis type, which is expBasis. If the need to support other basis types
+   // arises, they can be implemented following this example. Remember to also
+   // adapt RooGaussModel::canComputeBatchWithCuda().
+   if (basisType == expBasis) {
+      RooBatchCompute::ArgVector extraArgs{basisSign};
+      RooBatchCompute::compute(dataMap.config(this), RooBatchCompute::GaussModelExpBasis, output, size,
+                        {xVals, meanVals, meanSfVals, sigmaVals, sigmaSfVals, param1Vals}, extraArgs);
+      return;
+   }
+
+   // For now, if the arrays don't have the expected input shape, fall back to the scalar mode
+   if (xVals.size() != size || meanVals.size() != 1 || meanSfVals.size() != 1 || sigmaVals.size() != 1 ||
+       sigmaSfVals.size() != 1 || param1Vals.size() != 1 || param2Vals.size() != 1) {
+      return RooAbsPdf::computeBatch(output, size, dataMap);
+   }
+
+   for (unsigned int i = 0; i < size; ++i) {
+      output[i] = evaluate(xVals[i], meanVals[0] * meanSfVals[0], sigmaVals[0] * sigmaSfVals[0], param1Vals[0],
+                           param2Vals[0], _basisCode);
+   }
+}
+
+double RooGaussModel::evaluate(double x, double mean, double sigma, double param1, double param2, int basisCode)
 {
   // *** 1st form: Straight Gaussian, used for unconvoluted PDF or expBasis with 0 lifetime ***
-  static Double_t root2(std::sqrt(2.)) ;
-  static Double_t root2pi(std::sqrt(2.*std::atan2(0.,-1.))) ;
-  static Double_t rootpi(std::sqrt(std::atan2(0.,-1.))) ;
+  static double root2(std::sqrt(2.)) ;
+  static double root2pi(std::sqrt(2.*std::atan2(0.,-1.))) ;
+  static double rootpi(std::sqrt(std::atan2(0.,-1.))) ;
 
-  BasisType basisType = (BasisType)( (_basisCode == 0) ? 0 : (_basisCode/10) + 1 );
-  BasisSign basisSign = (BasisSign)( _basisCode - 10*(basisType-1) - 2 ) ;
+  BasisType basisType = getBasisType(basisCode);
+  BasisSign basisSign = (BasisSign)( basisCode - 10*(basisType-1) - 2 ) ;
 
-  Double_t tau = (_basisCode!=noBasis)?((RooAbsReal*)basis().getParameter(1))->getVal():0 ;
-  if (basisType == coshBasis && _basisCode!=noBasis ) {
-     Double_t dGamma = ((RooAbsReal*)basis().getParameter(2))->getVal();
+  double tau = (basisCode!=noBasis) ? param1 : 0.0;
+  if (basisType == coshBasis && basisCode!=noBasis ) {
+     double dGamma = param2;
      if (dGamma==0) basisType = expBasis;
   }
 
   if (basisType==none || ((basisType==expBasis || basisType==cosBasis) && tau==0.)) {
-    Double_t xprime = (x-(mean*msf))/(sigma*ssf) ;
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 1st form" << endl ;
-
-    Double_t result = std::exp(-0.5*xprime*xprime)/(sigma*ssf*root2pi) ;
-    if (_basisCode!=0 && basisSign==Both) result *= 2 ;
+    double xprime = (x-mean)/sigma ;
+    double result = std::exp(-0.5*xprime*xprime)/(sigma*root2pi) ;
+    if (basisCode!=0 && basisSign==Both) result *= 2 ;
     return result ;
   }
 
   // *** 2nd form: 0, used for sinBasis, linBasis, and quadBasis with tau=0 ***
   if (tau==0) {
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 2nd form" << endl ;
     return 0. ;
   }
 
   // *** 3nd form: Convolution with exp(-t/tau), used for expBasis and cosBasis(omega=0) ***
-  Double_t omega =  (basisType==sinBasis  || basisType==cosBasis)  ? ((RooAbsReal*)basis().getParameter(2))->getVal() : 0 ;
-  Double_t dgamma = (basisType==sinhBasis || basisType==coshBasis) ? ((RooAbsReal*)basis().getParameter(2))->getVal() : 0 ;
-  Double_t _x = omega *tau ;
-  Double_t _y = tau*dgamma/2;
-  Double_t xprime = (x-(mean*msf))/tau ;
-  Double_t c = (sigma*ssf)/(root2*tau) ;
-  Double_t u = xprime/(2*c) ;
+  double omega =  (basisType==sinBasis  || basisType==cosBasis)  ? param2 : 0 ;
+  double dgamma = (basisType==sinhBasis || basisType==coshBasis) ? param2 : 0 ;
+  double _x = omega *tau ;
+  double _y = tau*dgamma/2;
+  double xprime = (x-mean)/tau ;
+  double c = sigma/(root2*tau) ;
+  double u = xprime/(2*c) ;
 
   if (basisType==expBasis || (basisType==cosBasis && _x==0.)) {
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 3d form tau=" << tau << endl ;
-    Double_t result(0) ;
+    double result(0) ;
     if (basisSign!=Minus) result += evalCerf(0,-u,c).real();
     if (basisSign!=Plus)  result += evalCerf(0, u,c).real();
-    if (TMath::IsNaN(result)) { cxcoutE(Tracing) << "RooGaussModel::getVal(" << GetName() << ") got nan during case 1 " << endl; }
     return result ;
   }
 
   // *** 4th form: Convolution with exp(-t/tau)*sin(omega*t), used for sinBasis(omega<>0,tau<>0) ***
   if (basisType==sinBasis) {
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 4th form omega = " << omega << ", tau = " << tau << endl ;
-    Double_t result(0) ;
+    double result(0) ;
     if (_x==0.) return result ;
     if (basisSign!=Minus) result += -evalCerf(-_x,-u,c).imag();
     if (basisSign!=Plus)  result += -evalCerf( _x, u,c).imag();
-    if (TMath::IsNaN(result)) cxcoutE(Tracing) << "RooGaussModel::getVal(" << GetName() << ") got nan during case 3 " << endl;
     return result ;
   }
 
   // *** 5th form: Convolution with exp(-t/tau)*cos(omega*t), used for cosBasis(omega<>0) ***
   if (basisType==cosBasis) {
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 5th form omega = " << omega << ", tau = " << tau << endl ;
-    Double_t result(0) ;
+    double result(0) ;
     if (basisSign!=Minus) result += evalCerf(-_x,-u,c).real();
     if (basisSign!=Plus)  result += evalCerf( _x, u,c).real();
-    if (TMath::IsNaN(result)) cxcoutE(Tracing) << "RooGaussModel::getVal(" << GetName() << ") got nan during case 4 " << endl;
     return result ;
   }
 
   // ***8th form: Convolution with exp(-|t|/tau)*cosh(dgamma*t/2), used for         coshBasisSum ***
   if (basisType==coshBasis || basisType ==sinhBasis) {
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 8th form tau = " << tau << endl ;
-    Double_t result(0);
+    double result(0);
     int sgn = ( basisType == coshBasis ? +1 : -1 );
     if (basisSign!=Minus) result += 0.5*(    evalCerf(0,-u,c*(1-_y)).real()+sgn*evalCerf(0,-u,c*(1+_y)).real()) ;
     if (basisSign!=Plus)  result += 0.5*(sgn*evalCerf(0, u,c*(1-_y)).real()+    evalCerf(0, u,c*(1+_y)).real()) ;
-    if (TMath::IsNaN(result)) cxcoutE(Tracing) << "RooGaussModel::getVal(" << GetName() << ") got nan during case 8 " << endl;
     return result ;
   }
 
   // *** 6th form: Convolution with (t/tau)*exp(-t/tau), used for linBasis ***
   if (basisType==linBasis) {
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 6th form tau = " << tau << endl ;
     R__ASSERT(basisSign==Plus);  // This should only be for positive times
 
-    Double_t f0 = std::exp(-xprime+c*c) * RooMath::erfc(-u+c);
-    Double_t f1 = std::exp(-u*u);
+    double f0 = std::exp(-xprime+c*c) * RooMath::erfc(-u+c);
+    double f1 = std::exp(-u*u);
     return (xprime - 2*c*c)*f0 + (2*c/rootpi)*f1 ;
   }
 
   // *** 7th form: Convolution with (t/tau)^2*exp(-t/tau), used for quadBasis ***
   if (basisType==quadBasis) {
-    if (verboseEval()>2) cout << "RooGaussModel::evaluate(" << GetName() << ") 7th form tau = " << tau << endl ;
     R__ASSERT(basisSign==Plus);  // This should only be for positive times
 
-    Double_t f0 = std::exp(-xprime+c*c) * RooMath::erfc(-u+c);
-    Double_t f1 = std::exp(-u*u);
-    Double_t x2c2 = xprime - 2*c*c;
+    double f0 = std::exp(-xprime+c*c) * RooMath::erfc(-u+c);
+    double f1 = std::exp(-u*u);
+    double x2c2 = xprime - 2*c*c;
     return ( x2c2*x2c2*f0 + (2*c/rootpi)*x2c2*f1 + 2*c*c*f0 );
   }
 
@@ -318,12 +327,12 @@ Int_t RooGaussModel::getAnalyticalIntegral(RooArgSet& allVars, RooArgSet& analVa
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) const
+double RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) const
 {
-  static const Double_t root2 = std::sqrt(2.) ;
-  //static Double_t rootPiBy2 = std::sqrt(std::atan2(0.0,-1.0)/2.0);
-  static const Double_t rootpi = std::sqrt(std::atan2(0.0,-1.0));
-  Double_t ssfInt(1.0) ;
+  static const double root2 = std::sqrt(2.) ;
+  //static double rootPiBy2 = std::sqrt(std::atan2(0.0,-1.0)/2.0);
+  static const double rootpi = std::sqrt(std::atan2(0.0,-1.0));
+  double ssfInt(1.0) ;
 
   // Code must be 1 or 2
   R__ASSERT(code==1||code==2) ;
@@ -333,19 +342,19 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   BasisSign basisSign = (BasisSign)( _basisCode - 10*(basisType-1) - 2 ) ;
 
   // *** 1st form: Straight Gaussian, used for unconvoluted PDF or expBasis with 0 lifetime ***
-  Double_t tau = (_basisCode!=noBasis)?((RooAbsReal*)basis().getParameter(1))->getVal():0 ;
+  double tau = (_basisCode!=noBasis)?((RooAbsReal*)basis().getParameter(1))->getVal():0 ;
   if (basisType == coshBasis && _basisCode!=noBasis ) {
-     Double_t dGamma = ((RooAbsReal*)basis().getParameter(2))->getVal();
+     double dGamma = ((RooAbsReal*)basis().getParameter(2))->getVal();
      if (dGamma==0) basisType = expBasis;
   }
   if (basisType==none || ((basisType==expBasis || basisType==cosBasis) && tau==0.)) {
-    Double_t xscale = root2*(sigma*ssf);
+    double xscale = root2*(sigma*ssf);
     if (verboseEval()>0) cout << "RooGaussModel::analyticalIntegral(" << GetName() << ") 1st form" << endl ;
 
-    Double_t xpmin = (x.min(rangeName)-(mean*msf))/xscale ;
-    Double_t xpmax = (x.max(rangeName)-(mean*msf))/xscale ;
+    double xpmin = (x.min(rangeName)-(mean*msf))/xscale ;
+    double xpmax = (x.max(rangeName)-(mean*msf))/xscale ;
 
-    Double_t result ;
+    double result ;
     if (_asympInt) { // modified FMV, 07/24/03
         result = 1.0 ;
     } else {
@@ -359,8 +368,8 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   }
 
 
-  Double_t omega = ((basisType==sinBasis)||(basisType==cosBasis)) ?  ((RooAbsReal*)basis().getParameter(2))->getVal() : 0 ;
-  Double_t dgamma =((basisType==sinhBasis)||(basisType==coshBasis)) ?  ((RooAbsReal*)basis().getParameter(2))->getVal() : 0 ;
+  double omega = ((basisType==sinBasis)||(basisType==cosBasis)) ?  ((RooAbsReal*)basis().getParameter(2))->getVal() : 0 ;
+  double dgamma =((basisType==sinhBasis)||(basisType==coshBasis)) ?  ((RooAbsReal*)basis().getParameter(2))->getVal() : 0 ;
 
   // *** 2nd form: unity, used for sinBasis and linBasis with tau=0 (PDF is zero) ***
   if (tau==0) {
@@ -369,15 +378,15 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   }
 
   // *** 3rd form: Convolution with exp(-t/tau), used for expBasis and cosBasis(omega=0) ***
-  Double_t c = (sigma*ssf)/(root2*tau) ;
-  Double_t xpmin = (x.min(rangeName)-(mean*msf))/tau ;
-  Double_t xpmax = (x.max(rangeName)-(mean*msf))/tau ;
-  Double_t umin = xpmin/(2*c) ;
-  Double_t umax = xpmax/(2*c) ;
+  double c = (sigma*ssf)/(root2*tau) ;
+  double xpmin = (x.min(rangeName)-(mean*msf))/tau ;
+  double xpmax = (x.max(rangeName)-(mean*msf))/tau ;
+  double umin = xpmin/(2*c) ;
+  double umax = xpmax/(2*c) ;
 
   if (basisType==expBasis || (basisType==cosBasis && omega==0.)) {
     if (verboseEval()>0) cout << "RooGaussModel::analyticalIntegral(" << GetName() << ") 3d form tau=" << tau << endl ;
-    Double_t result(0) ;
+    double result(0) ;
     if (basisSign!=Minus) result += evalCerfInt(+1,0,tau,-umin,-umax,c).real();
     if (basisSign!=Plus)  result += evalCerfInt(-1,0,tau, umin, umax,c).real();
     if (TMath::IsNaN(result)) { cxcoutE(Tracing) << "RooGaussModel::analyticalIntegral(" << GetName() << ") got nan during case 3 " << endl; }
@@ -385,12 +394,12 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   }
 
   // *** 4th form: Convolution with exp(-t/tau)*sin(omega*t), used for sinBasis(omega<>0,tau<>0) ***
-  Double_t _x = omega * tau ;
-  Double_t _y = tau*dgamma/2;
+  double _x = omega * tau ;
+  double _y = tau*dgamma/2;
 
   if (basisType==sinBasis) {
     if (verboseEval()>0) cout << "RooGaussModel::analyticalIntegral(" << GetName() << ") 4th form omega = " << omega << ", tau = " << tau << endl ;
-    Double_t result(0) ;
+    double result(0) ;
     if (_x==0) return result*ssfInt ;
     if (basisSign!=Minus) result += -1*evalCerfInt(+1,-_x,tau,-umin,-umax,c).imag();
     if (basisSign!=Plus)  result += -1*evalCerfInt(-1, _x,tau, umin, umax,c).imag();
@@ -401,7 +410,7 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   // *** 5th form: Convolution with exp(-t/tau)*cos(omega*t), used for cosBasis(omega<>0) ***
   if (basisType==cosBasis) {
     if (verboseEval()>0) cout << "RooGaussModel::analyticalIntegral(" << GetName() << ") 5th form omega = " << omega << ", tau = " << tau << endl ;
-    Double_t result(0) ;
+    double result(0) ;
     if (basisSign!=Minus) result += evalCerfInt(+1,-_x,tau,-umin,-umax,c).real();
     if (basisSign!=Plus)  result += evalCerfInt(-1, _x,tau, umin, umax,c).real();
     if (TMath::IsNaN(result)) { cxcoutE(Tracing) << "RooGaussModel::analyticalIntegral(" << GetName() << ") got nan during case 5 " << endl; }
@@ -412,7 +421,7 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   // *** 9th form: Convolution with exp(-|t|/tau)*sinh(dgamma*t/2), used for sinhBasis ***
   if (basisType==coshBasis || basisType == sinhBasis) {
     if (verboseEval()>0) {cout << "RooGaussModel::analyticalIntegral(" << GetName()                             << ") 8th form tau=" << tau << endl ; }
-    Double_t result(0) ;
+    double result(0) ;
     int sgn = ( basisType == coshBasis ? +1 : -1 );
     if (basisSign!=Minus) result += 0.5*(    evalCerfInt(+1,0,tau/(1-_y),-umin,-umax,c*(1-_y)).real()+ sgn*evalCerfInt(+1,0,tau/(1+_y),-umin,-umax,c*(1+_y)).real());
     if (basisSign!=Plus)  result += 0.5*(sgn*evalCerfInt(-1,0,tau/(1-_y), umin, umax,c*(1-_y)).real()+     evalCerfInt(-1,0,tau/(1+_y), umin, umax,c*(1+_y)).real());
@@ -424,16 +433,16 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   if (basisType==linBasis) {
     if (verboseEval()>0) cout << "RooGaussModel::analyticalIntegral(" << GetName() << ") 6th form tau=" << tau << endl ;
 
-    Double_t f0 = RooMath::erf(-umax) - RooMath::erf(-umin);
-    Double_t f1 = std::exp(-umax*umax) - std::exp(-umin*umin);
+    double f0 = RooMath::erf(-umax) - RooMath::erf(-umin);
+    double f1 = std::exp(-umax*umax) - std::exp(-umin*umin);
 
-    Double_t tmp1 = std::exp(-xpmax)*RooMath::erfc(-umax + c);
-    Double_t tmp2 = std::exp(-xpmin)*RooMath::erfc(-umin + c);
+    double tmp1 = std::exp(-xpmax)*RooMath::erfc(-umax + c);
+    double tmp2 = std::exp(-xpmin)*RooMath::erfc(-umin + c);
 
-    Double_t f2 = tmp1 - tmp2;
-    Double_t f3 = xpmax*tmp1 - xpmin*tmp2;
+    double f2 = tmp1 - tmp2;
+    double f3 = xpmax*tmp1 - xpmin*tmp2;
 
-    Double_t expc2 = std::exp(c*c);
+    double expc2 = std::exp(c*c);
 
     return -tau*(              f0 +
         (2*c/rootpi)*f1 +
@@ -446,22 +455,22 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
   if (basisType==quadBasis) {
     if (verboseEval()>0) cout << "RooGaussModel::analyticalIntegral(" << GetName() << ") 7th form tau=" << tau << endl ;
 
-    Double_t f0 = RooMath::erf(-umax) - RooMath::erf(-umin);
+    double f0 = RooMath::erf(-umax) - RooMath::erf(-umin);
 
-    Double_t tmpA1 = std::exp(-umax*umax);
-    Double_t tmpA2 = std::exp(-umin*umin);
+    double tmpA1 = std::exp(-umax*umax);
+    double tmpA2 = std::exp(-umin*umin);
 
-    Double_t f1 = tmpA1 - tmpA2;
-    Double_t f2 = umax*tmpA1 - umin*tmpA2;
+    double f1 = tmpA1 - tmpA2;
+    double f2 = umax*tmpA1 - umin*tmpA2;
 
-    Double_t tmpB1 = std::exp(-xpmax)*RooMath::erfc(-umax + c);
-    Double_t tmpB2 = std::exp(-xpmin)*RooMath::erfc(-umin + c);
+    double tmpB1 = std::exp(-xpmax)*RooMath::erfc(-umax + c);
+    double tmpB2 = std::exp(-xpmin)*RooMath::erfc(-umin + c);
 
-    Double_t f3 = tmpB1 - tmpB2;
-    Double_t f4 = xpmax*tmpB1 - xpmin*tmpB2;
-    Double_t f5 = xpmax*xpmax*tmpB1 - xpmin*xpmin*tmpB2;
+    double f3 = tmpB1 - tmpB2;
+    double f4 = xpmax*tmpB1 - xpmin*tmpB2;
+    double f5 = xpmax*xpmax*tmpB1 - xpmin*xpmin*tmpB2;
 
-    Double_t expc2 = std::exp(c*c);
+    double expc2 = std::exp(c*c);
 
     return -tau*( 2*f0 +
         (4*c/rootpi)*((1-c*c)*f1 + c*f2) +
@@ -475,23 +484,23 @@ Double_t RooGaussModel::analyticalIntegral(Int_t code, const char* rangeName) co
 
 ////////////////////////////////////////////////////////////////////////////////
 
-std::complex<Double_t> RooGaussModel::evalCerfInt(Double_t sign, Double_t _x, Double_t tau, Double_t umin, Double_t umax, Double_t c) const
+std::complex<double> RooGaussModel::evalCerfInt(double sign, double _x, double tau, double umin, double umax, double c) const
 {
-  std::complex<Double_t> diff(2., 0.);
+  std::complex<double> diff(2., 0.);
   if (!_asympInt) {
     diff = evalCerf(_x,umin,c);
     diff -= evalCerf(_x,umax,c);
     diff += RooMath::erf(umin) - RooMath::erf(umax);
     diff *= sign;
   }
-  diff *= std::complex<Double_t>(1., _x);
+  diff *= std::complex<double>(1., _x);
   diff *= tau / (1.+_x*_x);
   return diff;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 
-Int_t RooGaussModel::getGenerator(const RooArgSet& directVars, RooArgSet &generateVars, Bool_t /*staticInitOK*/) const
+Int_t RooGaussModel::getGenerator(const RooArgSet& directVars, RooArgSet &generateVars, bool /*staticInitOK*/) const
 {
   if (matchArgs(directVars,generateVars,x)) return 1 ;
   return 0 ;
@@ -502,11 +511,11 @@ Int_t RooGaussModel::getGenerator(const RooArgSet& directVars, RooArgSet &genera
 void RooGaussModel::generateEvent(Int_t code)
 {
   R__ASSERT(code==1) ;
-  Double_t xmin = x.min();
-  Double_t xmax = x.max();
+  double xmin = x.min();
+  double xmax = x.max();
   TRandom *generator = RooRandom::randomGenerator();
   while(true) {
-    Double_t xgen = generator->Gaus(mean*msf,sigma*ssf);
+    double xgen = generator->Gaus(mean*msf,sigma*ssf);
     if (xgen<xmax && xgen>xmin) {
       x = xgen ;
       return ;
