@@ -36,10 +36,11 @@ implementations like RooAcceptReject and RooFoam
 #include "RooMsgService.h"
 #include "RooRealBinding.h"
 
-#include <assert.h>
+#include <cassert>
 
 using namespace std;
 
+RooAbsNumGenerator::RooAbsNumGenerator() = default;
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -49,26 +50,25 @@ using namespace std;
 /// cloned and so will not be disturbed during the generation process.
 
 RooAbsNumGenerator::RooAbsNumGenerator(const RooAbsReal &func, const RooArgSet &genVars, bool verbose, const RooAbsReal* maxFuncVal) :
-  _cloneSet(0), _funcClone(0), _funcMaxVal(maxFuncVal), _verbose(verbose), _funcValStore(0), _funcValPtr(0), _cache(0)
+  _funcMaxVal(maxFuncVal), _verbose(verbose)
 {
   // Clone the function and all nodes that it depends on so that this generator
   // is independent of any existing objects.
   RooArgSet nodes(func,func.GetName());
-  _cloneSet= (RooArgSet*) nodes.snapshot(true);
-  if (!_cloneSet) {
+  if (nodes.snapshot(_cloneSet, true)) {
     oocoutE(nullptr, Generation) << "RooAbsNumGenerator::RooAbsNumGenerator(" << func.GetName() << ") Couldn't deep-clone function, abort," << endl ;
     RooErrorHandler::softAbort() ;
   }
 
   // Find the clone in the snapshot list
-  _funcClone = (RooAbsReal*)_cloneSet->find(func.GetName());
+  _funcClone = (RooAbsReal*)_cloneSet.find(func.GetName());
 
 
   // Check that each argument is fundamental, and separate them into
   // sets of categories and reals. Check that the area of the generating
   // space is finite.
   _isValid= true;
-  const RooAbsArg *found = 0;
+  const RooAbsArg *found = nullptr;
   for (RooAbsArg const* arg : genVars) {
     if(!arg->isFundamental()) {
       oocoutE(nullptr, Generation) << func.GetName() << "::RooAbsNumGenerator: cannot generate values for derived \""
@@ -77,21 +77,21 @@ RooAbsNumGenerator::RooAbsNumGenerator(const RooAbsReal &func, const RooArgSet &
       continue;
     }
     // look for this argument in the generating function's dependents
-    found= (const RooAbsArg*)_cloneSet->find(arg->GetName());
+    found= (const RooAbsArg*)_cloneSet.find(arg->GetName());
     if(found) {
       arg= found;
     } else {
       // clone any variables we generate that we haven't cloned already
-      arg= _cloneSet->addClone(*arg);
+      arg= _cloneSet.addClone(*arg);
     }
-    assert(0 != arg);
+    assert(nullptr != arg);
     // is this argument a category or a real?
     const RooCategory *catVar= dynamic_cast<const RooCategory*>(arg);
     const RooRealVar *realVar= dynamic_cast<const RooRealVar*>(arg);
-    if(0 != catVar) {
+    if(nullptr != catVar) {
       _catVars.add(*catVar);
     }
-    else if(0 != realVar) {
+    else if(nullptr != realVar) {
       if(realVar->hasMin() && realVar->hasMax()) {
    _realVars.add(*realVar);
       }
@@ -113,19 +113,17 @@ RooAbsNumGenerator::RooAbsNumGenerator(const RooAbsReal &func, const RooArgSet &
   }
 
   // create a fundamental type for storing function values
-  _funcValStore= dynamic_cast<RooRealVar*>(_funcClone->createFundamental());
-  assert(0 != _funcValStore);
+  _funcValStore= std::unique_ptr<RooAbsArg>{_funcClone->createFundamental()};
 
   // create a new dataset to cache trial events and function values
   RooArgSet cacheArgs(_catVars);
   cacheArgs.add(_realVars);
   cacheArgs.add(*_funcValStore);
-  _cache= new RooDataSet("cache","Accept-Reject Event Cache",cacheArgs);
-  assert(0 != _cache);
+  _cache= std::make_unique<RooDataSet>("cache","Accept-Reject Event Cache",cacheArgs);
 
   // attach our function clone to the cache dataset
   const RooArgSet *cacheVars= _cache->get();
-  assert(0 != cacheVars);
+  assert(nullptr != cacheVars);
   _funcClone->recursiveRedirectServers(*cacheVars,false);
 
   // update ours sets of category and real args to refer to the cache dataset
@@ -139,17 +137,7 @@ RooAbsNumGenerator::RooAbsNumGenerator(const RooAbsReal &func, const RooArgSet &
 }
 
 
-
-////////////////////////////////////////////////////////////////////////////////
-/// Destructor
-
-RooAbsNumGenerator::~RooAbsNumGenerator()
-{
-  delete _cloneSet;
-  delete _cache ;
-  delete _funcValStore ;
-}
-
+RooAbsNumGenerator::~RooAbsNumGenerator() = default;
 
 
 ////////////////////////////////////////////////////////////////////////////////
