@@ -18,7 +18,7 @@
 /** \class RooAddPdf
     \ingroup Roofitcore
 
-RooAddPdf is an efficient implementation of a sum of PDFs of the form
+Efficient implementation of a sum of PDFs of the form
 
 \f[
  \sum_{i=1}^{n} c_i \cdot \mathrm{PDF}_i
@@ -63,28 +63,32 @@ An (enforced) condition for this assumption is that each \f$ \mathrm{PDF}_i \f$
 
 */
 
-#include "RooAddPdf.h"
+#include <RooAddPdf.h>
+
+#include <RooAddGenContext.h>
+#include <RooAddition.h>
+#include <RooBatchCompute.h>
+#include <RooDataSet.h>
+#include <RooGenericPdf.h>
+#include <RooGlobalFunc.h>
+#include <RooProduct.h>
+#include <RooRatio.h>
+#include <RooRealConstant.h>
+#include <RooRealProxy.h>
+#include <RooRealSumFunc.h>
+#include <RooRealSumPdf.h>
+#include <RooRealVar.h>
+#include <RooRecursiveFraction.h>
 
-#include "RooAddition.h"
-#include "RooRealSumFunc.h"
 #include "RooAddHelpers.h"
-#include "RooAddGenContext.h"
-#include "RooBatchCompute.h"
-#include "RooDataSet.h"
-#include "RooGlobalFunc.h"
-#include "RooRealProxy.h"
-#include "RooRealVar.h"
-#include "RooRealConstant.h"
-#include "RooRealSumPdf.h"
-#include "RooRecursiveFraction.h"
-#include "RooGenericPdf.h"
-#include "RooProduct.h"
-#include "RooRatio.h"
+#include "RooFitImplHelpers.h"
+
+#include <ROOT/StringUtils.hxx>
 
 #include <algorithm>
 #include <memory>
-#include <sstream>
 #include <set>
+#include <sstream>
 
 ClassImp(RooAddPdf);
 
@@ -100,7 +104,7 @@ RooAddPdf::RooAddPdf(const char *name, const char *title) :
   _coefList("!coefficients","List of coefficients",this),
   _coefErrCount{_errorCount}
 {
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -296,7 +300,7 @@ RooAddPdf::RooAddPdf(const RooAddPdf& other, const char* name) :
 {
   _coefErrCount = _errorCount ;
   finalizeConstruction();
-  TRACE_CREATE
+  TRACE_CREATE;
 }
 
 
@@ -316,12 +320,53 @@ void RooAddPdf::fixCoefNormalization(const RooArgSet& refCoefNorm)
     return ;
   }
 
+   // Also set an attribute with this information, which is the easiest way to
+  // preserve this in the JSON IO.
+  setStringAttribute("ref_coef_norm", RooHelpers::getColonSeparatedNameString(refCoefNorm, ',').c_str());
+
   _refCoefNorm.removeAll() ;
   _refCoefNorm.add(refCoefNorm) ;
 
   _projCacheMgr.reset() ;
 }
 
+const RooArgSet &RooAddPdf::getCoefNormalization() const
+{
+   materializeRefCoefNormFromAttribute();
+   return _refCoefNorm;
+}
+
+// For the JSON IO, we are not storing the _refCoefNorm directly. Instead, it
+// is stored by names in a string attribute. This function should be called
+// internally before _refCoefNorm is used to materialize it from the attribute
+// if necessary.
+void RooAddPdf::materializeRefCoefNormFromAttribute() const
+{
+   // _refCoefNorm was already materialized
+   if (!_refCoefNorm.empty())
+      return;
+
+   std::vector<std::string> names;
+   if (auto attrib = getStringAttribute("ref_coef_norm")) {
+      names = ROOT::Split(attrib, ",", /*skipEmpty=*/true);
+   } else {
+      return;
+   }
+
+   RooArgSet refCoefNorm;
+
+   RooArgSet serverSet;
+   RooHelpers::getSortedComputationGraph(*this, serverSet);
+   for (std::string const &name : names) {
+      if (RooAbsArg *arg = serverSet.find(name.c_str())) {
+         refCoefNorm.add(*arg);
+      } else {
+         throw std::runtime_error("Internal logic error in RooAddPdf::materializeRefCoefNormFromAttribute()");
+      }
+   }
+
+   const_cast<RooAddPdf *>(this)->fixCoefNormalization(refCoefNorm);
+}
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -360,18 +405,18 @@ void RooAddPdf::fixCoefRange(const char* rangeName)
 AddCacheElem* RooAddPdf::getProjCache(const RooArgSet* nset, const RooArgSet* iset) const
 {
   // Check if cache already exists
-  auto cache = static_cast<AddCacheElem*>(_projCacheMgr.getObj(nset,iset,0,normRange()));
+  auto cache = static_cast<AddCacheElem*>(_projCacheMgr.getObj(nset,iset,nullptr,normRange()));
   if (cache) {
     return cache ;
   }
 
+  // Make sure _refCoefNorm is defined
+  materializeRefCoefNormFromAttribute();
+
   //Create new cache
   cache = new AddCacheElem{*this, _pdfList, _coefList, nset, iset, _refCoefNorm,
                            _refCoefRangeName ? RooNameReg::str(_refCoefRangeName) : "",
                            _verboseEval};
-  //std::cout << std::endl;
-  //cache->print();
-  //std::cout << std::endl;
 
   _projCacheMgr.setObj(nset,iset,cache,RooNameReg::ptr(normRange())) ;
 
@@ -415,6 +460,9 @@ std::pair<const RooArgSet*, AddCacheElem*> RooAddPdf::getNormAndCache(const RooA
   if(nset && nset->empty()) nset = nullptr;
 
   if (nset == nullptr) {
+    // Make sure _refCoefNorm is defined
+    materializeRefCoefNormFromAttribute();
+
     if (!_refCoefNorm.empty()) {
       nset = &_refCoefNorm ;
     }
@@ -472,7 +520,7 @@ double RooAddPdf::getValV(const RooArgSet* normSet) const
 
   // Process change in last data set used
   bool nsetChanged(false) ;
-  if (!isActiveNormSet(nset) || _norm==0) {
+  if (!isActiveNormSet(nset) || _norm==nullptr) {
     nsetChanged = syncNormalization(nset) ;
   }
 
@@ -496,11 +544,17 @@ double RooAddPdf::getValV(const RooArgSet* normSet) const
   return _value;
 }
 
+void RooAddPdf::translate(RooFit::Detail::CodeSquashContext &ctx) const
+{
+   RooRealSumPdf::translateImpl(ctx, this, _pdfList, _coefList);
+}
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Compute addition of PDFs in batches.
-void RooAddPdf::computeBatch(cudaStream_t* stream, double* output, size_t nEvents, RooFit::Detail::DataMap const& dataMap) const
+void RooAddPdf::computeBatch(double* output, size_t nEvents, RooFit::Detail::DataMap const& dataMap) const
 {
+  RooBatchCompute::Config config = dataMap.config(this);
+
   _coefCache.resize(_pdfList.size());
   for(std::size_t i = 0; i < _coefList.size(); ++i) {
     auto coefVals = dataMap.at(&_coefList[i]);
@@ -509,10 +563,10 @@ void RooAddPdf::computeBatch(cudaStream_t* stream, double* output, size_t nEvent
     // With CUDA, we can't do that because the inputs might be on the device.
     // That's why we throw an exception then.
     if(coefVals.size() > 1) {
-      if(stream) {
+      if (config.useCuda()) {
         throw std::runtime_error("The RooAddPdf doesn't support per-event coefficients in CUDA mode yet!");
       }
-      RooAbsReal::computeBatch(stream, output, nEvents, dataMap);
+      RooAbsReal::computeBatch(output, nEvents, dataMap);
       return;
     }
     _coefCache[i] = coefVals[0];
@@ -520,12 +574,10 @@ void RooAddPdf::computeBatch(cudaStream_t* stream, double* output, size_t nEvent
 
   RooBatchCompute::VarVector pdfs;
   RooBatchCompute::ArgVector coefs;
-  auto normAndCache = getNormAndCache(nullptr);
-  const RooArgSet* nset = normAndCache.first;
-  AddCacheElem* cache = normAndCache.second;
+  AddCacheElem* cache = getProjCache(nullptr);
   // We don't sync the coefficient values from the _coefList to the _coefCache
   // because we have already done it using the dataMap.
-  updateCoefficients(*cache, nset, /*syncCoefValues=*/false);
+  updateCoefficients(*cache, nullptr, /*syncCoefValues=*/false);
 
   for (unsigned int pdfNo = 0; pdfNo < _pdfList.size(); ++pdfNo)
   {
@@ -536,8 +588,7 @@ void RooAddPdf::computeBatch(cudaStream_t* stream, double* output, size_t nEvent
       coefs.push_back(_coefCache[pdfNo] / cache->suppNormVal(pdfNo) );
     }
   }
-  auto dispatch = stream ? RooBatchCompute::dispatchCUDA : RooBatchCompute::dispatchCPU;
-  dispatch->compute(stream, RooBatchCompute::AddPdf, output, nEvents, pdfs, coefs);
+  RooBatchCompute::compute(config, RooBatchCompute::AddPdf, output, nEvents, pdfs, coefs);
 }
 
 
@@ -555,7 +606,7 @@ void RooAddPdf::resetErrorCounters(Int_t resetValue)
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Check if PDF is valid for given normalization set.
-/// Coeffient and PDF must be non-overlapping, but pdf-coefficient
+/// Coefficient and PDF must be non-overlapping, but pdf-coefficient
 /// pairs may overlap each other
 
 bool RooAddPdf::checkObservables(const RooArgSet* nset) const
@@ -577,6 +628,8 @@ bool RooAddPdf::checkObservables(const RooArgSet* nset) const
 Int_t RooAddPdf::getAnalyticalIntegralWN(RooArgSet& allVars, RooArgSet& analVars,
                 const RooArgSet* normSet, const char* rangeName) const
 {
+  // Make sure _refCoefNorm is defined
+  materializeRefCoefNormFromAttribute();
 
   RooArgSet allAnalVars(*std::unique_ptr<RooArgSet>{getObservables(allVars)}) ;
 
@@ -658,7 +711,7 @@ double RooAddPdf::analyticalIntegralWN(Int_t code, const RooArgSet* normSet, con
 
   cxcoutD(Caching) << "RooAddPdf::aiWN(" << GetName() << ") calling getProjCache with nset = " << (normSet?*normSet:RooArgSet()) << std::endl ;
 
-  if ((normSet==0 || normSet->empty()) && !_refCoefNorm.empty()) {
+  if ((normSet==nullptr || normSet->empty()) && !_refCoefNorm.empty()) {
 //     cout << "WVE integration of RooAddPdf without normalization, but have reference set, using ref set for normalization" << std::endl ;
     normSet = &_refCoefNorm ;
   }
@@ -749,6 +802,9 @@ std::unique_ptr<RooAbsReal> RooAddPdf::createExpectedEventsFunc(const RooArgSet
 
    RooArgList prodList;
 
+   // Make sure _refCoefNorm is defined
+   materializeRefCoefNormFromAttribute();
+
    if (!_allExtendable) {
       // If the _refCoefNorm is empty or it's equal to normSet anyway, this is not
       // a conditional pdf and we don't need to do any transformation. See also
@@ -759,7 +815,7 @@ std::unique_ptr<RooAbsReal> RooAddPdf::createExpectedEventsFunc(const RooArgSet
 
       // Optionally multiply with fractional normalization. I this case, we
       // replace the original factor stored in "out".
-      if (_normRange) {
+      if (!_normRange.IsNull()) {
          std::unique_ptr<RooAbsReal> owner;
          RooArgList terms;
          // The integrals own each other in a chain. We do this because it's
@@ -803,6 +859,8 @@ std::unique_ptr<RooAbsReal> RooAddPdf::createExpectedEventsFunc(const RooArgSet
 
 void RooAddPdf::selectNormalization(const RooArgSet* depSet, bool force)
 {
+  // Make sure _refCoefNorm is defined
+  materializeRefCoefNormFromAttribute();
 
   if (!force && !_refCoefNorm.empty()) {
     return ;
@@ -904,11 +962,14 @@ bool RooAddPdf::redirectServersHook(const RooAbsCollection & newServerList, bool
 std::unique_ptr<RooAbsArg>
 RooAddPdf::compileForNormSet(RooArgSet const &normSet, RooFit::Detail::CompileContext &ctx) const
 {
+   // Make sure _refCoefNorm is defined
+   materializeRefCoefNormFromAttribute();
+
    auto newArg = std::unique_ptr<RooAbsReal>{static_cast<RooAbsReal *>(Clone())};
    ctx.markAsCompiled(*newArg);
 
    // In case conditional observables, e.g. p(x|y), the _refCoefNorm is set to
-   // all observables (x, y) and the normSet doesn't contain the condidional
+   // all observables (x, y) and the normSet doesn't contain the conditional
    // observables (so it only contains x in this example).
 
    // If the _refCoefNorm is empty or it's equal to normSet anyway, this is not
@@ -929,13 +990,13 @@ RooAddPdf::compileForNormSet(RooArgSet const &normSet, RooFit::Detail::CompileCo
    //   1. p(x, y) = p(x | y) * p(y)
    //   2. p(y) = Integral of p(x, y) over x
    //
-   // We conculde:
+   // We conclude:
    //                      p(x, y)
    //   p(x | y) = --------------------------
    //              Integral of p(x, y) over x
    //
    // What follows is the implementation of this formula in RooFit. By doing
-   // this here in complieForNormSet(), we don't invoke the old RooAddPdf
+   // this here in compileForNormSet(), we don't invoke the old RooAddPdf
    // projection caches (note that no conditional pdfs are on the right hand
    // side of the equation).
    std::string finalName = std::string(GetName()) + "_conditional";
