@@ -19,8 +19,7 @@
 \class RooBinIntegrator
 \ingroup Roofitcore
 
-RooBinIntegrator computes the integral over a binned distribution by summing the bin
-contents of all bins.
+Computes the integral over a binned distribution by summing the bin contents of all bins.
 **/
 
 #include "RooBinIntegrator.h"
@@ -28,17 +27,16 @@ contents of all bins.
 #include "RooArgSet.h"
 #include "RooRealVar.h"
 #include "RooNumber.h"
-#include "RooIntegratorBinding.h"
 #include "RooNumIntConfig.h"
 #include "RooNumIntFactory.h"
 #include "RooMsgService.h"
-#include "RunContext.h"
 #include "RooRealBinding.h"
 
 #include "TClass.h"
 #include "Math/Util.h"
 
-#include <assert.h>
+#include <cassert>
+#include <memory>
 
 
 
@@ -54,19 +52,21 @@ ClassImp(RooBinIntegrator);
 
 void RooBinIntegrator::registerIntegrator(RooNumIntFactory& fact)
 {
-  RooRealVar numBins("numBins","Number of bins in range",100) ;
-  RooBinIntegrator* proto = new RooBinIntegrator() ;
-  fact.storeProtoIntegrator(proto,RooArgSet(numBins)) ;
-  RooNumIntConfig::defaultConfig().method1D().setLabel(proto->IsA()->GetName()) ;
-}
+   RooRealVar numBins("numBins","Number of bins in range",100) ;
 
+   std::string name = "RooBinIntegrator";
 
+   auto creator = [](const RooAbsFunc &function, const RooNumIntConfig &config) {
+      return std::make_unique<RooBinIntegrator>(function, config);
+   };
 
-////////////////////////////////////////////////////////////////////////////////
-/// Default constructor
+   fact.registerPlugin(name, creator, {numBins},
+                     /*canIntegrate1D=*/true,
+                     /*canIntegrate2D=*/true,
+                     /*canIntegrateND=*/true,
+                     /*canIntegrateOpenEnded=*/false);
 
-RooBinIntegrator::RooBinIntegrator() : _numBins(0), _useIntegrandLimits(kFALSE), _x(0)
-{
+  RooNumIntConfig::defaultConfig().method1D().setLabel(name);
 }
 
 
@@ -76,57 +76,36 @@ RooBinIntegrator::RooBinIntegrator() : _numBins(0), _useIntegrandLimits(kFALSE),
 RooBinIntegrator::RooBinIntegrator(const RooAbsFunc& function, int numBins):
   RooAbsIntegrator(function)
 {
-  _useIntegrandLimits= kTRUE;
+  _useIntegrandLimits= true;
   assert(_function && _function->isValid());
 
   // Allocate coordinate buffer size after number of function dimensions
-  _x = new Double_t[_function->getDimension()] ;
+  _x.resize(_function->getDimension());
   _numBins = numBins;
 
   _xmin.resize(_function->getDimension()) ;
   _xmax.resize(_function->getDimension()) ;
 
-  auto realBinding = dynamic_cast<const RooRealBinding*>(_function);
-
-  // We could use BatchMode for RooRealBindings as they implement getValues().
-  // However, this is not efficient right now, because every time getValue() is
-  // called, a new RooFitDriver is created. Needs to be refactored.
-
-  //const bool useBatchMode = realBinding;
-  const bool useBatchMode = false;
-
-  if (useBatchMode) {
-    _evalData.reset(new RooBatchCompute::RunContext());
-    _evalDataOrig.reset(new RooBatchCompute::RunContext());
-  }
-
   for (UInt_t i=0 ; i<_function->getDimension() ; i++) {
     _xmin[i]= _function->getMinLimit(i);
     _xmax[i]= _function->getMaxLimit(i);
 
     // Retrieve bin configuration from integrand
-    std::unique_ptr<list<Double_t>> tmp{ _function->binBoundaries(i) };
+    std::unique_ptr<list<double>> tmp{ _function->binBoundaries(i) };
     if (!tmp) {
-      oocoutW((TObject*)0,Integration) << "RooBinIntegrator::RooBinIntegrator WARNING: integrand provide no binning definition observable #" 
+      oocoutW(nullptr,Integration) << "RooBinIntegrator::RooBinIntegrator WARNING: integrand provide no binning definition observable #"
           << i << " substituting default binning of " << _numBins << " bins" << endl ;
-      tmp.reset( new list<Double_t> );
+      tmp = std::make_unique<list<double>>( );
       for (Int_t j=0 ; j<=_numBins ; j++) {
         tmp->push_back(_xmin[i]+j*(_xmax[i]-_xmin[i])/_numBins) ;
       }
     }
     _binb.emplace_back(tmp->begin(), tmp->end());
 
-    if (useBatchMode) {
-      const std::vector<double>& binb = _binb.back();
-      RooSpan<double> binCentres = _evalDataOrig->makeBatch(realBinding->observable(i), binb.size() - 1);
-      for (unsigned int ibin = 0; ibin < binb.size() - 1; ++ibin) {
-        binCentres[ibin] = (binb[ibin + 1] + binb[ibin]) / 2.;
-      }
-    }
   }
   checkLimits();
 
-} 
+}
 
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -138,37 +117,24 @@ RooBinIntegrator::RooBinIntegrator(const RooAbsFunc& function, const RooNumIntCo
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-/// Clone integrator with new function binding and configuration. Needed by RooNumIntFactory
-
-RooAbsIntegrator* RooBinIntegrator::clone(const RooAbsFunc& function, const RooNumIntConfig& config) const
-{
-  return new RooBinIntegrator(function,config) ;
-}
-
-
-
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Destructor
 
 RooBinIntegrator::~RooBinIntegrator()
 {
-  if(_x) delete[] _x;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Change our integration limits. Return kTRUE if the new limits are
-/// ok, or otherwise kFALSE. Always returns kFALSE and does nothing
+/// Change our integration limits. Return true if the new limits are
+/// ok, or otherwise false. Always returns false and does nothing
 /// if this object was constructed to always use our integrand's limits.
 
-Bool_t RooBinIntegrator::setLimits(Double_t *xmin, Double_t *xmax) 
+bool RooBinIntegrator::setLimits(double *xmin, double *xmax)
 {
   if(_useIntegrandLimits) {
-    oocoutE((TObject*)0,Integration) << "RooBinIntegrator::setLimits: cannot override integrand's limits" << endl;
-    return kFALSE;
+    oocoutE(nullptr,Integration) << "RooBinIntegrator::setLimits: cannot override integrand's limits" << endl;
+    return false;
   }
   _xmin[0]= *xmin;
   _xmax[0]= *xmax;
@@ -177,13 +143,13 @@ Bool_t RooBinIntegrator::setLimits(Double_t *xmin, Double_t *xmax)
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Check that our integration range is finite and otherwise return kFALSE.
+/// Check that our integration range is finite and otherwise return false.
 /// Update the limits from the integrand if requested.
 
-Bool_t RooBinIntegrator::checkLimits() const 
+bool RooBinIntegrator::checkLimits() const
 {
   if(_useIntegrandLimits) {
-    assert(0 != integrand() && integrand()->isValid());
+    assert(nullptr != integrand() && integrand()->isValid());
     _xmin.resize(_function->getDimension()) ;
     _xmax.resize(_function->getDimension()) ;
     for (UInt_t i=0 ; i<_function->getDimension() ; i++) {
@@ -193,21 +159,21 @@ Bool_t RooBinIntegrator::checkLimits() const
   }
   for (UInt_t i=0 ; i<_function->getDimension() ; i++) {
     if (_xmax[i]<=_xmin[i]) {
-      oocoutE((TObject*)0,Integration) << "RooBinIntegrator::checkLimits: bad range with min >= max (_xmin = " << _xmin[i] << " _xmax = " << _xmax[i] << ")" << endl;
-      return kFALSE;
+      oocoutE(nullptr,Integration) << "RooBinIntegrator::checkLimits: bad range with min >= max (_xmin = " << _xmin[i] << " _xmax = " << _xmax[i] << ")" << endl;
+      return false;
     }
     if (RooNumber::isInfinite(_xmin[i]) || RooNumber::isInfinite(_xmax[i])) {
-      return kFALSE ;
+      return false ;
     }
   }
-  
-  return kTRUE;
+
+  return true;
 }
 
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Calculate numeric integral at given set of function binding parameters.
-Double_t RooBinIntegrator::integral(const Double_t *) 
+double RooBinIntegrator::integral(const double *)
 {
   assert(isValid());
 
@@ -216,28 +182,12 @@ Double_t RooBinIntegrator::integral(const Double_t *)
   if (_function->getDimension() == 1) {
     const std::vector<double>& binb = _binb[0];
 
-    if (_evalData) {
-      // Real bindings support batch evaluations. Can fast track now.
-      auto realBinding = static_cast<const RooRealBinding*>(integrand());
-
-      // Reset computation results to only contain known bin centres, and keep all memory intact:
-      _evalData->spans = _evalDataOrig->spans;
-      auto results = realBinding->getValuesOfBoundFunction(*_evalData);
-      assert(results.size() == binb.size() - 1);
-
-      for (unsigned int ibin = 0; ibin < binb.size() - 1; ++ibin) {
-        const double width = binb[ibin + 1] - binb[ibin];
-        sum += results[ibin] * width;
-      }
-    } else {
-      // Need to use single-value interface
-      for (unsigned int ibin=0; ibin < binb.size() - 1; ++ibin) {
-        const double xhi = binb[ibin + 1];
-        const double xlo = binb[ibin];
-        const double xcenter = (xhi+xlo)/2.;
-        const double binInt = integrand(xvec(xcenter))*(xhi-xlo) ;
-        sum += binInt ;
-      }
+    for (unsigned int ibin=0; ibin < binb.size() - 1; ++ibin) {
+      const double xhi = binb[ibin + 1];
+      const double xlo = binb[ibin];
+      const double xcenter = (xhi+xlo)/2.;
+      const double binInt = integrand(xvec(xcenter))*(xhi-xlo) ;
+      sum += binInt ;
     }
   } else if (_function->getDimension() == 2) {
     const std::vector<double>& binbx = _binb[0];
@@ -246,8 +196,8 @@ Double_t RooBinIntegrator::integral(const Double_t *)
     for (unsigned int ibin1=0; ibin1 < binbx.size() - 1; ++ibin1) {
       const double x1hi = binbx[ibin1 + 1];
       const double x1lo = binbx[ibin1];
-      Double_t x1center = (x1hi+x1lo)/2 ;
-      
+      double x1center = (x1hi+x1lo)/2 ;
+
       for (unsigned int ibin2=0; ibin2 < binby.size() - 1; ++ibin2) {
         const double x2hi = binby[ibin2 + 1];
         const double x2lo = binby[ibin2];
@@ -265,7 +215,7 @@ Double_t RooBinIntegrator::integral(const Double_t *)
     for (unsigned int ibin1=0; ibin1 < binbx.size() - 1; ++ibin1) {
       const double x1hi = binbx[ibin1 + 1];
       const double x1lo = binbx[ibin1];
-      Double_t x1center = (x1hi+x1lo)/2 ;
+      double x1center = (x1hi+x1lo)/2 ;
 
       for (unsigned int ibin2=0; ibin2 < binby.size() - 1; ++ibin2) {
         const double x2hi = binby[ibin2 + 1];
@@ -284,7 +234,7 @@ Double_t RooBinIntegrator::integral(const Double_t *)
     }
   }
 
-  return sum;
+  return sum.Sum();
 }
 
 
