@@ -24,7 +24,7 @@
  * RooCustomizer supports two kinds of modifications:
  *
  * - replaceArg(leaf_arg, repl_arg):
- * Replaces each occurence of leaf_arg with repl_arg in the composite pdf.
+ * Replaces each occurrence of leaf_arg with repl_arg in the composite pdf.
  *
  * - splitArg(split_arg):
  * Build multiple clones of the same prototype. Each
@@ -87,7 +87,7 @@
  * ### Reuse nodes to customise a different PDF
  * By default, the customizer clones the prototype leaf node when splitting a leaf,
  * but the user can feed pre-defined split leafs in leaf list. These leafs
- * must have the name <split_leaf>_<splitcat_label> to be picked up. The list
+ * must have the name `<split_leaf>_<splitcat_label>` to be picked up. The list
  * of pre-supplied leafs may be partial, any missing split leafs will be auto
  * generated.
  *
@@ -149,6 +149,7 @@
 
 #include "RooCustomizer.h"
 
+#include "RooFactoryWSTool.h"
 #include "RooAbsCategoryLValue.h"
 #include "RooAbsCategory.h"
 #include "RooAbsArg.h"
@@ -175,18 +176,23 @@
 
 using namespace std;
 
-ClassImp(RooCustomizer); 
-;
 
 namespace {
 
+/// Factory interface
+class CustIFace : public RooFactoryWSTool::IFace {
+public:
+  std::string create(RooFactoryWSTool& ft, const char* typeName, const char* instanceName, std::vector<std::string> args) override ;
+} ;
+
+
 static Int_t init();
 
 Int_t dummy = init() ;
 
-static Int_t init()
+Int_t init()
 {
-  RooFactoryWSTool::IFace* iface = new RooCustomizer::CustIFace ;
+  RooFactoryWSTool::IFace* iface = new CustIFace ;
   RooFactoryWSTool::registerSpecial("EDIT",iface) ;
   (void) dummy;
   return 0 ;
@@ -214,17 +220,16 @@ static Int_t init()
 /// \param[in,out] splitLeafsAll All leafs that are used when customising are collected here.
 /// If this set already contains leaves, they will be used for customising if the names match
 /// as above.
-/// 
+///
 
 RooCustomizer::RooCustomizer(const RooAbsArg& pdf, const RooAbsCategoryLValue& masterCat,
     RooArgSet& splitLeafs, RooArgSet* splitLeafsAll) :
-  TNamed(pdf.GetName(),pdf.GetTitle()),
-  _sterile(kFALSE),
-  _owning(kTRUE),
-  _masterPdf((RooAbsArg*)&pdf), 
-  _masterCat((RooAbsCategoryLValue*)&masterCat), 
-  _masterBranchList("masterBranchList"), 
-  _masterLeafList("masterLeafList"), 
+  _sterile(false),
+  _owning(true),
+  _masterPdf((RooAbsArg*)&pdf),
+  _masterCat((RooAbsCategoryLValue*)&masterCat),
+  _masterBranchList("masterBranchList"),
+  _masterLeafList("masterLeafList"),
   _internalCloneBranchList("cloneBranchList"),
   _cloneNodeListAll(splitLeafsAll),
   _cloneNodeListOwned(&splitLeafs)
@@ -242,17 +247,13 @@ RooCustomizer::RooCustomizer(const RooAbsArg& pdf, const RooAbsCategoryLValue& m
 /// suffix for any cloned branch nodes
 
 RooCustomizer::RooCustomizer(const RooAbsArg& pdf, const char* name) :
-  TNamed(pdf.GetName(),pdf.GetTitle()),
-  _sterile(kTRUE), 
-  _owning(kFALSE),
+  _sterile(true),
+  _owning(false),
   _name(name),
-  _masterPdf((RooAbsArg*)&pdf), 
-  _masterCat(0), 
-  _masterBranchList("masterBranchList"), 
-  _masterLeafList("masterLeafList"), 
-  _internalCloneBranchList("cloneBranchList"),
-  _cloneNodeListAll(0),
-  _cloneNodeListOwned(0)
+  _masterPdf((RooAbsArg*)&pdf),
+  _masterBranchList("masterBranchList"),
+  _masterLeafList("masterLeafList"),
+  _internalCloneBranchList("cloneBranchList")
 {
   _cloneBranchList = &_internalCloneBranchList ;
 
@@ -265,25 +266,12 @@ RooCustomizer::RooCustomizer(const RooAbsArg& pdf, const char* name) :
 ////////////////////////////////////////////////////////////////////////////////
 /// Initialize the customizer
 
-void RooCustomizer::initialize() 
+void RooCustomizer::initialize()
 {
   _masterPdf->leafNodeServerList(&_masterLeafList) ;
   _masterPdf->branchNodeServerList(&_masterBranchList) ;
 }
 
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// Destructor
-
-RooCustomizer::~RooCustomizer() 
-{
-
-}
-
-
-  
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Split all arguments in 'set' into individualized clones for each
 /// defined state of 'splitCat'. The 'splitCats' category must be
@@ -292,11 +280,11 @@ RooCustomizer::~RooCustomizer()
 ///
 /// Splitting is only available on customizers created with a master index category
 
-void RooCustomizer::splitArgs(const RooArgSet& set, const RooAbsCategory& splitCat) 
+void RooCustomizer::splitArgs(const RooArgSet& set, const RooAbsCategory& splitCat)
 {
   if (_sterile) {
-    coutE(InputArguments) << "RooCustomizer::splitArgs(" << _name 
-			  << ") ERROR cannot set spitting rules on this sterile customizer" << endl ;
+    oocoutE(nullptr, InputArguments) << "RooCustomizer::splitArgs(" << _name
+           << ") ERROR cannot set spitting rules on this sterile customizer" << endl ;
     return ;
   }
 
@@ -315,17 +303,17 @@ void RooCustomizer::splitArgs(const RooArgSet& set, const RooAbsCategory& splitC
 ///
 /// Splitting is only available on customizers created with a master index category
 
-void RooCustomizer::splitArg(const RooAbsArg& arg, const RooAbsCategory& splitCat) 
+void RooCustomizer::splitArg(const RooAbsArg& arg, const RooAbsCategory& splitCat)
 {
   if (_splitArgList.FindObject(arg.GetName())) {
-    coutE(InputArguments) << "RooCustomizer(" << GetName() << ") ERROR: multiple splitting rules defined for " 
-			  << arg.GetName() << " only using first rule" << endl ;
+    oocoutE(nullptr, InputArguments) << "RooCustomizer(" << _masterPdf->GetName() << ") ERROR: multiple splitting rules defined for "
+           << arg.GetName() << " only using first rule" << endl ;
     return ;
   }
 
   if (_sterile) {
-    coutE(InputArguments) << "RooCustomizer::splitArg(" << _name 
-	 << ") ERROR cannot set spitting rules on this sterile customizer" << endl ;
+    oocoutE(nullptr, InputArguments) << "RooCustomizer::splitArg(" << _name
+    << ") ERROR cannot set spitting rules on this sterile customizer" << endl ;
     return ;
   }
 
@@ -336,13 +324,13 @@ void RooCustomizer::splitArg(const RooAbsArg& arg, const RooAbsCategory& splitCa
 
 
 ////////////////////////////////////////////////////////////////////////////////
-/// Replace any occurence of arg 'orig' with arg 'subst'
+/// Replace any occurrence of arg 'orig' with arg 'subst'
 
-void RooCustomizer::replaceArg(const RooAbsArg& orig, const RooAbsArg& subst) 
+void RooCustomizer::replaceArg(const RooAbsArg& orig, const RooAbsArg& subst)
 {
   if (_replaceArgList.FindObject(orig.GetName())) {
-    coutE(InputArguments) << "RooCustomizer(" << GetName() << ") ERROR: multiple replacement rules defined for " 
-	 << orig.GetName() << " only using first rule" << endl ;
+    oocoutE(nullptr, InputArguments) << "RooCustomizer(" << _masterPdf->GetName() << ") ERROR: multiple replacement rules defined for "
+    << orig.GetName() << " only using first rule" << endl ;
     return ;
   }
 
@@ -358,10 +346,10 @@ void RooCustomizer::replaceArg(const RooAbsArg& orig, const RooAbsArg& subst)
 /// modification. The returned head node owns all cloned branch nodes
 /// that were created in the cloning process.
 
-RooAbsArg* RooCustomizer::build(Bool_t verbose) 
+RooAbsArg* RooCustomizer::build(bool verbose)
 {
   // Execute build
-  RooAbsArg* ret =  doBuild(_name.Length()>0?_name.Data():0,verbose) ;
+  RooAbsArg* ret =  doBuild(_name.Length()>0?_name.Data():nullptr,verbose) ;
 
   // Make root object own all cloned nodes
 
@@ -394,19 +382,19 @@ RooAbsArg* RooCustomizer::build(Bool_t verbose)
 /// is owned by the customizer.  This function cannot be called on
 /// customizer build with the sterile constructor.
 
-RooAbsArg* RooCustomizer::build(const char* masterCatState, Bool_t verbose) 
+RooAbsArg* RooCustomizer::build(const char* masterCatState, bool verbose)
 {
   if (_sterile) {
-    coutE(InputArguments) << "RooCustomizer::build(" << _name 
-			  << ") ERROR cannot use leaf spitting build() on this sterile customizer" << endl ;
-    return 0 ;
+    oocoutE(nullptr, InputArguments) << "RooCustomizer::build(" << _name
+           << ") ERROR cannot use leaf spitting build() on this sterile customizer" << endl ;
+    return nullptr ;
   }
 
   // Set masterCat to given state
   if (_masterCat->setLabel(masterCatState)) {
-    coutE(InputArguments) << "RooCustomizer::build(" << _masterPdf->GetName() << "): ERROR label '" << masterCatState 
-			  << "' not defined for master splitting category " << _masterCat->GetName() << endl ;
-    return 0 ;
+    oocoutE(nullptr, InputArguments) << "RooCustomizer::build(" << _masterPdf->GetName() << "): ERROR label '" << masterCatState
+           << "' not defined for master splitting category " << _masterCat->GetName() << endl ;
+    return nullptr ;
   }
 
   return doBuild(masterCatState,verbose) ;
@@ -417,7 +405,7 @@ RooAbsArg* RooCustomizer::build(const char* masterCatState, Bool_t verbose)
 ////////////////////////////////////////////////////////////////////////////////
 /// Back-end implementation of the p.d.f building functionality
 
-RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, Bool_t verbose) 
+RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, bool verbose)
 {
   // Find nodes that must be split according to provided description, Clone nodes, change their names
   RooArgSet masterNodesToBeSplit("masterNodesToBeSplit") ;
@@ -431,88 +419,88 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, Bool_t verbose)
 
   //   cout << "loop over " << nodeList.getSize() << " nodes" << endl ;
   for (auto node : nodeList) {
-    RooAbsArg* theSplitArg = !_sterile?(RooAbsArg*) _splitArgList.FindObject(node->GetName()):0 ;
+    RooAbsArg* theSplitArg = !_sterile?(RooAbsArg*) _splitArgList.FindObject(node->GetName()):nullptr ;
     if (theSplitArg) {
       RooAbsCategory* splitCat = (RooAbsCategory*) _splitCatList.At(_splitArgList.IndexOf(theSplitArg)) ;
       if (verbose) {
-	coutI(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() 
-			      << "): tree node " << node->GetName() << " is split by category " << splitCat->GetName() << endl ;
+   oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName()
+               << "): tree node " << node->GetName() << " is split by category " << splitCat->GetName() << endl ;
       }
-      
+
       TString newName(node->GetName()) ;
       if (masterCatState) {
-	newName.Append("_") ;
-	newName.Append(splitCat->getCurrentLabel()) ;	
+   newName.Append("_") ;
+   newName.Append(splitCat->getCurrentLabel()) ;
       }
 
       // Check if this node instance already exists
       RooAbsArg* specNode = _cloneNodeListAll ? _cloneNodeListAll->find(newName) : _cloneNodeListOwned->find(newName) ;
       if (specNode) {
 
-	// Copy instance to one-time use list for this build
-	clonedMasterNodes.add(*specNode) ;
-	if (verbose) {
-	  coutI(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() 
-				<< ") Adding existing node specialization " << newName << " to clonedMasterNodes" << endl ;
-	}
+   // Copy instance to one-time use list for this build
+   clonedMasterNodes.add(*specNode) ;
+   if (verbose) {
+     oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName()
+            << ") Adding existing node specialization " << newName << " to clonedMasterNodes" << endl ;
+   }
 
-	// Affix attribute with old name to clone to support name changing server redirect
-	TString nameAttrib("ORIGNAME:") ;
-	nameAttrib.Append(node->GetName()) ;
-	specNode->setAttribute(nameAttrib) ;
-	
-	if (!specNode->getStringAttribute("origName")) {
-	  specNode->setStringAttribute("origName",node->GetName()) ;
-	}
+   // Affix attribute with old name to clone to support name changing server redirect
+   TString nameAttrib("ORIGNAME:") ;
+   nameAttrib.Append(node->GetName()) ;
+   specNode->setAttribute(nameAttrib) ;
+
+   if (!specNode->getStringAttribute("origName")) {
+     specNode->setStringAttribute("origName",node->GetName()) ;
+   }
 
 
 
       } else {
 
-	if (node->isDerived()) {
-	  coutW(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() 
-				<< "): WARNING: branch node " << node->GetName() << " is split but has no pre-defined specializations" << endl ;
-	}
+   if (node->isDerived()) {
+     oocoutW(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName()
+            << "): WARNING: branch node " << node->GetName() << " is split but has no pre-defined specializations" << endl ;
+   }
 
-	TString newTitle(node->GetTitle()) ;
-	newTitle.Append(" (") ;
-	newTitle.Append(splitCat->getCurrentLabel()) ;
-	newTitle.Append(")") ;
-      
-	// Create a new clone
-	RooAbsArg* clone = (RooAbsArg*) node->Clone(newName.Data()) ;
-	clone->setStringAttribute("factory_tag",0) ;
-	clone->SetTitle(newTitle) ;
+   TString newTitle(node->GetTitle()) ;
+   newTitle.Append(" (") ;
+   newTitle.Append(splitCat->getCurrentLabel()) ;
+   newTitle.Append(")") ;
 
-	// Affix attribute with old name to clone to support name changing server redirect
-	TString nameAttrib("ORIGNAME:") ;
-	nameAttrib.Append(node->GetName()) ;
-	clone->setAttribute(nameAttrib) ;
+   // Create a new clone
+   RooAbsArg* clone = (RooAbsArg*) node->Clone(newName.Data()) ;
+   clone->removeStringAttribute("factory_tag") ;
+   clone->SetTitle(newTitle) ;
 
-	if (!clone->getStringAttribute("origName")) {
-	  clone->setStringAttribute("origName",node->GetName()) ;
-	}
+   // Affix attribute with old name to clone to support name changing server redirect
+   TString nameAttrib("ORIGNAME:") ;
+   nameAttrib.Append(node->GetName()) ;
+   clone->setAttribute(nameAttrib) ;
 
-	// Add to one-time use list and life-time use list
-	clonedMasterNodes.add(*clone) ;
-	if (_owning) {
-	  _cloneNodeListOwned->addOwned(*clone) ;	
-	} else {
-	  _cloneNodeListOwned->add(*clone) ;	
-	}
-	if (_cloneNodeListAll) {
-	  _cloneNodeListAll->add(*clone) ;	
-	}	
+   if (!clone->getStringAttribute("origName")) {
+     clone->setStringAttribute("origName",node->GetName()) ;
+   }
+
+   // Add to one-time use list and life-time use list
+   clonedMasterNodes.add(*clone) ;
+   if (_owning) {
+     _cloneNodeListOwned->addOwned(std::unique_ptr<RooAbsArg>{clone});
+   } else {
+     _cloneNodeListOwned->add(*clone) ;
+   }
+   if (_cloneNodeListAll) {
+     _cloneNodeListAll->add(*clone) ;
+   }
       }
-      masterNodesToBeSplit.add(*node) ;     
+      masterNodesToBeSplit.add(*node) ;
     }
 
     RooAbsArg* ReplaceArg = (RooAbsArg*) _replaceArgList.FindObject(node->GetName()) ;
     if (ReplaceArg) {
       RooAbsArg* substArg = (RooAbsArg*) _replaceSubList.At(_replaceArgList.IndexOf(ReplaceArg)) ;
       if (verbose) {
-	coutI(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() 
-			      << "): tree node " << node->GetName() << " will be replaced by " << substArg->GetName() << endl ;
+   oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName()
+               << "): tree node " << node->GetName() << " will be replaced by " << substArg->GetName() << endl ;
       }
 
       // Affix attribute with old name to support name changing server redirect
@@ -529,38 +517,38 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, Bool_t verbose)
   // Find branches that are affected by splitting and must be cloned
   RooArgSet masterBranchesToBeCloned("masterBranchesToBeCloned") ;
   for (auto branch : _masterBranchList) {
-    
+
     // If branch is split itself, don't handle here
     if (masterNodesToBeSplit.find(branch->GetName())) {
       if (verbose) {
-	coutI(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node " << branch->GetName() << " is already split" << endl ;
+   oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node " << branch->GetName() << " is already split" << endl ;
       }
       continue ;
     }
     if (masterNodesToBeReplaced.find(branch->GetName())) {
       if (verbose) {
-	coutI(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node " << branch->GetName() << " is already replaced" << endl ;
+   oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node " << branch->GetName() << " is already replaced" << endl ;
       }
       continue ;
     }
 
     if (branch->dependsOn(masterNodesToBeSplit)) {
       if (verbose) {
-	coutI(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node " 
-			      << branch->IsA()->GetName() << "::" << branch->GetName() << " cloned: depends on a split parameter" << endl ;
+   oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node "
+               << branch->ClassName() << "::" << branch->GetName() << " cloned: depends on a split parameter" << endl ;
       }
       masterBranchesToBeCloned.add(*branch) ;
     } else if (branch->dependsOn(masterNodesToBeReplaced)) {
       if (verbose) {
-	coutI(ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node " 
-			      << branch->IsA()->GetName() << "::" << branch->GetName() << " cloned: depends on a replaced parameter" << endl ;
+   oocoutI(nullptr, ObjectHandling) << "RooCustomizer::build(" << _masterPdf->GetName() << ") Branch node "
+               << branch->ClassName() << "::" << branch->GetName() << " cloned: depends on a replaced parameter" << endl ;
       }
       masterBranchesToBeCloned.add(*branch) ;
     }
   }
 
-  // Clone branches, changes their names 
-  RooAbsArg* cloneTopPdf = 0;
+  // Clone branches, changes their names
+  RooAbsArg* cloneTopPdf = nullptr;
   RooArgSet clonedMasterBranches("clonedMasterBranches") ;
 
   for (auto branch : masterBranchesToBeCloned) {
@@ -572,7 +560,7 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, Bool_t verbose)
 
     // Affix attribute with old name to clone to support name changing server redirect
     RooAbsArg* clone = (RooAbsArg*) branch->Clone(newName.Data()) ;
-    clone->setStringAttribute("factory_tag",0) ;
+    clone->removeStringAttribute("factory_tag") ;
     TString nameAttrib("ORIGNAME:") ;
     nameAttrib.Append(branch->GetName()) ;
     clone->setAttribute(nameAttrib) ;
@@ -580,8 +568,8 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, Bool_t verbose)
     if (!clone->getStringAttribute("origName")) {
       clone->setStringAttribute("origName",branch->GetName()) ;
     }
-    
-    clonedMasterBranches.add(*clone) ;      
+
+    clonedMasterBranches.add(*clone) ;
 
     // Save pointer to clone of top-level pdf
     if (branch==_masterPdf) cloneTopPdf=(RooAbsArg*)clone ;
@@ -592,49 +580,22 @@ RooAbsArg* RooCustomizer::doBuild(const char* masterCatState, Bool_t verbose)
   } else {
     _cloneBranchList->add(clonedMasterBranches) ;
   }
-    
-  // Reconnect cloned branches to each other and to cloned nodess
+
+  // Reconnect cloned branches to each other and to cloned nodes
   for (auto branch : clonedMasterBranches) {
-    branch->redirectServers(clonedMasterBranches,kFALSE,kTRUE) ;
-    branch->redirectServers(clonedMasterNodes,kFALSE,kTRUE) ;
-    branch->redirectServers(masterReplacementNodes,kFALSE,kTRUE) ;
+    branch->redirectServers(clonedMasterBranches,false,true) ;
+    branch->redirectServers(clonedMasterNodes,false,true) ;
+    branch->redirectServers(masterReplacementNodes,false,true) ;
   }
 
   return cloneTopPdf ? cloneTopPdf : _masterPdf ;
 }
 
 
-////////////////////////////////////////////////////////////////////////////////
-/// Print name of customizer
-
-void RooCustomizer::printName(ostream& os) const 
-{
-  os << GetName() ;
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// Print title of customizer
-
-void RooCustomizer::printTitle(ostream& os) const 
-{
-  os << GetTitle() ;
-}
-
-
-////////////////////////////////////////////////////////////////////////////////
-/// Print class name of customizer
-
-void RooCustomizer::printClassName(ostream& os) const 
-{
-  os << IsA()->GetName() ;
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 /// Print arguments of customizer, i.e. input p.d.f and input master category (if any)
 
-void RooCustomizer::printArgs(ostream& os) const 
+void RooCustomizer::printArgs(ostream& os) const
 {
   os << "[ masterPdf=" << _masterPdf->GetName() ;
   if (_masterCat) {
@@ -648,7 +609,7 @@ void RooCustomizer::printArgs(ostream& os) const
 ////////////////////////////////////////////////////////////////////////////////
 /// Print customizer configuration details
 
-void RooCustomizer::printMultiline(ostream& os, Int_t /*content*/, Bool_t /*verbose*/, TString indent) const
+void RooCustomizer::printMultiline(ostream& os, Int_t /*content*/, bool /*verbose*/, TString indent) const
 {
   os << indent << "RooCustomizer for " << _masterPdf->GetName() << (_sterile?" (sterile)":"") << endl ;
 
@@ -667,7 +628,7 @@ void RooCustomizer::printMultiline(ostream& os, Int_t /*content*/, Bool_t /*verb
       os << indent << "   " << _replaceSubList.At(i)->GetName() << " replaces " << _replaceArgList.At(i)->GetName() << endl ;
     }
   }
-  
+
   return ;
 }
 
@@ -677,18 +638,23 @@ void RooCustomizer::printMultiline(ostream& os, Int_t /*content*/, Bool_t /*verb
 /// Install the input RooArgSet as container in which all cloned branches
 /// will be stored
 
-void RooCustomizer::setCloneBranchSet(RooArgSet& cloneBranchSet) 
+void RooCustomizer::setCloneBranchSet(RooArgSet& cloneBranchSet)
 {
   _cloneBranchList = &cloneBranchSet ;
   _cloneBranchList->useHashMapForFind(true);
 }
 
 
+RooAbsPdf const& RooCustomizer::pdf() const {
+  return static_cast<RooAbsPdf&>(*_masterPdf);
+}
 
 
+namespace {
+
 ////////////////////////////////////////////////////////////////////////////////
 
-std::string RooCustomizer::CustIFace::create(RooFactoryWSTool& ft, const char* typeName, const char* instanceName, std::vector<std::string> args) 
+std::string CustIFace::create(RooFactoryWSTool& ft, const char* typeName, const char* instanceName, std::vector<std::string> args)
 {
   // Check number of arguments
   if (args.size()<2) {
@@ -700,19 +666,19 @@ std::string RooCustomizer::CustIFace::create(RooFactoryWSTool& ft, const char* t
   }
 
   // Check that first arg exists as RooAbsArg
-  RooAbsArg* arg = ft.ws().arg(args[0].c_str()) ;
+  RooAbsArg* arg = ft.ws().arg(args[0]) ;
   if (!arg) {
     throw string(Form("RooCustomizer::CustIFace::create() ERROR: input RooAbsArg %s does not exist",args[0].c_str())) ;
   }
 
   // If name of new object is same as original, execute in sterile mode (i.e no suffixes attached), and rename original nodes in workspace upon import
   if (args[0]==instanceName) {
-    instanceName=0 ;
+    instanceName=nullptr ;
   }
 
   // Create a customizer
   RooCustomizer cust(*arg,instanceName) ;
-  
+
   for (unsigned int i=1 ; i<args.size() ; i++) {
     char buf[1024] ;
     strlcpy(buf,args[i].c_str(),1024) ;
@@ -720,9 +686,9 @@ std::string RooCustomizer::CustIFace::create(RooFactoryWSTool& ft, const char* t
     if (!sep) {
       throw string(Form("RooCustomizer::CustIFace::create() ERROR: unknown argument: %s, expect form orig=subst",args[i].c_str())) ;
     }
-    *sep = 0 ;    
+    *sep = 0 ;
     RooAbsArg* orig = ft.ws().arg(buf) ;
-    RooAbsArg* subst(0) ;
+    RooAbsArg* subst(nullptr) ;
     if (string(sep+1).find("$REMOVE")==0) {
 
       // Create a removal dummy ;
@@ -731,18 +697,18 @@ std::string RooCustomizer::CustIFace::create(RooFactoryWSTool& ft, const char* t
       // If removal instructed was annotated with target node, encode these in removal dummy
       char* sep2 = strchr(sep+1,'(') ;
       if (sep2) {
-	char buf2[1024] ;
-	strlcpy(buf2,sep2+1,1024) ;
-	char* saveptr ;
-	char* tok = R__STRTOK_R(buf2,",)",&saveptr) ;
-	while(tok) {
-	  //cout << "$REMOVE is restricted to " << tok << endl ;
-	  subst->setAttribute(Form("REMOVE_FROM_%s",tok)) ;
-	  tok = R__STRTOK_R(0,",)",&saveptr) ;
-	}
+   char buf2[1024] ;
+   strlcpy(buf2,sep2+1,1024) ;
+   char* saveptr ;
+   char* tok = R__STRTOK_R(buf2,",)",&saveptr) ;
+   while(tok) {
+     //cout << "$REMOVE is restricted to " << tok << endl ;
+     subst->setAttribute(Form("REMOVE_FROM_%s",tok)) ;
+     tok = R__STRTOK_R(nullptr,",)",&saveptr) ;
+   }
       } else {
-	// Otherwise mark as universal removal node
-	subst->setAttribute("REMOVE_ALL") ;
+   // Otherwise mark as universal removal node
+   subst->setAttribute("REMOVE_ALL") ;
       }
 
     } else {
@@ -757,12 +723,12 @@ std::string RooCustomizer::CustIFace::create(RooFactoryWSTool& ft, const char* t
     if (orig && subst) {
       cust.replaceArg(*orig,*subst) ;
     } else {
-      oocoutW((TObject*)0,ObjectHandling) << "RooCustomizer::CustIFace::create() WARNING: input or replacement of a replacement operation not found, operation ignored"<< endl ;
+      oocoutW(nullptr,ObjectHandling) << "RooCustomizer::CustIFace::create() WARNING: input or replacement of a replacement operation not found, operation ignored"<< endl ;
     }
   }
 
   // Build the desired edited object
-  RooAbsArg* targ = cust.build(kFALSE)  ;
+  RooAbsArg* targ = cust.build(false)  ;
   if (!targ) {
     throw string(Form("RooCustomizer::CustIFace::create() ERROR in customizer build, object %snot created",instanceName)) ;
   }
@@ -774,8 +740,10 @@ std::string RooCustomizer::CustIFace::create(RooFactoryWSTool& ft, const char* t
     // Now import everything. What we didn't touch gets recycled, everything else was cloned here:
     ft.ws().import(cust.cloneBranchList(), RooFit::Silence(true), RooFit::RecycleConflictNodes(true),    RooFit::NoRecursion(false));
   } else {
-    ft.ws().import(cust.cloneBranchList(), RooFit::Silence(true), RooFit::RenameConflictNodes("orig",1), RooFit::NoRecursion(true));
+    ft.ws().import(cust.cloneBranchList(), RooFit::Silence(true), RooFit::RenameConflictNodes("orig",true), RooFit::NoRecursion(true));
   }
 
   return string(instanceName?instanceName:targ->GetName()) ;
 }
+
+} // namespace
