@@ -153,8 +153,8 @@ RooTreeDataStore::RooTreeDataStore(RooStringView name, RooStringView title, RooA
   // WVE NEED TO ADJUST THIS FOR WEIGHTS
 
   // Protected constructor for internal use only
-  _tree = 0 ;
-  _cacheTree = 0 ;
+  _tree = nullptr ;
+  _cacheTree = nullptr ;
   createTree(makeTreeName(), title);
 
   // Deep clone cutVar and attach clone to this dataset
@@ -167,11 +167,11 @@ RooTreeDataStore::RooTreeDataStore(RooStringView name, RooStringView title, RooA
   // Constructor from existing data set with list of variables that preserves the cache
   initialize();
 
-  attachCache(0,((RooTreeDataStore&)tds)._cachedVars) ;
+  attachCache(nullptr,((RooTreeDataStore&)tds)._cachedVars) ;
 
   // WVE copy values of cached variables here!!!
   _cacheTree->CopyEntries(((RooTreeDataStore&)tds)._cacheTree) ;
-  _cacheOwner = 0 ;
+  _cacheOwner = nullptr ;
 
   loadValues(&tds,cloneVar.get(),cutRange,nStart,nStop);
 }
@@ -217,7 +217,7 @@ RooRealVar* RooTreeDataStore::weightVar(const RooArgSet& allVars, const char* wg
     RooRealVar* wgt = dynamic_cast<RooRealVar*>(allVars.find(wgtName)) ;
     return wgt ;
   }
-  return 0 ;
+  return nullptr ;
 }
 
 
@@ -267,9 +267,9 @@ RooTreeDataStore::RooTreeDataStore(const RooTreeDataStore& other, const char* ne
 ////////////////////////////////////////////////////////////////////////////////
 
 RooTreeDataStore::RooTreeDataStore(const RooTreeDataStore& other, const RooArgSet& vars, const char* newname) :
-  RooAbsDataStore(other,varsNoWeight(vars,other._wgtVar?other._wgtVar->GetName():0),newname),
+  RooAbsDataStore(other,varsNoWeight(vars,other._wgtVar?other._wgtVar->GetName():nullptr),newname),
   _varsww(vars),
-  _wgtVar(other._wgtVar?weightVar(vars,other._wgtVar->GetName()):0),
+  _wgtVar(other._wgtVar?weightVar(vars,other._wgtVar->GetName()):nullptr),
   _extWgtArray(other._extWgtArray),
   _extWgtErrLoArray(other._extWgtErrLoArray),
   _extWgtErrHiArray(other._extWgtErrHiArray),
@@ -327,7 +327,7 @@ void RooTreeDataStore::initialize()
 void RooTreeDataStore::createTree(RooStringView name, RooStringView title)
 {
   if (!_tree) {
-    _tree = new TTree(name,title);
+    _tree = new TTree(name.c_str(),title.c_str());
     _tree->ResetBit(kCanDelete);
     _tree->ResetBit(kMustCleanup);
     _tree->SetDirectory(nullptr);
@@ -345,8 +345,8 @@ void RooTreeDataStore::createTree(RooStringView name, RooStringView title)
   }
 
   if (!_cacheTree) {
-    _cacheTree = new TTree(TString{static_cast<const char*>(name)} + "_cacheTree", TString{static_cast<const char*>(title)});
-    _cacheTree->SetDirectory(0) ;
+    _cacheTree = new TTree(TString{name.c_str()} + "_cacheTree", TString{title.c_str()});
+    _cacheTree->SetDirectory(nullptr) ;
     gDirectory->RecursiveRemove(_cacheTree) ;
   }
 
@@ -375,11 +375,12 @@ void RooTreeDataStore::loadValues(const TTree *t, const RooFormulaVar* select, c
   tClone->SetDirectory(t->GetDirectory());
 
   // Clone list of variables
-  std::unique_ptr<RooArgSet> sourceArgSet( _varsww.snapshot(false) );
+  RooArgSet sourceArgSet;
+  _varsww.snapshot(sourceArgSet, false);
 
   // Check that we have the branches:
   bool missingBranches = false;
-  for (const auto var : *sourceArgSet) {
+  for (const auto var : sourceArgSet) {
      if (!tClone->GetBranch(var->GetName())) {
         missingBranches = true;
         coutE(InputArguments) << "Didn't find a branch in Tree '" << tClone->GetName() << "' to read variable '"
@@ -394,7 +395,7 @@ void RooTreeDataStore::loadValues(const TTree *t, const RooFormulaVar* select, c
   }
 
   // Attach args in cloned list to cloned source tree
-  for (const auto sourceArg : *sourceArgSet) {
+  for (const auto sourceArg : sourceArgSet) {
     sourceArg->attachToTree(*tClone,_defTreeBufSize) ;
   }
 
@@ -402,7 +403,7 @@ void RooTreeDataStore::loadValues(const TTree *t, const RooFormulaVar* select, c
   std::unique_ptr<RooFormulaVar> selectClone;
   if (select) {
     selectClone.reset( static_cast<RooFormulaVar*>(select->cloneTree()) );
-    selectClone->recursiveRedirectServers(*sourceArgSet) ;
+    selectClone->recursiveRedirectServers(sourceArgSet) ;
     selectClone->setOperMode(RooAbsArg::ADirty,true) ;
   }
 
@@ -416,9 +417,9 @@ void RooTreeDataStore::loadValues(const TTree *t, const RooFormulaVar* select, c
 
     // Copy from source to destination
     bool allOK(true) ;
-    for (unsigned int j=0; j < sourceArgSet->size(); ++j) {
+    for (unsigned int j=0; j < sourceArgSet.size(); ++j) {
       auto destArg = _varsww[j];
-      const auto sourceArg = (*sourceArgSet)[j];
+      const auto sourceArg = sourceArgSet[j];
 
       destArg->copyCache(sourceArg) ;
       sourceArg->copyCache(destArg) ;
@@ -556,7 +557,7 @@ const RooArgSet* RooTreeDataStore::get(Int_t index) const
 
   Int_t ret = ((RooTreeDataStore*)this)->GetEntry(index, 1) ;
 
-  if(!ret) return 0;
+  if(!ret) return nullptr;
 
   if (_doDirtyProp) {
     // Raise all dirty flags
@@ -784,12 +785,12 @@ RooAbsArg* RooTreeDataStore::addColumn(RooAbsArg& newVar, bool adjustRange)
   checkInit() ;
 
   // Create a fundamental object of the right type to hold newVar values
-  RooAbsArg* valHolder= newVar.createFundamental();
+  auto valHolder = std::unique_ptr<RooAbsArg>{newVar.createFundamental()}.release();
   // Sanity check that the holder really is fundamental
   if(!valHolder->isFundamental()) {
     coutE(InputArguments) << GetName() << "::addColumn: holder argument is not fundamental: \""
     << valHolder->GetName() << "\"" << endl;
-    return 0;
+    return nullptr;
   }
 
   // WVE need to reset TTRee buffers to original datamembers here
@@ -1023,7 +1024,7 @@ void RooTreeDataStore::resetCache()
 
   // Delete & recreate cache tree
   delete _cacheTree ;
-  _cacheTree = 0 ;
+  _cacheTree = nullptr ;
   createTree(makeTreeName().c_str(), GetTitle());
 
   return ;
@@ -1194,7 +1195,7 @@ std::string RooTreeDataStore::makeTreeName() const {
 /// Get the weights of the events in the range [first, first+len).
 /// This implementation will fill a vector with every event retrieved one by one
 /// (even if the weight is constant). Then, it returns a span.
-RooSpan<const double> RooTreeDataStore::getWeightBatch(std::size_t first, std::size_t len) const {
+std::span<const double> RooTreeDataStore::getWeightBatch(std::size_t first, std::size_t len) const {
 
   if (_extWgtArray) {
     return {_extWgtArray + first, len};
