@@ -105,7 +105,7 @@ void HypoTestInverter::SetCloseProof(bool flag) {
 
 RooRealVar * HypoTestInverter::GetVariableToScan(const HypoTestCalculatorGeneric &hc) {
 
-   RooRealVar * varToScan = 0;
+   RooRealVar * varToScan = nullptr;
    const ModelConfig * mc = hc.GetNullModel();
    if (mc) {
       const RooArgSet * poi  = mc->GetParametersOfInterest();
@@ -166,9 +166,9 @@ void HypoTestInverter::CheckInputModels(const HypoTestCalculatorGeneric &hc,cons
 HypoTestInverter::HypoTestInverter( ) :
    fTotalToysRun(0),
    fMaxToys(0),
-   fCalculator0(0),
-   fScannedVariable(0),
-   fResults(0),
+   fCalculator0(nullptr),
+   fScannedVariable(nullptr),
+   fResults(nullptr),
    fUseCLs(false),
    fScanLog(false),
    fSize(0),
@@ -192,9 +192,9 @@ HypoTestInverter::HypoTestInverter( HypoTestCalculatorGeneric& hc,
                                     RooRealVar* scannedVariable, double size ) :
    fTotalToysRun(0),
    fMaxToys(0),
-   fCalculator0(0),
+   fCalculator0(nullptr),
    fScannedVariable(scannedVariable),
-   fResults(0),
+   fResults(nullptr),
    fUseCLs(false),
    fScanLog(false),
    fSize(size),
@@ -247,7 +247,7 @@ HypoTestInverter::HypoTestInverter( HybridCalculator& hc,
    fMaxToys(0),
    fCalculator0(&hc),
    fScannedVariable(scannedVariable),
-   fResults(0),
+   fResults(nullptr),
    fUseCLs(false),
    fScanLog(false),
    fSize(size),
@@ -280,7 +280,7 @@ HypoTestInverter::HypoTestInverter( FrequentistCalculator& hc,
    fMaxToys(0),
    fCalculator0(&hc),
    fScannedVariable(scannedVariable),
-   fResults(0),
+   fResults(nullptr),
    fUseCLs(false),
    fScanLog(false),
    fSize(size),
@@ -312,7 +312,7 @@ HypoTestInverter::HypoTestInverter( AsymptoticCalculator& hc,
    fMaxToys(0),
    fCalculator0(&hc),
    fScannedVariable(scannedVariable),
-   fResults(0),
+   fResults(nullptr),
    fUseCLs(false),
    fScanLog(false),
    fSize(size),
@@ -343,9 +343,9 @@ HypoTestInverter::HypoTestInverter( RooAbsData& data, ModelConfig &sbModel, Mode
                 RooRealVar * scannedVariable,  ECalculatorType type, double size) :
    fTotalToysRun(0),
    fMaxToys(0),
-   fCalculator0(0),
+   fCalculator0(nullptr),
    fScannedVariable(scannedVariable),
-   fResults(0),
+   fResults(nullptr),
    fUseCLs(false),
    fScanLog(false),
    fSize(size),
@@ -378,8 +378,8 @@ HypoTestInverter::HypoTestInverter( RooAbsData& data, ModelConfig &sbModel, Mode
 HypoTestInverter::HypoTestInverter(const HypoTestInverter & rhs) :
    IntervalCalculator(),
    fTotalToysRun(0),
-   fCalculator0(0), fScannedVariable(0),  // add these for Coverity
-   fResults(0)
+   fCalculator0(nullptr), fScannedVariable(nullptr),  // add these for Coverity
+   fResults(nullptr)
 {
    (*this) = rhs;
 }
@@ -415,7 +415,7 @@ HypoTestInverter & HypoTestInverter::operator= (const HypoTestInverter & rhs) {
 HypoTestInverter::~HypoTestInverter()
 {
    if (fResults) delete fResults;
-   fCalculator0 = 0;
+   fCalculator0 = nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -427,7 +427,7 @@ TestStatistic * HypoTestInverter::GetTestStatistic( ) const
       return fCalculator0->GetTestStatSampler()->GetTestStatistic();
    }
    else
-      return 0;
+      return nullptr;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -447,15 +447,15 @@ bool HypoTestInverter::SetTestStatistic(TestStatistic& stat)
 
 void  HypoTestInverter::Clear()  {
    if (fResults) delete fResults;
-   fResults = 0;
-   fLimitPlot.reset(nullptr);
+   fResults = nullptr;
+   fLimitPlot.reset();
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 /// create a new HypoTestInverterResult to hold all computed results
 
 void  HypoTestInverter::CreateResults() const {
-   if (fResults == 0) {
+   if (fResults == nullptr) {
       TString results_name = "result_";
       results_name += fScannedVariable->GetName();
       fResults = new HypoTestInverterResult(results_name,*fScannedVariable,ConfidenceLevel());
@@ -530,7 +530,7 @@ HypoTestResult * HypoTestInverter::Eval(HypoTestCalculatorGeneric &hc, bool adap
 
    // run the hypothesis test
    HypoTestResult *  hcResult = hc.GetHypoTest();
-   if (hcResult == 0) {
+   if (hcResult == nullptr) {
       oocoutE(nullptr,Eval) << "HypoTestInverter::Eval - HypoTest failed" << std::endl;
       return hcResult;
    }
@@ -773,7 +773,7 @@ bool HypoTestInverter::RunLimit(double &limit, double &limitErr, double absAccur
 
    RooRealVar *r = fScannedVariable;
 
-  if ((hint != 0) && (*hint > r->getMin())) {
+  if ((hint != nullptr) && (*hint > r->getMin())) {
      r->setMax(std::min<double>(3.0 * (*hint), r->getMax()));
      r->setMin(std::max<double>(0.3 * (*hint), r->getMin()));
      oocoutI(nullptr,InputArguments) << "HypoTestInverter::RunLimit - Use hint value " << *hint
@@ -1024,13 +1024,13 @@ SamplingDistribution * HypoTestInverter::GetLowerLimitDistribution(bool rebuild,
    if (!rebuild) {
       if (!fResults) {
          oocoutE(nullptr,InputArguments) << "HypoTestInverter::GetLowerLimitDistribution(false) - result not existing\n";
-         return 0;
+         return nullptr;
       }
       return fResults->GetLowerLimitDistribution();
    }
 
-   TList * clsDist = 0;
-   TList * clsbDist = 0;
+   TList * clsDist = nullptr;
+   TList * clsbDist = nullptr;
    if (fUseCLs) clsDist = &fResults->fExpPValues;
    else clsbDist = &fResults->fExpPValues;
 
@@ -1051,13 +1051,13 @@ SamplingDistribution * HypoTestInverter::GetUpperLimitDistribution(bool rebuild,
    if (!rebuild) {
       if (!fResults) {
          oocoutE(nullptr,InputArguments) << "HypoTestInverter::GetUpperLimitDistribution(false) - result not existing\n";
-         return 0;
+         return nullptr;
       }
       return fResults->GetUpperLimitDistribution();
    }
 
-   TList * clsDist = 0;
-   TList * clsbDist = 0;
+   TList * clsDist = nullptr;
+   TList * clsbDist = nullptr;
    if (fUseCLs) clsDist = &fResults->fExpPValues;
    else clsbDist = &fResults->fExpPValues;
 
@@ -1081,13 +1081,13 @@ void HypoTestInverter::SetData(RooAbsData & data) {
 
 SamplingDistribution * HypoTestInverter::RebuildDistributions(bool isUpper, int nToys, TList * clsDist, TList * clsbDist, TList * clbDist, const char *outputfile) {
 
-   if (!fScannedVariable || !fCalculator0) return 0;
+   if (!fScannedVariable || !fCalculator0) return nullptr;
    // get first background snapshot
    const ModelConfig * bModel = fCalculator0->GetAlternateModel();
    const ModelConfig * sbModel = fCalculator0->GetNullModel();
-   if (!bModel || ! sbModel) return 0;
+   if (!bModel || ! sbModel) return nullptr;
    RooArgSet paramPoint;
-   if (!sbModel->GetParametersOfInterest()) return 0;
+   if (!sbModel->GetParametersOfInterest()) return nullptr;
    paramPoint.add(*sbModel->GetParametersOfInterest());
 
    const RooArgSet * poibkg = bModel->GetSnapshot();
@@ -1104,7 +1104,7 @@ SamplingDistribution * HypoTestInverter::RebuildDistributions(bool isUpper, int
    ToyMCSampler * toymcSampler = dynamic_cast<ToyMCSampler *>(fCalculator0->GetTestStatSampler() );
    if (!toymcSampler) {
       oocoutE(nullptr,InputArguments) << "HypoTestInverter::RebuildDistribution - no toy MC sampler existing" << std::endl;
-      return 0;
+      return nullptr;
    }
    // set up test stat sampler in case of asymptotic calculator
    if (dynamic_cast<RooStats::AsymptoticCalculator*>(fCalculator0) ) {
@@ -1133,7 +1133,7 @@ SamplingDistribution * HypoTestInverter::RebuildDistributions(bool isUpper, int
       if (nPoints <=0) {
          oocoutE(nullptr,InputArguments) << "HypoTestInverter - result is not existing and number of point to scan is not set"
                                              << std::endl;
-         return 0;
+         return nullptr;
       }
    }
 
@@ -1239,7 +1239,7 @@ SamplingDistribution * HypoTestInverter::RebuildDistributions(bool isUpper, int
 
       HypoTestInverterResult * r  = inverter.GetInterval();
 
-      if (r == 0) continue;
+      if (r == nullptr) continue;
 
       double value = (isUpper) ? r->UpperLimit() : r->LowerLimit();
       limit_values.push_back( value );
