@@ -32,10 +32,7 @@ objects.
 
 #include "RooMsgService.h"
 
-#include <limits>
-#define NaN std::numeric_limits<float>::quiet_NaN()
 #include "TMath.h"
-#define IsNaN(a) TMath::IsNaN(a)
 
 ClassImp(RooStats::SamplingDistPlot);
 
@@ -45,45 +42,14 @@ using namespace std;
 ////////////////////////////////////////////////////////////////////////////////
 /// SamplingDistPlot default constructor with bin size
 
-SamplingDistPlot::SamplingDistPlot(Int_t nbins) :
-   fHist(0),
-   fLegend(nullptr),
-   fItems(),
-   fOtherItems(),
-   fRooPlot(nullptr),
-   fLogXaxis(false),
-   fLogYaxis(false),
-   fXMin(NaN), fXMax(NaN), fYMin(NaN), fYMax(NaN),
-   fApplyStyle(true),
-   fFillStyle(3004)
-{
-  fIsWeighted = false;
-  fBins = nbins;
-  fMarkerType = 20;
-  fColor = 1;
-}
+SamplingDistPlot::SamplingDistPlot(Int_t nbins) : fBins{nbins} {}
 
 ////////////////////////////////////////////////////////////////////////////////
 /// SamplingDistPlot constructor with bin size, min and max
 
-SamplingDistPlot::SamplingDistPlot(Int_t nbins, double min, double max) :
-   fHist(0),
-   fLegend(nullptr),
-   fItems(),
-   fOtherItems(),
-   fRooPlot(nullptr),
-   fLogXaxis(false),
-   fLogYaxis(false),
-   fXMin(NaN), fXMax(NaN), fYMin(NaN), fYMax(NaN),
-   fApplyStyle(true),
-   fFillStyle(3004)
+SamplingDistPlot::SamplingDistPlot(Int_t nbins, double min, double max) : fBins{nbins}
 {
-  fIsWeighted = false;
-  fBins = nbins;
-  fMarkerType = 20;
-  fColor = 1;
-
-  SetXRange( min, max );
+   SetXRange(min, max);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -133,11 +99,11 @@ double SamplingDistPlot::AddSamplingDistribution(const SamplingDistribution *sam
    double binWidth = (xmax-xmin)/(fBins);
    double xlow = xmin - 1.5*binWidth;
    double xup  = xmax + 1.5*binWidth;
-   if( !IsNaN(fXMin) ) xlow = fXMin;
-   if( !IsNaN(fXMax) ) xup = fXMax;
+   if( !TMath::IsNaN(fXMin) ) xlow = fXMin;
+   if( !TMath::IsNaN(fXMax) ) xup = fXMax;
 
    fHist = new TH1F(samplingDist->GetName(), samplingDist->GetTitle(), fBins, xlow, xup);
-   fHist->SetDirectory(0);  // make the object managed by this class
+   fHist->SetDirectory(nullptr);  // make the object managed by this class
 
    if( fVarName.Length() == 0 ) fVarName = samplingDist->GetVarName();
    fHist->GetXaxis()->SetTitle(fVarName.Data());
@@ -189,7 +155,7 @@ double SamplingDistPlot::AddSamplingDistributionShaded(const SamplingDistributio
    double scaleFactor = AddSamplingDistribution(samplingDist, drawOptions);
 
    TH1F *shaded = (TH1F*)fHist->Clone((string(samplingDist->GetName())+string("_shaded")).c_str());
-   shaded->SetDirectory(0);
+   shaded->SetDirectory(nullptr);
    shaded->SetFillStyle(fFillStyle++);
    shaded->SetLineWidth(1);
 
@@ -228,7 +194,7 @@ void SamplingDistPlot::AddLine(double x1, double y1, double x2, double y2, const
 void SamplingDistPlot::AddTH1(TH1* h, Option_t *drawOptions) {
    if(fLegend  &&  h->GetTitle()) fLegend->AddEntry(h, h->GetTitle(), "L");
    TH1 * hcopy = (TH1*) h->Clone();
-   hcopy->SetDirectory(0);
+   hcopy->SetDirectory(nullptr);
    addObject(hcopy, drawOptions);
 }
 void SamplingDistPlot::AddTF1(TF1* f, const char* title, Option_t *drawOptions) {
@@ -260,7 +226,7 @@ void SamplingDistPlot::SetSampleWeights(const SamplingDistribution* samplingDist
 void SamplingDistPlot::addObject(TObject *obj, Option_t *drawOptions)
 {
 
-  if(0 == obj) {
+  if(nullptr == obj) {
     std::cerr << fName << "::addObject: called with a null pointer" << std::endl;
     return;
   }
@@ -278,7 +244,7 @@ void SamplingDistPlot::addObject(TObject *obj, Option_t *drawOptions)
 
 void SamplingDistPlot::addOtherObject(TObject *obj, Option_t *drawOptions)
 {
-  if(0 == obj) {
+  if(nullptr == obj) {
      coutE(InputArguments) << fName << "::addOtherObject: called with a null pointer" << std::endl;
      return;
   }
@@ -296,12 +262,12 @@ void SamplingDistPlot::addOtherObject(TObject *obj, Option_t *drawOptions)
 void SamplingDistPlot::Draw(Option_t * /*options */) {
    ApplyDefaultStyle();
 
-   double theMin(0.), theMax(0.), theYMin(NaN), theYMax(0.);
+   double theMin(0.), theMax(0.), theYMin(std::numeric_limits<float>::quiet_NaN()), theYMax(0.);
    GetAbsoluteInterval(theMin, theMax, theYMax);
-   if( !IsNaN(fXMin) ) theMin = fXMin;
-   if( !IsNaN(fXMax) ) theMax = fXMax;
-   if( !IsNaN(fYMin) ) theYMin = fYMin;
-   if( !IsNaN(fYMax) ) theYMax = fYMax;
+   if( !TMath::IsNaN(fXMin) ) theMin = fXMin;
+   if( !TMath::IsNaN(fXMax) ) theMax = fXMax;
+   if( !TMath::IsNaN(fYMin) ) theYMin = fYMin;
+   if( !TMath::IsNaN(fYMax) ) theYMax = fYMax;
 
    RooRealVar xaxis("xaxis", fVarName.Data(), theMin, theMax);
 
@@ -318,11 +284,11 @@ void SamplingDistPlot::Draw(Option_t * /*options */) {
      return;
    }
    fRooPlot->SetTitle("");
-   if( !IsNaN(theYMax) ) {
+   if( !TMath::IsNaN(theYMax) ) {
       //coutI(InputArguments) << "Setting maximum to " << theYMax << endl;
       fRooPlot->SetMaximum(theYMax);
    }
-   if( !IsNaN(theYMin) ) {
+   if( !TMath::IsNaN(theYMin) ) {
       //coutI(InputArguments) << "Setting minimum to " << theYMin << endl;
       fRooPlot->SetMinimum(theYMin);
    }
@@ -331,15 +297,15 @@ void SamplingDistPlot::Draw(Option_t * /*options */) {
       //obj->Draw(fIterator->GetOption());
       // add cloned objects to avoid mem leaks
       TH1 * cloneObj = (TH1*)obj->Clone();
-      if( !IsNaN(theYMax) ) {
+      if( !TMath::IsNaN(theYMax) ) {
          //coutI(InputArguments) << "Setting maximum of TH1 to " << theYMax << endl;
          cloneObj->SetMaximum(theYMax);
       }
-      if( !IsNaN(theYMin) ) {
+      if( !TMath::IsNaN(theYMin) ) {
          //coutI(InputArguments) << "Setting minimum of TH1 to " << theYMin << endl;
          cloneObj->SetMinimum(theYMin);
       }
-      cloneObj->SetDirectory(0);  // transfer ownership of the object
+      cloneObj->SetDirectory(nullptr);  // transfer ownership of the object
       fRooPlot->addTH1(cloneObj, obj->GetOption());
    }
 
@@ -420,7 +386,7 @@ void SamplingDistPlot::GetAbsoluteInterval(double &theMin, double &theMax, doubl
 /// fill color for the associated shaded TH1F.
 
 void SamplingDistPlot::SetLineColor(Color_t color, const SamplingDistribution *samplDist) {
-   if (samplDist == 0) {
+   if (samplDist == nullptr) {
       fHist->SetLineColor(color);
 
       TString shadedName(fHist->GetName());
@@ -458,7 +424,7 @@ void SamplingDistPlot::SetLineColor(Color_t color, const SamplingDistribution *s
 
 void SamplingDistPlot::SetLineWidth(Width_t lwidth, const SamplingDistribution *samplDist)
 {
-  if(samplDist == 0){
+  if(samplDist == nullptr){
     fHist->SetLineWidth(lwidth);
   }
   else{
@@ -477,7 +443,7 @@ void SamplingDistPlot::SetLineWidth(Width_t lwidth, const SamplingDistribution *
 
 void SamplingDistPlot::SetLineStyle(Style_t style, const SamplingDistribution *samplDist)
 {
-  if(samplDist == 0){
+  if(samplDist == nullptr){
     fHist->SetLineStyle(style);
   }
   else{
@@ -496,7 +462,7 @@ void SamplingDistPlot::SetLineStyle(Style_t style, const SamplingDistribution *s
 
 void SamplingDistPlot::SetMarkerStyle(Style_t style, const SamplingDistribution *samplDist)
 {
-  if(samplDist == 0){
+  if(samplDist == nullptr){
     fHist->SetMarkerStyle(style);
   }
   else{
@@ -515,7 +481,7 @@ void SamplingDistPlot::SetMarkerStyle(Style_t style, const SamplingDistribution
 
 void SamplingDistPlot::SetMarkerColor(Color_t color, const SamplingDistribution *samplDist)
 {
-  if(samplDist == 0){
+  if(samplDist == nullptr){
     fHist->SetMarkerColor(color);
   }
   else{
@@ -534,7 +500,7 @@ void SamplingDistPlot::SetMarkerColor(Color_t color, const SamplingDistribution
 
 void SamplingDistPlot::SetMarkerSize(Size_t size, const SamplingDistribution *samplDist)
 {
-  if(samplDist == 0){
+  if(samplDist == nullptr){
     fHist->SetMarkerSize(size);
   }
   else{
@@ -570,7 +536,7 @@ TH1F* SamplingDistPlot::GetTH1F(const SamplingDistribution *samplDist)
 
 void SamplingDistPlot::RebinDistribution(Int_t rebinFactor, const SamplingDistribution *samplDist)
 {
-  if(samplDist == 0){
+  if(samplDist == nullptr){
     fHist->Rebin(rebinFactor);
   }
   else{
