@@ -227,10 +227,10 @@ void TMVA::DecisionTreeNode::Print(std::ostream& os) const
       << " nType: " << this->GetNodeType()
       << std::endl;
 
-   os << "My address is " << long(this) << ", ";
-   if (this->GetParent() != NULL) os << " parent at addr: "         << long(this->GetParent()) ;
-   if (this->GetLeft()   != NULL) os << " left daughter at addr: "  << long(this->GetLeft());
-   if (this->GetRight()  != NULL) os << " right daughter at addr: " << long(this->GetRight()) ;
+   os << "My address is " << (Longptr_t)this << ", ";
+   if (this->GetParent() != NULL) os << " parent at addr: "         << (Longptr_t)this->GetParent();
+   if (this->GetLeft()   != NULL) os << " left daughter at addr: "  << (Longptr_t)this->GetLeft();
+   if (this->GetRight()  != NULL) os << " right daughter at addr: " << (Longptr_t)this->GetRight();
 
    os << " **** > " << std::endl;
 }
@@ -462,7 +462,7 @@ void TMVA::DecisionTreeNode::ReadAttributes(void* node, UInt_t /* tmva_Version_C
       this->SetNFisherCoeff(nCoef);
       Double_t tmp;
       for (Int_t i=0; i< (Int_t) this->GetNFisherCoeff(); i++) {
-         gTools().ReadAttr(node, Form("fC%d",i),  tmp          );
+         gTools().ReadAttr(node, TString::Format("fC%d",i).Data(),  tmp);
          this->SetFisherCoeff(i,tmp);
       }
    }else{
@@ -493,7 +493,7 @@ void TMVA::DecisionTreeNode::AddAttributesToNode(void* node) const
 {
    gTools().AddAttr(node, "NCoef", GetNFisherCoeff());
    for (Int_t i=0; i< (Int_t) this->GetNFisherCoeff(); i++)
-      gTools().AddAttr(node, Form("fC%d",i),  this->GetFisherCoeff(i));
+      gTools().AddAttr(node, TString::Format("fC%d",i).Data(),  this->GetFisherCoeff(i));
 
    gTools().AddAttr(node, "IVar",  GetSelector());
    gTools().AddAttr(node, "Cut",   GetCutValue());
