@@ -195,10 +195,10 @@ void TMVA::RuleFitParams::InitNtuple()
    fNTLinCoeff = (fNLinear>0 ? new Double_t[fNLinear] : 0);
 
    for (UInt_t i=0; i<fNRules; i++) {
-      fGDNtuple->Branch(Form("a%d",i+1),&fNTCoeff[i],Form("a%d/D",i+1));
+      fGDNtuple->Branch(TString::Format("a%d",i+1).Data(),&fNTCoeff[i],TString::Format("a%d/D",i+1).Data());
    }
    for (UInt_t i=0; i<fNLinear; i++) {
-      fGDNtuple->Branch(Form("b%d",i+1),&fNTLinCoeff[i],Form("b%d/D",i+1));
+      fGDNtuple->Branch(TString::Format("b%d",i+1).Data(),&fNTLinCoeff[i],TString::Format("b%d/D",i+1).Data());
    }
 }
 
@@ -329,7 +329,7 @@ Double_t TMVA::RuleFitParams::Risk(UInt_t ind1,UInt_t ind2, Double_t neff) const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-/// risk assessment for tau model <itau>
+/// risk assessment for tau model `<itau>`
 
 Double_t TMVA::RuleFitParams::Risk(UInt_t ind1,UInt_t ind2, Double_t neff, UInt_t itau) const
 {
@@ -1070,7 +1070,6 @@ Double_t TMVA::RuleFitParams::ErrorRateRocRaw( std::vector<Double_t> & sFsig,
    // Double_t drejb;
    Double_t deffs;
    Double_t area=0;
-   Int_t    npok=0;
    //
    // loop over range of F [minf,maxf]
    //
@@ -1080,7 +1079,6 @@ Double_t TMVA::RuleFitParams::ErrorRateRocRaw( std::vector<Double_t> & sFsig,
                            std::bind(std::greater_equal<Double_t>(), std::placeholders::_1, fcut));
       nesig = sFsig.end()-indit; // number of sig accepted with F>cut
       if (TMath::Abs(pnesig-nesig)>0) {
-         npok++;
          indit = std::find_if(sFbkg.begin(), sFbkg.end(),
                               std::bind(std::greater_equal<Double_t>(), std::placeholders::_1, fcut));
          nrbkg = indit-sFbkg.begin(); // number of bkg rejected with F>cut
@@ -1197,7 +1195,7 @@ void TMVA::RuleFitParams::ErrorRateRocTst()
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Estimates the error rate with the current set of parameters.
-/// using the <Perf> subsample.
+/// using the `<Perf>` subsample.
 /// Return the tau index giving the lowest error
 
 UInt_t TMVA::RuleFitParams::RiskPerfTst()
@@ -1291,7 +1289,6 @@ void TMVA::RuleFitParams::MakeTstGradientVector()
    //
    // Loop over all events
    //
-   UInt_t nsfok=0;
    for (UInt_t i=fPathIdx1; i<fPathIdx2+1; i++) {
       const Event *e = (*events)[i];
       UInt_t nrules=0;
@@ -1305,7 +1302,6 @@ void TMVA::RuleFitParams::MakeTstGradientVector()
          if (fGDErrTstOK[itau]) {
             sF = fRuleEnsemble->EvalEvent( i, fGDOfsTst[itau], fGDCoefTst[itau], fGDCoefLinTst[itau] );
             if (TMath::Abs(sF)<1.0) {
-               nsfok++;
                r = 0;
                y = (fRuleFit->GetMethodRuleFit()->DataInfo().IsSignal(e)?1.0:-1.0);
                r = norm*(y - sF) * fRuleFit->GetTrainingEventWeight(i);
