@@ -7,10 +7,13 @@
  *************************************************************************/
 
 #include "ROOT/RDF/RDisplay.hxx"
-#include "TInterpreter.h"
 
 #include <iomanip>
+#include <iostream>
 #include <limits>
+#include <sstream>
+#include <string>
+#include <vector>
 
 namespace ROOT {
 namespace Internal {
