@@ -8,6 +8,7 @@
  * For the list of contributors see $ROOTSYS/README/CREDITS.             *
  *************************************************************************/
 
+#include "ROOT/InternalTreeUtils.hxx"
 #include "ROOT/RDataFrame.hxx"
 #include "ROOT/RDataSource.hxx"
 #include "ROOT/RDF/RDatasetSpec.hxx"
@@ -76,11 +77,13 @@ You can directly see RDataFrame in action in our [tutorials](https://root.cern.c
    - [Special helper columns: `rdfentry_` and `rdfslot_`](\ref helper-cols)
    - [Just-in-time compilation: column type inference and explicit declaration of column types](\ref jitting)
    - [User-defined custom actions](\ref generic-actions)
-   - [Friend trees](\ref friends)
+   - [Dataset joins with friend trees](\ref friends)
    - [Reading data formats other than ROOT trees](\ref other-file-formats)
    - [Computation graphs (storing and reusing sets of transformations)](\ref callgraphs)
    - [Visualizing the computation graph](\ref representgraph)
    - [Activating RDataFrame execution logs](\ref rdf-logging)
+   - [Creating an RDataFrame from a dataset specification file](\ref rdf-from-spec)
+   - [Adding a progress bar](\ref progressbar)
 - [Efficient analysis in Python](\ref python)
 - <a class="el" href="classROOT_1_1RDataFrame.html#reference" onclick="javascript:toggleInherit('pub_methods_classROOT_1_1RDF_1_1RInterface')">Class reference</a>
 
@@ -308,7 +311,7 @@ This is useful to generate simple datasets on the fly: the contents of each even
 For data sources other than TTrees and TChains, RDataFrame objects are constructed using ad-hoc factory functions (see e.g. FromCSV(), FromSqlite(), FromArrow()):
 
 ~~~{.cpp}
-auto df = ROOT::RDF::MakeCsvDataFrame("input.csv");
+auto df = ROOT::RDF::FromCSV("input.csv");
 // use df as usual
 ~~~
 
@@ -680,10 +683,7 @@ parts of the RDataFrame API currently work with this package. The subset that is
 with support for more operations coming in the future. Data sources other than TTree and TChain (e.g. CSV, RNTuple) are
 currently not supported.
 
-**Note** that the distributed RDataFrame module is available in a ROOT installation if the following criteria are met:
-- PyROOT is available
-- RDataFrame is available
-- The version of the Python interpreter used to build ROOT is greater or equal than 3.7
+\note The distributed RDataFrame module requires at least Python version 3.8.
 
 ### Connecting to a Spark cluster
 
@@ -830,6 +830,66 @@ Without this, two partial histograms resulting from two distributed tasks would
 to errors when merging them. Failing to pass a histogram model will raise an error on the client side, before starting
 the distributed execution.
 
+### Live visualization in distributed mode with dask
+
+The live visualization feature allows real-time data representation of plots generated during the execution 
+of a distributed RDataFrame application. 
+It enables visualizing intermediate results as they are computed across multiple nodes of a Dask cluster
+by creating a canvas and continuously updating it as partial results become available. 
+
+The LiveVisualize() function can be imported from the Python package **ROOT.RDF.Experimental.Distributed**:
+
+~~~{.py}
+import ROOT
+
+LiveVisualize = ROOT.RDF.Experimental.Distributed.LiveVisualize
+~~~
+
+The function takes drawable objects (e.g. histograms) and optional callback functions as argument, it accepts 4 different input formats:
+
+- Passing a list or tuple of drawables: 
+You can pass a list or tuple containing the plots you want to visualize. For example:
+
+~~~{.py}
+LiveVisualize([h_gaus, h_exp, h_random])
+~~~
+
+- Passing a list or tuple of drawables with a global callback function: 
+You can also include a global callback function that will be applied to all plots. For example:
+
+~~~{.py}
+def set_fill_color(hist):
+    hist.SetFillColor(ROOT.kBlue)
+
+LiveVisualize([h_gaus, h_exp, h_random], set_fill_color)
+~~~
+
+- Passing a Dictionary of drawables and callback functions: 
+For more control, you can create a dictionary where keys are plots and values are corresponding (optional) callback functions. For example:
+
+~~~{.py}
+plot_callback_dict = {
+    graph: set_marker,
+    h_exp: fit_exp,
+    tprofile_2d: None
+}
+
+LiveVisualize(plot_callback_dict)
+~~~
+
+- Passing a Dictionary of drawables and callback functions with a global callback function: 
+You can also combine a dictionary of plots and callbacks with a global callback function:
+
+~~~{.py}
+LiveVisualize(plot_callback_dict, write_to_tfile)
+~~~
+
+\note The allowed operations to pass to LiveVisualize are:
+      - Histo1D(), Histo2D(), Histo3D()
+      - Graph()
+      - Profile1D(), Profile2D()
+
+\warning The Live Visualization feature is only supported for the Dask backend.
 
 \anchor parallel-execution
 ## Performance tips and parallel execution
@@ -973,7 +1033,7 @@ not limited to string expressions but they can also pass any valid C++ callable,
 complex functors. The callable can be applied to zero or more existing columns and it will always receive their
 _nominal_ values in input.
 
-**Varying multiple columns in lockstep**
+#### Varying multiple columns in lockstep
 
 In the following Python snippet we use the Vary() signature that allows varying multiple columns simultaneously or
 "in lockstep":
@@ -991,7 +1051,7 @@ this case we also have to explicitly pass a variation name as there is no one co
 
 The call above will produce variations "ptAndEta:down" and "ptAndEta:up".
 
-**Combining multiple variations**
+#### Combining multiple variations
 
 Even if a result depends on multiple variations, only one is applied at a time, i.e. there will be no result produced
 by applying multiple systematic variations at the same time.
@@ -1249,26 +1309,56 @@ Notice how we created one `double` variable for each processing slot and later m
 
 
 \anchor friends
-### Friend trees
-Friend TTrees are supported by RDataFrame.
-Friend TTrees with a TTreeIndex are supported starting from ROOT v6.24.
+### Dataset joins with friend trees
 
-To use friend trees in RDataFrame, it is necessary to add the friends directly to
-the tree and instantiate an RDataFrame with the main tree:
+Vertically concatenating multiple trees that have the same columns (creating a logical dataset with the same columns and
+more rows) is trivial in RDataFrame: just pass the tree name and a list of file names to RDataFrame's constructor, or create a TChain
+out of the desired trees and pass that to RDataFrame.
+
+Horizontal concatenations of trees or chains (creating a logical dataset with the same number of rows and the union of the
+columns of multiple trees) leverages TTree's "friend" mechanism.
+
+Simple joins of trees that do not have the same number of rows are also possible with indexed friend trees (see below).
+
+To use friend trees in RDataFrame, set up trees with the appropriate relationships and then instantiate an RDataFrame
+with the main tree:
 
 ~~~{.cpp}
-TTree t([...]);
-TTree ft([...]);
-t.AddFriend(&ft, "myFriend");
+TTree main([...]);
+TTree friend([...]);
+main.AddFriend(&friend, "myFriend");
 
-RDataFrame d(t);
-auto f = d.Filter("myFriend.MyCol == 42");
+RDataFrame df(main);
+auto df2 = df.Filter("myFriend.MyCol == 42");
 ~~~
 
-Columns coming from the friend trees can be referred to by their full name, like in the example above,
+The same applies for TChains. Columns coming from the friend trees can be referred to by their full name, like in the example above,
 or the friend tree name can be omitted in case the column name is not ambiguous (e.g. "MyCol" could be used instead of
-      "myFriend.MyCol" in the example above).
+"myFriend.MyCol" in the example above if there is no column "MyCol" in the main tree).
 
+\note A common source of confusion is that trees that are written out from a multi-thread Snapshot() call will have their
+      entries (block-wise) shuffled with respect to the original tree. Such trees cannot be used as friends of the original
+      one: rows will be mismatched.
+
+Indexed friend trees provide a way to perform simple joins of multiple trees over a common column.
+When a certain entry in the main tree (or chain) is loaded, the friend trees (or chains) will then load an entry where the
+"index" columns have a value identical to the one in the main one. For example, in Python:
+
+~~~{.py}
+main_tree = ...
+aux_tree = ...
+
+# If a friend tree has an index on `commonColumn`, when the main tree loads
+# a given row, it also loads the row of the friend tree that has the same
+# value of `commonColumn`
+aux_tree.BuildIndex("commonColumn")
+
+mainTree.AddFriend(aux_tree)
+
+df = ROOT.RDataFrame(mainTree)
+~~~
+
+RDataFrame supports indexed friend TTrees from ROOT v6.24 in single-thread mode and from v6.28/02 in multi-thread mode.
 
 \anchor other-file-formats
 ### Reading data formats other than ROOT trees
@@ -1278,7 +1368,7 @@ RDataFrame calls into concrete RDataSource implementations to retrieve informati
 and to advance the readers to the desired data entry.
 Some predefined RDataSources are natively provided by ROOT such as the ROOT::RDF::RCsvDS which allows to read comma separated files:
 ~~~{.cpp}
-auto tdf = ROOT::RDF::MakeCsvDataFrame("MuRun2010B.csv");
+auto tdf = ROOT::RDF::FromCSV("MuRun2010B.csv");
 auto filteredEvents =
    tdf.Filter("Q1 * Q2 == -1")
       .Define("m", "sqrt(pow(E1 + E2, 2) - (pow(px1 + px2, 2) + pow(py1 + py2, 2) + pow(pz1 + pz2, 2)))");
@@ -1353,6 +1443,96 @@ verbosity = ROOT.Experimental.RLogScopedVerbosity(ROOT.Detail.RDF.RDFLogChannel(
 
 More information (e.g. start and end of each multi-thread task) is printed using `ELogLevel.kDebug` and even more
 (e.g. a full dump of the generated code that RDataFrame just-in-time-compiles) using `ELogLevel.kDebug+10`.
+
+\anchor rdf-from-spec
+### Creating an RDataFrame from a dataset specification file
+
+RDataFrame can be created using a dataset specification JSON file: 
+
+~~~{.python}
+import ROOT
+
+df = ROOT.RDF.Experimental.FromSpec("spec.json")
+~~~
+
+The input dataset specification JSON file needs to be provided by the user and it describes all necessary samples and
+their associated metadata information. The main required key is the "samples" (at least one sample is needed) and the
+required sub-keys for each sample are "trees" and "files". Additionally, one can specify a metadata dictionary for each
+sample in the "metadata" key.
+
+A simple example for the formatting of the specification in the JSON file is the following:
+
+~~~{.cpp}
+{
+   "samples": {
+      "sampleA": {
+         "trees": ["tree1", "tree2"],
+         "files": ["file1.root", "file2.root"],
+         "metadata": {
+            "lumi": 10000.0, 
+            "xsec": 1.0,
+            "sample_category" = "data"
+            }
+      },
+      "sampleB": {
+         "trees": ["tree3", "tree4"],
+         "files": ["file3.root", "file4.root"],
+         "metadata": {
+            "lumi": 0.5, 
+            "xsec": 1.5,
+            "sample_category" = "MC_background"
+            }
+      }
+   }
+}
+~~~
+
+The metadata information from the specification file can be then accessed using the DefinePerSample function.
+For example, to access luminosity information (stored as a double):
+
+~~~{.python}
+df.DefinePerSample("lumi", 'rdfsampleinfo_.GetD("lumi")')
+~~~
+
+or sample_category information (stored as a string):
+
+~~~{.python}
+df.DefinePerSample("sample_category", 'rdfsampleinfo_.GetS("sample_category")')
+~~~
+
+or directly the filename:
+
+~~~{.python}
+df.DefinePerSample("name", "rdfsampleinfo_.GetSampleName()")
+~~~
+
+An example implementation of the "FromSpec" method is available in tutorial: df106_HiggstoFourLeptons.py, which also
+provides a corresponding exemplary JSON file for the dataset specification.
+
+\anchor progressbar
+### Adding a progress bar 
+
+A progress bar showing the processed event statistics can be added to any RDataFrame program.
+The event statistics include elapsed time, currently processed file, currently processed events, the rate of event processing 
+and an estimated remaining time (per file being processed). It is recorded and printed in the terminal every m events and every 
+n seconds (by default m = 1000 and n = 1). The ProgressBar can be also added when the multithread (MT) mode is enabled. 
+
+ProgressBar is added after creating the dataframe object (df):
+~~~{.cpp}
+ROOT::RDataFrame df("tree", "file.root");
+ROOT::RDF::Experimental::AddProgressBar(df);
+~~~
+
+Alternatively, RDataFrame can be cast to an RNode first, giving the user more flexibility 
+For example, it can be called at any computational node, such as Filter or Define, not only the head node,
+with no change to the ProgressBar function itself: 
+~~~{.cpp}
+ROOT::RDataFrame df("tree", "file.root");
+auto df_1 = ROOT::RDF::RNode(df.Filter("x>1"));
+ROOT::RDF::Experimental::AddProgressBar(df_1);
+~~~
+Examples of implemented progress bars can be seen by running [Higgs to Four Lepton tutorial](https://root.cern/doc/master/df106__HiggsToFourLeptons_8py_source.html) and [Dimuon tutorial](https://root.cern/doc/master/df102__NanoAODDimuonAnalysis_8C.html). 
+
 */
 // clang-format on
 
@@ -1403,7 +1583,7 @@ RDataFrame::RDataFrame(std::string_view treeName, std::string_view filenameglob,
 {
    const std::string treeNameInt(treeName);
    const std::string filenameglobInt(filenameglob);
-   auto chain = std::make_shared<TChain>(treeNameInt.c_str(), "", TChain::kWithoutGlobalRegistration);
+   auto chain = ROOT::Internal::TreeUtils::MakeChainForMT(treeNameInt);
    chain->Add(filenameglobInt.c_str());
    GetProxiedPtr()->SetTree(std::move(chain));
 }
@@ -1424,7 +1604,7 @@ RDataFrame::RDataFrame(std::string_view treeName, const std::vector<std::string>
    : RInterface(std::make_shared<RDFDetail::RLoopManager>(nullptr, defaultColumns))
 {
    std::string treeNameInt(treeName);
-   auto chain = std::make_shared<TChain>(treeNameInt.c_str(), "", TChain::kWithoutGlobalRegistration);
+   auto chain = ROOT::Internal::TreeUtils::MakeChainForMT(treeNameInt);
    for (auto &f : fileglobs)
       chain->Add(f.c_str());
    GetProxiedPtr()->SetTree(std::move(chain));
@@ -1476,12 +1656,18 @@ RDataFrame::RDataFrame(std::unique_ptr<ROOT::RDF::RDataSource> ds, const ColumnN
 /// A dataset specification includes trees and file names,
 /// as well as an optional friend list and/or entry range.
 ///
-/// ### Example usage:
+/// ### Example usage from Python:
 /// ~~~{.py}
-/// spec = ROOT.RDF.Experimental.RDatasetSpec("tree", "file.root", (3, 5))
-/// spec.AddFriend([("tree1", "a.root"), ("tree2", "b.root")], "alias")
+/// spec = (
+///     ROOT.RDF.Experimental.RDatasetSpec()
+///     .AddSample(("data", "tree", "file.root"))
+///     .WithGlobalFriends("friendTree", "friend.root", "alias")
+///     .WithGlobalRange((100, 200))
+/// )
 /// df = ROOT.RDataFrame(spec)
 /// ~~~
+///
+/// See also ROOT::RDataFrame::FromSpec().
 RDataFrame::RDataFrame(ROOT::RDF::Experimental::RDatasetSpec spec)
    : RInterface(std::make_shared<RDFDetail::RLoopManager>(std::move(spec)))
 {
@@ -1490,9 +1676,39 @@ RDataFrame::RDataFrame(ROOT::RDF::Experimental::RDatasetSpec spec)
 namespace RDF {
 namespace Experimental {
 
+////////////////////////////////////////////////////////////////////////////
+/// \brief Create the RDataFrame from the dataset specification file.
+/// \param[in] jsonFile Path to the dataset specification JSON file. 
+///
+/// The input dataset specification JSON file must include a number of keys that
+/// describe all the necessary samples and their associated metadata information.
+/// The main key, "samples", is required and at least one sample is needed. Each
+/// sample must have at least one key "trees" and at least one key "files" from
+/// which the data is read. Optionally, one or more metadata information can be
+/// added, as well as the friend list information.
+///
+/// ### Example specification file JSON:
+/// The following is an example of the dataset specification JSON file formatting: 
+///~~~{.cpp}
+/// {
+///    "samples": {
+///       "sampleA": {
+///          "trees": ["tree1", "tree2"],
+///          "files": ["file1.root", "file2.root"],
+///          "metadata": {"lumi": 1.0, }
+///       },
+///       "sampleB": {
+///          "trees": ["tree3", "tree4"],
+///          "files": ["file3.root", "file4.root"],
+///          "metadata": {"lumi": 0.5, }
+///       },
+///       ...
+///     },
+/// }
+///~~~
 ROOT::RDataFrame FromSpec(const std::string &jsonFile)
 {
-   const nlohmann::json fullData = nlohmann::json::parse(std::ifstream(jsonFile));
+   const nlohmann::ordered_json fullData = nlohmann::ordered_json::parse(std::ifstream(jsonFile));
    if (!fullData.contains("samples") || fullData["samples"].size() == 0) {
       throw std::runtime_error(
          R"(The input specification does not contain any samples. Please provide the samples in the specification like:
@@ -1509,7 +1725,7 @@ ROOT::RDataFrame FromSpec(const std::string &jsonFile)
          "metadata": {"lumi": 0.5, }
       },
       ...
-    },
+   },
 })");
    }
 
