@@ -1,4 +1,5 @@
 #include "ntuple_test.hxx"
+#include <TRandom3.h>
 
 namespace {
 /// An RPageSink that keeps counters of (vector) commit of (sealed) pages; used to test RPageSinkBuf
@@ -40,7 +41,7 @@ protected:
    RPage ReservePage(ColumnHandle_t columnHandle, std::size_t nElements) final
    {
       auto elementSize = columnHandle.fColumn->GetElement()->GetSize();
-      return fPageAllocator.NewPage(columnHandle.fId, elementSize, nElements);
+      return fPageAllocator.NewPage(columnHandle.fPhysicalId, elementSize, nElements);
    }
    void ReleasePage(RPage &page) final { fPageAllocator.DeletePage(page); }
 
@@ -235,18 +236,21 @@ TEST(RNTuple, PageFillingString) {
       *fldX = "";
       ntuple->Fill();
       ntuple->CommitCluster();
-      // 2 pages: 16 and 8 characters
-      *fldX = "012345678901234567890123";
+      // 2 pages: 16 and 10 characters
+      *fldX = "01234567890123456789012";
       ntuple->Fill();
+      *fldX = "012";
+      ntuple->Fill(); // main write page is half full here; RColumn::AppendV should flush the shadow page
    }
 
    auto ntuple = RNTupleReader::Open("ntpl", fileGuard.GetPath());
    auto viewX = ntuple->GetView<std::string>("x");
-   ASSERT_EQ(4u, ntuple->GetNEntries());
-   EXPECT_EQ("01234567890123456",        viewX(0));
-   EXPECT_EQ("0123456789012345",         viewX(1));
-   EXPECT_EQ("",                         viewX(2));
-   EXPECT_EQ("012345678901234567890123", viewX(3));
+   ASSERT_EQ(5u, ntuple->GetNEntries());
+   EXPECT_EQ("01234567890123456",       viewX(0));
+   EXPECT_EQ("0123456789012345",        viewX(1));
+   EXPECT_EQ("",                        viewX(2));
+   EXPECT_EQ("01234567890123456789012", viewX(3));
+   EXPECT_EQ("012",                     viewX(4));
 
    const auto desc = ntuple->GetDescriptor();
    EXPECT_EQ(4u, desc->GetNClusters());
@@ -265,7 +269,7 @@ TEST(RNTuple, PageFillingString) {
    const auto &pr4 = cd4.GetPageRange(1);
    ASSERT_EQ(2u, pr4.fPageInfos.size());
    EXPECT_EQ(16u, pr4.fPageInfos[0].fNElements);
-   EXPECT_EQ(8u, pr4.fPageInfos[1].fNElements);
+   EXPECT_EQ(10u, pr4.fPageInfos[1].fNElements);
 }
 
 TEST(RPageSinkBuf, Basics)
@@ -423,7 +427,7 @@ TEST(RPageSinkBuf, ParallelZip) {
 TEST(RPageSinkBuf, CommitSealedPageV)
 {
    RNTupleWriteOptions options;
-   options.SetApproxUnzippedPageSize(8);
+   options.SetApproxUnzippedPageSize(16);
 
 #ifdef R__USE_IMT
    ROOT::DisableImplicitMT();
@@ -433,15 +437,16 @@ TEST(RPageSinkBuf, CommitSealedPageV)
       auto &counters = static_cast<RPageSinkMock *>(sink.get())->fCounters;
 
       auto model = RNTupleModel::Create();
-      auto u32Field = model->MakeField<std::uint32_t>("u32");
-      auto u16Field = model->MakeField<std::uint16_t>("u16");
+      auto u64Field = model->MakeField<std::uint64_t>("u64");
+      auto u32Field = model->MakeField<std::uint16_t>("u32");
+      auto strField = model->MakeField<std::string>("str");
       auto ntuple = std::make_unique<RNTupleWriter>(std::move(model), std::make_unique<RPageSinkBuf>(std::move(sink)));
       ntuple->Fill();
       ntuple->Fill();
       ntuple->Fill();
       ntuple->CommitCluster();
       // Parallel zip not available; all pages committed separately
-      EXPECT_EQ(3, counters.fNCommitPage);
+      EXPECT_EQ(5, counters.fNCommitPage);
       EXPECT_EQ(0, counters.fNCommitSealedPage);
       EXPECT_EQ(0, counters.fNCommitSealedPageV);
    }
@@ -453,8 +458,9 @@ TEST(RPageSinkBuf, CommitSealedPageV)
       auto &counters = static_cast<RPageSinkMock *>(sink.get())->fCounters;
 
       auto model = RNTupleModel::Create();
+      auto u64Field = model->MakeField<std::uint64_t>("u64");
       auto u32Field = model->MakeField<std::uint32_t>("u32");
-      auto u16Field = model->MakeField<std::uint16_t>("u16");
+      auto strField = model->MakeField<std::string>("str");
       auto ntuple = std::make_unique<RNTupleWriter>(std::move(model), std::make_unique<RPageSinkBuf>(std::move(sink)));
       ntuple->Fill();
       ntuple->Fill();
@@ -464,7 +470,7 @@ TEST(RPageSinkBuf, CommitSealedPageV)
       EXPECT_EQ(0, counters.fNCommitPage);
       EXPECT_EQ(1, counters.fNCommitSealedPageV);
 #else
-      EXPECT_EQ(2, counters.fNCommitPage);
+      EXPECT_EQ(3, counters.fNCommitPage);
       EXPECT_EQ(0, counters.fNCommitSealedPageV);
 #endif
       EXPECT_EQ(0, counters.fNCommitSealedPage);
